@@ -25,15 +25,14 @@ NS_ASSUME_NONNULL_BEGIN
     return value;
 }
 
+// LCOV_EXCL_START
+/* PythonTransient is used in the implementation of
+ * methods written in Python, OC_Python* classes
+ * don't have such methods.
+ */
 - (PyObject*)__pyobjc_PythonTransient__:(int*)cookie
 {
     *cookie = 0;
-    /*
-     * XXX: Check if 'value' can ever be NULL
-     *      if not: replace by PyObjC_Assert check
-     *
-     * XXX: return [self __pyobjc_PythonObject__];
-     */
     if (likely(value)) {
         Py_INCREF(value);
         return value;
@@ -41,6 +40,7 @@ NS_ASSUME_NONNULL_BEGIN
         Py_RETURN_NONE;
     }
 }
+// LCOV_EXCL_STOP
 
 + (BOOL)supportsSecureCoding
 {
@@ -58,8 +58,16 @@ NS_ASSUME_NONNULL_BEGIN
 
     PyObjC_BEGIN_WITH_GIL
 
+        /* XXX: This is different from other proxies
+         *      to make it easier to hit a race in
+         *      proxy-registry.m for testing.
+         *
+         *      With the current implementation of
+         *      proxy-registry (using weak refs) either
+         *      order works correctly.
+         */
         PyObjC_UnregisterObjCProxy(value, self);
-        Py_CLEAR(value);
+        Py_XDECREF(value);
 
     PyObjC_END_WITH_GIL
 
@@ -278,7 +286,7 @@ NS_ASSUME_NONNULL_BEGIN
         Py_ssize_t size = PyTuple_Size(value);
 
         if ([coder allowsKeyedCoding]) {
-            if (size > INT_MAX) { // LCOV_BT_EXCL_LINE
+            if (size > INT_MAX) { // LCOV_BR_EXCL_LINE
                 /* Excluded from coverage tests because this would require creating
                  * rather huge tuples, with corresponding memory usage.
                  */
@@ -346,13 +354,15 @@ NS_ASSUME_NONNULL_BEGIN
      */
     NSUInteger i;
 
-    if (count > 0 && objects == NULL) {
+    if (count > 0 && objects == NULL) { // LCOV_BR_EXCL_LINE
+        // LCOV_EXCL_START
         [self release];
         return nil;
+        // LCOV_EXCL_STOP
     }
 
     PyObjC_BEGIN_WITH_GIL
-        if (PyTuple_CheckExact(value) && (NSUInteger)PyTuple_Size(value) == count) {
+        if (PyTuple_CheckExact(value) && (NSUInteger)PyTuple_Size(value) == count) { // LCOV_BR_EXCL_LINE
             for (i = 0; i < count; i++) {
                 PyObject* v;
 
@@ -368,12 +378,14 @@ NS_ASSUME_NONNULL_BEGIN
                 }
 
                 /* XXX: Can this every be true? */
-                if (PyTuple_GET_ITEM(value, i) != NULL) {
+                if (PyTuple_GET_ITEM(value, i) != NULL) { // LCOV_BR_EXCL_LINE
+                    // LCOV_EXCL_START
                     /* use temporary object to avoid race condition */
                     PyObject* t = PyTuple_GET_ITEM(value, i);
                     PyTuple_SET_ITEM(value, i, NULL);
                     Py_DECREF(t);
-                }
+                    // LCOV_EXCL_STOP
+                } // LCOV_EXCL_LINE
                 PyTuple_SET_ITEM(value, i, v);
             }
         } else {
@@ -449,7 +461,7 @@ NS_ASSUME_NONNULL_BEGIN
 
     }
 
-    switch (code) {
+    switch (code) { // LCOV_BR_EXCL_LINE
     case 1:
         /* This code was used by some previous versions of PyObjC
          * (before 2.2) and is kept around for backward compatibility.
@@ -470,7 +482,7 @@ NS_ASSUME_NONNULL_BEGIN
         if (tmpVal == nil) {
             return nil;
         }
-        PyObjC_Assert(tmpVal == self, nil);
+        assert(tmpVal == self);
         self = tmpVal;
 
         PyObjC_BEGIN_WITH_GIL
@@ -494,7 +506,7 @@ NS_ASSUME_NONNULL_BEGIN
         PyObjC_END_WITH_GIL
 
         tmpVal = [super initWithCoder:coder];
-        PyObjC_Assert(tmpVal == self, nil);
+        assert(tmpVal == self);
         return tmpVal;
 
     case 3:
@@ -507,17 +519,13 @@ NS_ASSUME_NONNULL_BEGIN
             PyObject* decoded = PyObjC_decodeWithCoder(coder, self);
             if (decoded == NULL) {
                 PyObjC_GIL_FORWARD_EXC();
-            }
+            } // LCOV_EXCL_LINE
 
             SET_FIELD(value, decoded);
 
             id actual = PyObjC_RegisterObjCProxy(value, self);
-            if (actual != self) {
-                [self release];
-                self = actual;
-            } else if (actual != nil) {
-                [actual release];
-            }
+            [self release];
+            self = actual;
 
         PyObjC_END_WITH_GIL
 
@@ -552,12 +560,12 @@ NS_ASSUME_NONNULL_BEGIN
             value = PyTuple_New(size);
             if (value == NULL) {          // LCOV_BR_EXCL_LINE
                 PyObjC_GIL_FORWARD_EXC(); // LCOV_EXCL_LINE
-            }
+            } // LCOV_EXCL_LINE
 
         PyObjC_END_WITH_GIL
 
         tmpVal = [super initWithCoder:coder];
-        PyObjC_Assert(tmpVal == self, nil);
+        assert(tmpVal == self);
         return tmpVal;
 
     case 5:
@@ -595,9 +603,9 @@ NS_ASSUME_NONNULL_BEGIN
 
         PyObjC_END_WITH_GIL
         tmpVal = [super initWithCoder:coder];
-        PyObjC_Assert(tmpVal == self, nil);
-        // LCOV_EXCL_STOP
+        assert(tmpVal == self);
         return tmpVal;
+        // LCOV_EXCL_STOP
 
     default:
         // LCOV_EXCL_START
@@ -624,12 +632,12 @@ NS_ASSUME_NONNULL_BEGIN
         PyObject* copy = PyObjC_Copy(value);
         if (copy == NULL) {
             PyObjC_GIL_FORWARD_EXC();
-        }
+        } // LCOV_EXCL_LINE
 
         if (depythonify_python_object(copy, &result) == -1) {
             Py_DECREF(copy);
             PyObjC_GIL_FORWARD_EXC();
-        }
+        } // LCOV_EXCL_LINE
 
         Py_DECREF(copy);
 
@@ -647,12 +655,15 @@ NS_ASSUME_NONNULL_BEGIN
         PyObject* copy = PySequence_List(value);
         if (copy == NULL) {
             PyObjC_GIL_FORWARD_EXC();
-        }
+        } // LCOV_EXCL_LINE
 
-        if (depythonify_python_object(copy, &result) == -1) {
+        if (depythonify_python_object(copy, &result) == -1) { // LCOV_BR_EXCL_LINE
+            /* Should never fail, 'copy' is an instance of PyList_Type */
+            // LCOV_EXCL_START
             Py_DECREF(copy);
             PyObjC_GIL_FORWARD_EXC();
-        }
+            // LCOV_EXCL_STOP
+        } // LCOV_EXCL_LINE
 
         Py_DECREF(copy);
 
