@@ -37,20 +37,12 @@ NS_ASSUME_NONNULL_BEGIN
 - (id _Nullable)initWithPyObject:(PyObject*)obj
 {
     // XXX: Fix callers to do the registration for us.
-    id actual = PyObjC_RegisterObjCProxy(obj, self);
-    if (actual != self) {
-        /*
-         * Race between two threads creating a proxy, use the
-         * first one that got registered.
-         */
-        [self release];
-        return actual;
-    } else if (actual != nil) {
-        [actual release];
-    }
-
     SET_FIELD_INCREF(pyObject, obj);
 
+    id actual = PyObjC_RegisterObjCProxy(obj, self);
+    [self release];
+    self = actual;
+
     return self;
 }
 
@@ -82,12 +74,12 @@ NS_ASSUME_NONNULL_BEGIN
         copy = PyObjC_Copy(pyObject);
         if (copy == NULL) {
             PyObjC_GIL_FORWARD_EXC();
-        }
+        } // LCOV_EXCL_LINE
 
         if (depythonify_python_object(copy, &result) == -1) {
             Py_DECREF(copy);
             PyObjC_GIL_FORWARD_EXC();
-        }
+        } // LCOV_EXCL_LINE
         Py_DECREF(copy);
 
     PyObjC_END_WITH_GIL
@@ -128,14 +120,14 @@ NS_ASSUME_NONNULL_BEGIN
             Py_DECREF(repr);
             if (err == -1) {
                 PyObjC_GIL_FORWARD_EXC();
-            }
+            } // LCOV_EXCL_LINE
 
             PyObjC_GIL_RETURN(result);
         }
         PyObjC_GIL_FORWARD_EXC();
 
     PyObjC_END_WITH_GIL
-}
+} // LCOV_EXCL_LINE
 
 - (void)doesNotRecognizeSelector:(SEL)aSelector
 {
@@ -153,8 +145,8 @@ check_argcount(PyObject* pymethod, Py_ssize_t argcount)
 
     if (PyObjC_is_pyfunction(pymethod)) {
         func_code = (PyCodeObject*)PyObjC_get_code(pymethod);
-        if (func_code == NULL) {
-            return -1;
+        if (func_code == NULL) { // LCOV_BR_EXCL_LINE
+            return -1; // LCOV_EXCL_LINE
         }
         if (argcount == func_code->co_argcount) {
             Py_DECREF(func_code);
@@ -164,8 +156,8 @@ check_argcount(PyObject* pymethod, Py_ssize_t argcount)
 
     } else if (PyObjC_is_pymethod(pymethod)) {
         func_code = PyObjC_get_code(pymethod);
-        if (func_code == NULL) {
-            return -1;
+        if (func_code == NULL) { // LCOV_BR_EXCL_LINE
+            return -1; // LCOV_EXCL_LINE
         }
         if (argcount == func_code->co_argcount - 1) {
             Py_DECREF(func_code);
@@ -205,8 +197,9 @@ static PyObject* _Nullable get_method_for_selector(PyObject* obj, SEL aSelector)
 
     PyObject* py_meth_name =
         PyObjC_SELToPythonName(aSelector);
-    if (py_meth_name == NULL) {
-        return NULL;
+    if (py_meth_name == NULL) { // LCOV_BR_EXCL_LINE
+        /* Can only fail due to memory errors */
+        return NULL; // LCOV_EXCL_LINE
     }
     pymethod = PyObject_GetAttr(obj, py_meth_name);
     Py_CLEAR(py_meth_name);
@@ -241,7 +234,7 @@ static PyObject* _Nullable get_method_for_selector(PyObject* obj, SEL aSelector)
         if (m) {
             Py_DECREF(m);
             PyObjC_GIL_RETURN(YES);
-        } else {
+        } else { // LCOV_EXCL_LINE
             PyErr_Clear();
             PyObjC_GIL_RETURN(NO);
         }
@@ -282,7 +275,7 @@ static PyObject* _Nullable get_method_for_selector(PyObject* obj, SEL aSelector)
     if (m != NULL) {
         /* A real Objective-C method */
         const char* typestr = method_getTypeEncoding(m);
-        if (typestr == NULL) { // LCOv_BR_EXCL_LINE
+        if (typestr == NULL) { // LCOV_BR_EXCL_LINE
             return nil;        // LCOV_EXCL_LINE
         }
         return [NSMethodSignature signatureWithObjCTypes:typestr];
@@ -294,7 +287,7 @@ static PyObject* _Nullable get_method_for_selector(PyObject* obj, SEL aSelector)
         if (!pymethod) {
             PyErr_Clear();
             PyObjC_GIL_RETURN(nil);
-        }
+        } // LCOV_EXCL_LINE
 
         if (PyObjC_is_pymethod(pymethod)) {
             argcount = PyObjC_num_arguments(pymethod) - 1;
@@ -303,9 +296,13 @@ static PyObject* _Nullable get_method_for_selector(PyObject* obj, SEL aSelector)
             argcount = PyObjC_num_arguments(pymethod);
         }
         Py_DECREF(pymethod);
-        if (argcount < 0) {
-            PyObjC_GIL_FORWARD_EXC();
-        }
+        if (argcount < 0) { // LCOV_BR_EXCL_LINE
+            /* Cannot fail because get_method_for_selector
+             * will only return a method or function for which
+             * the number of arguments can be calculated.
+             */
+            PyObjC_GIL_FORWARD_EXC(); // LCOV_EXCL_LINE
+        } // LCOV_EXCL_LINE
 
         encoding = alloca(argcount + 4);
         memset(encoding, '@', argcount + 3);
@@ -492,9 +489,12 @@ static PyObject* _Nullable get_method_for_selector(PyObject* obj, SEL aSelector)
     PyObjC_BEGIN_WITH_GIL
 
         retsize = PyObjCRT_SizeOfType(rettype);
-        if (retsize == -1) {
-            PyObjC_GIL_FORWARD_EXC();
-        }
+        if (retsize == -1) { // LCOV_BR_EXCL_LINE
+            /* Cannot happen unless the ObjC runtime contains
+             * invalid data.
+             */
+            PyObjC_GIL_FORWARD_EXC(); // LCOV_EXCL_LINE
+        } // LCOV_EXCL_LINE
 
         retbuffer = alloca(retsize);
 
@@ -527,7 +527,7 @@ static PyObject* _Nullable get_method_for_selector(PyObject* obj, SEL aSelector)
             Py_DECREF(pymethod);
             PyObjC_GIL_FORWARD_EXC();
             // LCOV_EXCL_STOP
-        }
+        } // LCOV_EXCL_LINE
         for (i = 2; i < argcount; i++) {
             const char* argtype;
             char*       argbuffer;
@@ -544,11 +544,16 @@ static PyObject* _Nullable get_method_for_selector(PyObject* obj, SEL aSelector)
             // LCOV_EXCL_STOP
 
             argsize = PyObjCRT_SizeOfType(argtype);
-            if (argsize == -1) {
+            if (argsize == -1) { // LCOV_BR_EXCL_LINE
+                /* Cannot happen unless the ObjC runtime
+                 * contains invalid data.
+                 */
+                // LCOV_EXCL_START
                 Py_DECREF(args);
                 Py_DECREF(pymethod);
                 PyObjC_GIL_FORWARD_EXC();
-            }
+                // LCOV_EXCL_STOP
+            } // LCOV_EXCL_LINE
             argbuffer = alloca(argsize);
 
             Py_BEGIN_ALLOW_THREADS
@@ -568,7 +573,7 @@ static PyObject* _Nullable get_method_for_selector(PyObject* obj, SEL aSelector)
                 Py_DECREF(args);
                 Py_DECREF(pymethod);
                 PyObjC_GIL_FORWARD_EXC();
-            }
+            } // LCOV_EXCL_LINE
 
             PyTuple_SET_ITEM(args, i - 2, pyarg);
         }
@@ -580,14 +585,13 @@ static PyObject* _Nullable get_method_for_selector(PyObject* obj, SEL aSelector)
 
         if (result == NULL) {
             PyObjC_GIL_FORWARD_EXC();
-            return;
-        }
+        } // LCOV_EXCL_LINE
 
         err = depythonify_c_value(rettype, result, retbuffer);
         Py_DECREF(result);
         if (err == -1) {
             PyObjC_GIL_FORWARD_EXC();
-        } else {
+        } else { // LCOV_EXCL_LINE
             Py_BEGIN_ALLOW_THREADS
                 @try {
                     [invocation setReturnValue:retbuffer];
@@ -619,6 +623,12 @@ static PyObject* _Nullable get_method_for_selector(PyObject* obj, SEL aSelector)
         }
     PyObjC_END_WITH_GIL
 }
+
+// LCOV_EXCL_START
+/* PythonTransient is used in the implementation of
+ * methods written in Python, OC_Python* classes
+ * don't have such methods.
+ */
 - (PyObject*)__pyobjc_PythonTransient__:(int*)cookie
 {
     PyObjC_BEGIN_WITH_GIL
@@ -639,6 +649,7 @@ static PyObject* _Nullable get_method_for_selector(PyObject* obj, SEL aSelector)
 
     return rval;
 }
+// LCOV_EXCL_STOP
 
 /*
  * Implementation for Key-Value Coding.
@@ -665,7 +676,7 @@ static PyObject* _Nullable get_method_for_selector(PyObject* obj, SEL aSelector)
     PyObjC_BEGIN_WITH_GIL
         if (PyObjC_GetKey(pyObject, key, &res) == -1) {
             PyObjC_GIL_FORWARD_EXC();
-        }
+        } // LCOV_EXCL_LINE
 
     PyObjC_END_WITH_GIL
 
@@ -687,7 +698,7 @@ static PyObject* _Nullable get_method_for_selector(PyObject* obj, SEL aSelector)
     PyObjC_BEGIN_WITH_GIL
         if (PyObjC_SetKey(pyObject, key, value) == -1) {
             PyObjC_GIL_FORWARD_EXC();
-        }
+        } // LCOV_EXCL_LINE
 
     PyObjC_END_WITH_GIL
 }
@@ -721,7 +732,7 @@ static PyObject* _Nullable get_method_for_selector(PyObject* obj, SEL aSelector)
     PyObjC_BEGIN_WITH_GIL
         if (PyObjC_GetKeyPath(pyObject, keyPath, &res) == -1) {
             PyObjC_GIL_FORWARD_EXC();
-        }
+        } // LCOV_EXCL_LINE
 
     PyObjC_END_WITH_GIL
 
@@ -738,7 +749,7 @@ static PyObject* _Nullable get_method_for_selector(PyObject* obj, SEL aSelector)
     PyObjC_BEGIN_WITH_GIL
         if (PyObjC_SetKeyPath(pyObject, keyPath, value) == -1) {
             PyObjC_GIL_FORWARD_EXC();
-        }
+        } // LCOV_EXCL_LINE
 
     PyObjC_END_WITH_GIL
 }
@@ -760,12 +771,11 @@ static PyObject* _Nullable get_method_for_selector(PyObject* obj, SEL aSelector)
     }
 }
 
+/* Deprecated in macOS 10.3, not called in practice */
+// LCOV_EXCL_START
 - (void)unableToSetNilForKey:(NSString*)key
 {
-    @throw [NSException
-        exceptionWithName:NSUndefinedKeyException
-                   reason:[NSString stringWithFormat:@"cannot set Nil for key: %@", key]
-                 userInfo:nil];
+    [self setNilValueForKey:key];
 }
 
 - (id)handleQueryWithUnboundKey:(NSString*)key
@@ -773,6 +783,27 @@ static PyObject* _Nullable get_method_for_selector(PyObject* obj, SEL aSelector)
     return [self valueForUndefinedKey:key];
 }
 
+- (void)handleTakeValue:value forUnboundKey:(NSString*)key
+{
+    [self setValue:value forUndefinedKey:key];
+}
+// LCOV_EXCL_STOP
+
+
+
+// LCOV_EXCL_START
+/* These are defined for NSObject, but are only invoked by
+ * the setValue:forKey: implementation, and ours never does
+ * that.
+ */
+- (void)setNilValueForKey:(NSString*)key
+{
+    @throw [NSException
+        exceptionWithName:NSUndefinedKeyException
+                   reason:[NSString stringWithFormat:@"cannot set Nil for key: %@", key]
+                 userInfo:nil];
+}
+
 - (id)valueForUndefinedKey:(NSString*)key
 {
     @throw [NSException
@@ -781,11 +812,6 @@ static PyObject* _Nullable get_method_for_selector(PyObject* obj, SEL aSelector)
                  userInfo:nil];
 }
 
-- (void)handleTakeValue:value forUnboundKey:(NSString*)key
-{
-    [self setValue:value forUndefinedKey:key];
-}
-
 - (void)setValue:value forUndefinedKey:(NSString*)key
 {
     @throw [NSException
@@ -794,6 +820,7 @@ static PyObject* _Nullable get_method_for_selector(PyObject* obj, SEL aSelector)
                                                      key, value]
                  userInfo:nil];
 }
+// LCOV_EXCL_STOP
 
 - (void)addObserver:(NSObject*)observer
          forKeyPath:(NSString*)keyPath
@@ -841,19 +868,21 @@ static PyObject* _Nullable get_method_for_selector(PyObject* obj, SEL aSelector)
         if (otherPyObject == NULL) {
             PyErr_Clear();
             PyObjC_GIL_RETURN(NO);
-        }
-        if (otherPyObject == pyObject) {
-            PyObjC_GIL_RETURN(YES);
-        }
+        } // LCOV_EXCL_LINE
+        if (otherPyObject == pyObject) { // LCOV_BR_EXCL_LINE
+            /* Should never happen because of the test if
+             * self is other earlier in the method.
+             */
+            PyObjC_GIL_RETURN(YES); // LCOV_EXCL_LINE
+        } // LCOV_EXCL_LINE
         switch (PyObject_RichCompareBool(pyObject, otherPyObject, Py_EQ)) {
         case -1:
             PyErr_Clear();
         case 0:
             PyObjC_GIL_RETURN(NO);
-            break;
         default:
             PyObjC_GIL_RETURN(YES);
-        }
+        } // LCOV_EXCL_LINE
     PyObjC_END_WITH_GIL
 }
 
@@ -871,14 +900,17 @@ static PyObject* _Nullable get_method_for_selector(PyObject* obj, SEL aSelector)
         PyObject* otherPyObject = id_to_python(other);
         if (otherPyObject == NULL) {
             PyObjC_GIL_FORWARD_EXC();
-        }
-        if (otherPyObject == pyObject) {
-            PyObjC_GIL_RETURN(NSOrderedSame);
-        }
+        } // LCOV_EXCL_LINE
+        if (otherPyObject == pyObject) { // LCOV_BR_EXCL_LINE
+            /* Should never happen because of checking if self is other
+             * earlier in this method.
+             */
+            PyObjC_GIL_RETURN(NSOrderedSame); // LCOV_EXCL_LINE
+        } // LCOV_EXCL_LINE
         int r;
         if (PyObjC_Cmp(pyObject, otherPyObject, &r) == -1) {
             PyObjC_GIL_FORWARD_EXC();
-        }
+        } // LCOV_EXCL_LINE
         NSComparisonResult rval;
         switch (r) {
         case -1:
@@ -905,7 +937,7 @@ static PyObject* _Nullable get_method_for_selector(PyObject* obj, SEL aSelector)
         rval = PyObjC_encodeWithCoder(pyObject, coder);
         if (rval == -1) {
             PyObjC_GIL_FORWARD_EXC();
-        }
+        } // LCOV_EXCL_LINE
     PyObjC_END_WITH_GIL
 }
 
@@ -930,7 +962,7 @@ static PyObject* _Nullable get_method_for_selector(PyObject* obj, SEL aSelector)
         PyObject* decoded = PyObjC_decodeWithCoder(coder, self);
         if (decoded == NULL) {
             PyObjC_GIL_FORWARD_EXC();
-        }
+        } // LCOV_EXCL_LINE
 
         /* To make life more interesting the correct proxy
          * type for 'v' might not be OC_PythonObject, in particular
@@ -938,10 +970,16 @@ static PyObject* _Nullable get_method_for_selector(PyObject* obj, SEL aSelector)
          * of PyObjC, and in some error cases.
          */
         NSObject* temp;
-        if (depythonify_python_object(decoded, &temp) == -1) {
+        if (depythonify_python_object(decoded, &temp) == -1) { // LCOV_BR_EXCL_LINE
+            /* Cannot get here with testing, the decoder invokes
+             * 'setValue' above and that call has already depythonified
+             * the value.
+             */
+            // LCOV_EXCL_START
             Py_DECREF(decoded);
             PyObjC_GIL_FORWARD_EXC();
-        }
+            // LCOV_EXCL_STOP
+        } // LCOV_EXCL_LINE
 
         if (temp != (NSObject*)self) {
             [temp retain];
@@ -1071,12 +1109,12 @@ static PyObject* _Nullable get_method_for_selector(PyObject* obj, SEL aSelector)
  * Fake implementation for _cfTypeID, which gets called by
  * system frameworks on some occasions.
  */
-static BOOL     haveTypeID = NO;
-static CFTypeID _NSObjectTypeID;
+static PyObjC_ATOMIC BOOL     haveTypeID = NO;
+static PyObjC_ATOMIC CFTypeID _NSObjectTypeID;
 
 - (CFTypeID)_cfTypeID
 {
-    if (haveTypeID) {
+    if (!haveTypeID) {
         NSObject* obj   = [[NSObject alloc] init];
         _NSObjectTypeID = CFGetTypeID((CFTypeRef)obj);
         [obj release];
