@@ -25,17 +25,23 @@ NS_ASSUME_NONNULL_BEGIN
         PyErr_SetString(PyObjCExc_Error, "OC_PythonSet without a value");
         return NULL;
     }
-    PyObjC_Assert(value != NULL, ((PyObject* _Nonnull)NULL));
+    assert(value != NULL);
     Py_INCREF(value);
     return value;
 }
 
+// LCOV_EXCL_START
+/* PythonTransient is used in the implementation of
+ * methods written in Python, OC_Python* classes
+ * don't have such methods.
+ */
 - (PyObject*)__pyobjc_PythonTransient__:(int*)cookie
 {
     *cookie = 0;
     Py_XINCREF(value);
     return value;
 }
+// LCOV_EXCL_STOP
 
 + (BOOL)supportsSecureCoding
 {
@@ -64,10 +70,12 @@ NS_ASSUME_NONNULL_BEGIN
 
 - (Class)classForCoder
 {
-    if (PyFrozenSet_CheckExact(value)) {
-        return [NSSet class];
-    } else if (PyAnySet_CheckExact(value)) {
-        return [NSMutableSet class];
+    if (PyAnySet_CheckExact(value)) {
+        if (PyFrozenSet_CheckExact(value)) {
+            return [NSSet class];
+        } else {
+            return [NSMutableSet class];
+        }
     } else {
         return [OC_PythonSet class];
     }
@@ -106,7 +114,7 @@ NS_ASSUME_NONNULL_BEGIN
         PyObjC_BEGIN_WITH_GIL
             if (PyObjC_encodeWithCoder(value, coder) == -1) {
                 PyObjC_GIL_FORWARD_EXC();
-            }
+            } // LCOV_EXCL_LINE
         PyObjC_END_WITH_GIL
     }
 }
@@ -128,9 +136,11 @@ NS_ASSUME_NONNULL_BEGIN
 {
     NSUInteger i;
 
-    if (cnt > 0 && objects == NULL) {
+    if (cnt > 0 && objects == NULL) { // LCOV_BR_EXCL_LINE
+        // LCOV_EXCL_START
         [self release];
         return nil;
+        // LCOV_EXCL_STOP
     }
     PyObjC_BEGIN_WITH_GIL
         for (i = 0; i < cnt; i++) {
@@ -143,13 +153,13 @@ NS_ASSUME_NONNULL_BEGIN
                 cur = id_to_python(objects[i]);
                 if (cur == NULL) {
                     PyObjC_GIL_FORWARD_EXC();
-                }
+                } // LCOV_EXCL_LINE
             }
 
             if (PySet_Add(value, cur) < 0) {
                 Py_DECREF(cur);
                 PyObjC_GIL_FORWARD_EXC();
-            }
+            } // LCOV_EXCL_LINE
             Py_DECREF(cur);
         }
 
@@ -211,12 +221,8 @@ NS_ASSUME_NONNULL_BEGIN
         SET_FIELD(value, decoded);
 
         id actual = PyObjC_RegisterObjCProxy(value, self);
-        if (actual != self) {
-            [self release];
-            self = actual;
-        } else if (actual != nil) {
-            [actual release];
-        }
+        [self release];
+        self = actual;
     PyObjC_END_WITH_GIL
 
     return self;
@@ -235,12 +241,12 @@ NS_ASSUME_NONNULL_BEGIN
         PyObject* tmp = PyObjC_Copy(value);
         if (tmp == NULL) {
             PyObjC_GIL_FORWARD_EXC();
-        }
+        } // LCOV_EXCL_LINE
 
         if (depythonify_python_object(tmp, &result) == -1) {
             Py_DECREF(tmp);
             PyObjC_GIL_FORWARD_EXC();
-        }
+        } // LCOV_EXCL_LINE
         Py_DECREF(tmp);
 
     PyObjC_END_WITH_GIL
@@ -259,14 +265,14 @@ NS_ASSUME_NONNULL_BEGIN
         PyObject* tmp = PySet_New(value);
         if (tmp == NULL) {            // LCOV_BR_EXCL_LINE
             PyObjC_GIL_FORWARD_EXC(); // LCOV_EXCL_LINE
-        }
+        } // LCOV_EXCL_LINE
 
         if (depythonify_python_object(tmp, &result) == -1) { // LCOV_BR_EXCL_LINE
             // LCOV_EXCL_START
             Py_DECREF(tmp);
             PyObjC_GIL_FORWARD_EXC();
             // LCOV_EXCL_STOP
-        }
+        } // LCOV_EXCL_LINE
         Py_DECREF(tmp);
 
     PyObjC_END_WITH_GIL
@@ -283,7 +289,7 @@ NS_ASSUME_NONNULL_BEGIN
         PyObject* tmp = PySequence_List(value);
         if (tmp == NULL) {            // LCOV_BR_EXCL_LINE
             PyObjC_GIL_FORWARD_EXC(); // LCOV_EXCL_LINE
-        }
+        } // LCOV_EXCL_LINE
 
         if (depythonify_python_object( // LCOV_BR_EXCL_LINE
                 tmp, &result)
@@ -292,7 +298,7 @@ NS_ASSUME_NONNULL_BEGIN
             Py_DECREF(tmp);
             PyObjC_GIL_FORWARD_EXC();
             // LCOV_EXCL_STOP
-        }
+        } // LCOV_EXCL_LINE
 
         Py_DECREF(tmp);
 
@@ -316,18 +322,18 @@ NS_ASSUME_NONNULL_BEGIN
             tmp = PyObject_GetIter(value);
             if (tmp == NULL) {
                 PyObjC_GIL_FORWARD_EXC();
-            }
+            } // LCOV_EXCL_LINE
 
             v = PyIter_Next(tmp);
             Py_DECREF(tmp);
             if (v == NULL) {
                 PyObjC_GIL_FORWARD_EXC();
-            }
+            } // LCOV_EXCL_LINE
 
             if (depythonify_python_object(v, &result) == -1) {
                 Py_DECREF(v);
                 PyObjC_GIL_FORWARD_EXC();
-            }
+            } // LCOV_EXCL_LINE
 
             Py_DECREF(v);
         }
@@ -350,14 +356,14 @@ NS_ASSUME_NONNULL_BEGIN
             tmp = id_to_python(anObject);
             if (tmp == NULL) {
                 PyObjC_GIL_FORWARD_EXC();
-            }
+            } // LCOV_EXCL_LINE
         }
 
         r = PySequence_Contains(value, tmp);
         Py_DECREF(tmp);
         if (r == -1) {
             PyObjC_GIL_FORWARD_EXC();
-        }
+        } // LCOV_EXCL_LINE
 
     PyObjC_END_WITH_GIL
 
@@ -372,7 +378,7 @@ NS_ASSUME_NONNULL_BEGIN
         result = PySequence_Size(value);
         if (result == -1) {
             PyObjC_GIL_FORWARD_EXC();
-        }
+        } // LCOV_EXCL_LINE
 
     PyObjC_END_WITH_GIL
 
@@ -386,7 +392,7 @@ NS_ASSUME_NONNULL_BEGIN
         PyObject* tmp = PyObject_GetIter(value);
         if (tmp == NULL) {
             PyObjC_GIL_FORWARD_EXC();
-        }
+        } // LCOV_EXCL_LINE
 
         result = [OC_PythonEnumerator enumeratorWithPythonObject:tmp];
         Py_DECREF(tmp);
@@ -415,14 +421,14 @@ NS_ASSUME_NONNULL_BEGIN
             tmpMember = id_to_python(anObject);
             if (tmpMember == NULL) {
                 PyObjC_GIL_FORWARD_EXC();
-            }
+            } // LCOV_EXCL_LINE
         }
 
         r = PySequence_Contains(value, tmpMember);
         if (r == -1) {
             Py_DECREF(tmpMember);
             PyObjC_GIL_FORWARD_EXC();
-        }
+        } // LCOV_EXCL_LINE
 
         if (!r) {
             Py_DECREF(tmpMember);
@@ -437,15 +443,16 @@ NS_ASSUME_NONNULL_BEGIN
 
             if (tmp == NULL) {
                 PyObjC_GIL_FORWARD_EXC();
-            }
+            } // LCOV_EXCL_LINE
 
             while ((v = PyIter_Next(tmp)) != NULL) {
                 r = PyObject_RichCompareBool(v, tmpMember, Py_EQ);
                 if (r == -1) {
+                    Py_DECREF(v);
                     Py_DECREF(tmp);
                     Py_DECREF(tmpMember);
                     PyObjC_GIL_FORWARD_EXC();
-                }
+                } // LCOV_EXCL_LINE
 
                 if (r) {
                     /* Found the object */
@@ -456,14 +463,17 @@ NS_ASSUME_NONNULL_BEGIN
                             Py_DECREF(tmp);
                             Py_DECREF(tmpMember);
                             PyObjC_GIL_FORWARD_EXC();
-                        }
+                        } // LCOV_EXCL_LINE
                     }
+                    Py_DECREF(v);
                     break;
                 }
             }
-
             Py_DECREF(tmp);
             Py_DECREF(tmpMember);
+            if (PyErr_Occurred()) {
+                PyObjC_GIL_FORWARD_EXC();
+            } // LCOV_EXCL_LINE
         }
 
     PyObjC_END_WITH_GIL
@@ -480,14 +490,14 @@ NS_ASSUME_NONNULL_BEGIN
         if (PyFrozenSet_CheckExact(value)) {
             PyErr_SetString(PyExc_TypeError, "Cannot mutate a frozenset");
             PyObjC_GIL_FORWARD_EXC();
-        }
+        } // LCOV_EXCL_LINE
 
         PyObject* args[3] = {NULL, value};
         PyObject* r       = PyObject_VectorcallMethod(PyObjCNM_clear, args + 1,
                                                       1 | PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
         if (r == NULL) {
             PyObjC_GIL_FORWARD_EXC();
-        }
+        } // LCOV_EXCL_LINE
         Py_DECREF(r);
 
     PyObjC_END_WITH_GIL
@@ -499,12 +509,12 @@ NS_ASSUME_NONNULL_BEGIN
         PyObject* tmp = id_to_python(anObject);
         if (tmp == NULL) {
             PyObjC_GIL_FORWARD_EXC();
-        }
+        } // LCOV_EXCL_LINE
 
         if (PyFrozenSet_CheckExact(value)) {
             PyErr_SetString(PyExc_TypeError, "Cannot mutate a frozenset");
             PyObjC_GIL_FORWARD_EXC();
-        }
+        } // LCOV_EXCL_LINE
 
         PyObject* args[3] = {NULL, value, tmp};
         PyObject* r       = PyObject_VectorcallMethod(PyObjCNM_discard, args + 1,
@@ -512,7 +522,7 @@ NS_ASSUME_NONNULL_BEGIN
         Py_DECREF(tmp);
         if (r == NULL) {
             PyObjC_GIL_FORWARD_EXC();
-        }
+        } // LCOV_EXCL_LINE
         Py_DECREF(r);
 
     PyObjC_END_WITH_GIL
@@ -524,12 +534,12 @@ NS_ASSUME_NONNULL_BEGIN
         PyObject* tmp = id_to_python(anObject);
         if (tmp == NULL) {
             PyObjC_GIL_FORWARD_EXC();
-        }
+        } // LCOV_EXCL_LINE
 
         if (PyFrozenSet_CheckExact(value)) {
             PyErr_SetString(PyExc_TypeError, "Cannot mutate a frozenset");
             PyObjC_GIL_FORWARD_EXC();
-        }
+        } // LCOV_EXCL_LINE
 
         PyObject* args[3] = {NULL, value, tmp};
         PyObject* r       = PyObject_VectorcallMethod(PyObjCNM_add, args + 1,
@@ -537,7 +547,7 @@ NS_ASSUME_NONNULL_BEGIN
         Py_DECREF(tmp);
         if (r == NULL) {
             PyObjC_GIL_FORWARD_EXC();
-        }
+        } // LCOV_EXCL_LINE
         Py_DECREF(r);
 
     PyObjC_END_WITH_GIL
