@@ -64,7 +64,7 @@ const char* _Nullable PyObjCBlock_GetSignature(void* _block)
 {
     struct block_literal* block = (struct block_literal*)_block;
 
-    if (((intptr_t)(block->isa)) & 0x1) {
+    if (((intptr_t)(block->isa)) & 0x1) { // LCOV_BR_EXCL_LINE
         /* XXX: Hack to avoid a hard crash that I haven't been able
          * to pintpoint yet (test doesn't fail reliably, and I haven't
          * been able to trigger it with a unittest.
@@ -72,7 +72,7 @@ const char* _Nullable PyObjCBlock_GetSignature(void* _block)
          * Without this test the tests for blocks without metadata
          * sometimes crash due to invalid block pointers :-(
          */
-        return NULL;
+        return NULL; // LCOV_EXCL_LINE
     }
 
     if ((block->flags & BLOCK_HAS_SIGNATURE) != 0) {
@@ -189,16 +189,8 @@ PyObject* _Nullable PyObjCBlock_Call(PyObject* module __attribute__((__unused__)
     }
 
     block_ptr = PyObjCObject_GetObject(self);
-    if (block_ptr == nil) { // LCOV_BR_EXCL_LINE
-        /* There is no realistic way to get a 'nil' block here, the
-         * objc_object field is only set to 'nil' when an 'init' call
-         * fails and blocks are already initialized.
-         */
-        // LCOV_EXCL_START
-        PyErr_SetString(PyExc_ValueError, "Cannot call nil block");
-        return NULL;
-        // LCOV_EXCL_STOP
-    }
+    assert(block_ptr != nil);
+
     call_func = PyObjCBlock_GetFunction(block_ptr);
 
     argbuf_len = PyObjCRT_SizeOfReturnType(signature->rettype->type);
@@ -362,24 +354,18 @@ PyObject* _Nullable PyObjCBlock_Call(PyObject* module __attribute__((__unused__)
 
 #ifdef __arm64__
     retval = PyObjCFFI_BuildResult(signature, 1, argbuf, byref, byref_attr,
-                                   byref_out_count, NULL, 0, values);
+                                   byref_out_count, values);
 #else
     retval =
         PyObjCFFI_BuildResult(signature, 1, argbuf, byref, byref_attr, byref_out_count,
-                              NULL, 0, useStret ? values + 1 : values);
+                              useStret ? values + 1 : values);
 #endif
 
     if (variadicAllArgs) {
-        if (PyObjCFFI_FreeByRef(Py_SIZE(signature) + PyTuple_Size(args), byref, // LCOV_BR_EXCL_LINE
-                                byref_attr)
-            < 0) {
-            goto error; // LCOV_EXCL_LINE
-        }
+        PyObjCFFI_FreeByRef(Py_SIZE(signature) + PyTuple_Size(args), byref, byref_attr);
 
     } else {
-        if (PyObjCFFI_FreeByRef(Py_SIZE(signature), byref, byref_attr) < 0) { // LCOV_BR_EXCL_LINE
-            goto error; // LCOV_EXCL_LINE
-        }
+        PyObjCFFI_FreeByRef(Py_SIZE(signature), byref, byref_attr);
     }
 
     PyMem_Free(argbuf);
@@ -443,7 +429,7 @@ void* _Nullable PyObjCBlock_Create(PyObjCMethodSignature* signature, PyObject* c
 {
     struct block_literal block = gLiteralTemplate;
 
-    PyObjC_Assert(gGlobalBlockClass, NULL);
+    assert(gGlobalBlockClass != Nil);
 
     block.descriptor_memory = PyBytes_FromStringAndSize(NULL, sizeof(struct block_descriptor));
     if (block.descriptor_memory == NULL) { // LCOV_BR_EXCL_LINE
@@ -517,9 +503,12 @@ static PyObject* _Nullable pyobjc_PythonObject(NSObject* self,
      */
     self = [self copy];
 
-    rval = (PyObject*)PyObjCObject_New(self, PyObjCObject_kDEFAULT, NO);
+    rval = (PyObject*)PyObjCObject_New(self, PyObjCObject_kDEFAULT, YES);
+    [self release];
     if (rval == NULL) { // LCOV_BR_EXCL_LINE
-        return NULL;    // LCOV_EXCL_LINE
+        // LCOV_EXCL_START
+        return NULL;
+        // LCOV_EXCL_STOP
     } else {
         PyObject* actual = PyObjC_RegisterPythonProxy(self, rval);
         Py_DECREF(rval);
@@ -527,6 +516,11 @@ static PyObject* _Nullable pyobjc_PythonObject(NSObject* self,
     }
 }
 
+/* the __pyobjc_PythonTransient__ method is only used in the
+ * implementation of Python methods on a class. Those aren't
+ * supported for blocks.
+ */
+// LCOV_EXCL_START
 static PyObject* _Nullable pyobjc_PythonTransient(NSObject* self,
                                                   SEL  _sel __attribute__((__unused__)),
                                                   int* cookie)
@@ -539,9 +533,14 @@ static PyObject* _Nullable pyobjc_PythonTransient(NSObject* self,
     self = [self copy];
 
     *cookie = 1;
-    PyObject* result  = PyObjCObject_New(self, PyObjCObject_kDEFAULT, NO);
+    PyObject* result  = PyObjCObject_New(self, PyObjCObject_kDEFAULT, YES);
+    [self release];
+    if (result == NULL) {
+        return NULL;
+    }
     return result;
 }
+// LCOV_EXCL_STOP
 
 int
 PyObjCBlock_Setup(PyObject* module __attribute__((__unused__)))
