@@ -74,7 +74,7 @@ setup_gMethods_selectors(void)
 int
 PyObjCClass_FinishClass(Class objc_class)
 {
-    PyObjC_Assert(objc_class != nil, -1);
+    assert(objc_class != nil);
 
     objc_registerClassPair(objc_class);
     return 0;
@@ -89,8 +89,8 @@ PyObjCClass_FinishClass(Class objc_class)
 int
 PyObjCClass_UnbuildClass(Class objc_class __attribute__((__unused__)))
 {
-    PyObjC_Assert(objc_class != nil, -1);
-    PyObjC_Assert(objc_lookUpClass(class_getName(objc_class)) == nil, -1);
+    assert(objc_class != nil);
+    assert(objc_lookUpClass(class_getName(objc_class)) == nil);
 
     objc_disposeClassPair(objc_class);
     return 0;
@@ -131,7 +131,7 @@ static Class _Nullable build_intermediate_class(Class base_class, char* name)
 #ifdef Py_GIL_DISABLED
         PyMutex_Unlock(&intermediate_mutex);
 #endif
-        PyObjC_Assert(class_getSuperclass(intermediate_class) == base_class, Nil);
+        assert(class_getSuperclass(intermediate_class) == base_class);
 
         return intermediate_class;
     }
@@ -279,9 +279,9 @@ Class _Nullable PyObjCClass_BuildClass(Class super_class, PyObject* protocols, c
     PyObject*  instance_methods   = NULL;
     PyObject*  class_methods      = NULL;
 
-    PyObjC_Assert(super_class != Nil, Nil);
-    PyObjC_Assert(PyList_Check(protocols), Nil);
-    PyObjC_Assert(PyDict_Check(class_dict), Nil);
+    assert(super_class != Nil);
+    assert(PyList_Check(protocols));
+    assert(PyDict_Check(class_dict));
 
     if (objc_lookUpClass(name) != NULL) {
         PyErr_Format(PyObjCExc_Error, "%s is overriding existing Objective-C class",
@@ -318,10 +318,13 @@ Class _Nullable PyObjCClass_BuildClass(Class super_class, PyObject* protocols, c
     class_methods = PyTuple_GET_ITEM(rv, 2);
     Py_INCREF(class_methods);
     *has_dunder_new = PyObject_IsTrue(PyTuple_GET_ITEM(rv, 3));
+    if (*has_dunder_new == -1) {
+        goto error_cleanup;
+    }
     Py_DECREF(rv);
-    PyObjC_Assert(instance_variables != NULL, Nil);
-    PyObjC_Assert(instance_methods != NULL, Nil);
-    PyObjC_Assert(class_methods != NULL, Nil);
+    assert(instance_variables != NULL);
+    assert(instance_methods != NULL);
+    assert(class_methods != NULL);
     if (validate_tuple(instance_variables, is_ivar,
                        "invalid instance_variables in result of class dict transformer")
         == -1) {
@@ -404,11 +407,9 @@ Class _Nullable PyObjCClass_BuildClass(Class super_class, PyObject* protocols, c
             continue;
         }
 
-        /* PyObjCFormalProtocol_GetProtocol() is nonnull because we've already type
-         * checked */
         if (!class_addProtocol( // LCOV_BR_EXCL_LINE
                 new_class,
-                (Protocol* _Nonnull)PyObjCFormalProtocol_GetProtocol(wrapped_protocol))) {
+                PyObjCFormalProtocol_GetProtocol(wrapped_protocol))) {
             // LCOV_EXCL_START
             Py_DECREF(wrapped_protocol);
             goto error_cleanup;
@@ -757,18 +758,23 @@ free_ivars(id self, PyObject* cls)
             cur_cls = NULL;
             // LCOV_EXCL_STOP
 
-        } else if (PyTuple_Size(o) == 0) {
+        } else if (PyTuple_Size(o) == 0) { // LCOV_BR_EXCL_LINE
+            /* It should be impossible to have and empty
+             * __bases__ for PyObjC's classes.
+             */
+            // LCOV_EXCL_START
             PyErr_Clear();
             cur_cls = NULL;
             Py_DECREF(o);
+            // LCOV_EXCL_STOP
 
-        } else {
+        } else { // LCOV_EXCL_LINE
             cur_cls = PyTuple_GET_ITEM(o, 0);
-            if (cur_cls == (PyObject*)&PyObjCClass_Type) {
-                cur_cls = NULL;
-            }
+            if (cur_cls == (PyObject*)&PyObjCClass_Type) { // LCOV_BR_EXCL_LINE
+                cur_cls = NULL; // LCOV_EXCL_LINE
+            } // LCOV_EXCL_LINE
             Py_DECREF(o);
-        }
+        } // LCOV_EXCL_LINE
     }
 }
 
@@ -875,6 +881,9 @@ object_method_copyWithZone_(ffi_cif* cif __attribute__((__unused__)), void* resp
          *      It might be possible to make the two consistent, but that could
          *      break backward compatibility.
          *      This code also needs tests!
+         *
+         * XXX: Shouldn't this use the objc.ivar instances in the class proxy
+         *      for this class?
          */
         for (i = 0; i < ivarCount; i++) {
             Ivar        v = ivarList[i];
@@ -885,25 +894,30 @@ object_method_copyWithZone_(ffi_cif* cif __attribute__((__unused__)), void* resp
             typestr = ivar_getTypeEncoding(v);
             offset  = ivar_getOffset(v);
 
-            if (strcmp(typestr, @encode(PyObject*)) != 0)
-                continue;
-
-            /* A PyObject, increase it's refcount */
-            p = (PyObject**)(((char*)copy) + offset);
-            if (*p == NULL)
-                continue;
-            if (strcmp(ivar_getName(v), "__dict__") == 0) {
-                /* copy __dict__ */
-                *p = PyDict_Copy(*p);
-                if (*p == NULL) { // LCOV_BR_EXCL_LINE
-                    // LCOV_EXCL_START
-                    [copy release];
-                    PyObjCErr_ToObjCWithGILState(&state);
-                    return;
-                    // LCOV_EXCL_STOP
+            if (strcmp(typestr, @encode(PyObject*)) == 0) {
+                /* A PyObject, increase it's refcount */
+                p = (PyObject**)(((char*)copy) + offset);
+                if (*p == NULL)
+                    continue;
+                if (strcmp(ivar_getName(v), "__dict__") == 0) {
+                    /* copy __dict__ */
+                    *p = PyDict_Copy(*p);
+                    if (*p == NULL) { // LCOV_BR_EXCL_LINE
+                        // LCOV_EXCL_START
+                        [copy release];
+                        PyObjCErr_ToObjCWithGILState(&state);
+                        return;
+                        // LCOV_EXCL_STOP
+                    }
+                } else {
+                    Py_INCREF(*p);
                 }
-            } else {
-                Py_INCREF(*p);
+            } else if (*typestr == _C_ID) {
+                /* XXX: This doesn't do the correct thing for
+                 *      weak reference like in outlets. Should
+                 *      look at the Python proxy!
+                 */
+                [*(id*)(((char*)copy) + offset) retain];
             }
         }
 
@@ -960,11 +974,13 @@ object_method_forwardInvocation(ffi_cif* cif __attribute__((__unused__)),
 
     @try {
         theSelector = [invocation selector];
+    // LCOV_EXCL_START
     } @catch (NSObject* localException) {
         Py_DECREF(pyself);
         PyGILState_Release(state);
         @throw;
     }
+    // LCOV_EXCL_STOP
 
     PyObject* pymeth = PyObjCObject_FindSelector(pyself, theSelector);
 
@@ -1169,7 +1185,7 @@ object_method_setValue_forKey_(ffi_cif* cif __attribute__((__unused__)),
             }
             PyGILState_Release(state);
         } else {
-            @throw;
+            @throw; // LCOV_EXCL_LINE
         }
     }
 }
