@@ -11,48 +11,19 @@ PyObjC_GetClassList(bool ignore_invalid_identifiers)
 {
     PyObject* result    = NULL;
     Class*    buffer    = NULL;
-    int       bufferLen = 0;
-    int       neededLen = 0;
-    int       i;
-
-    /*
-     * objc_getClassList returns the number of classes known in the runtime,
-     * the documented way to fetch the list is:
-     * 1. call ret = objc_getClassList(NULL, 0);
-     * 2. allocate a buffer of 'ret' class-pointers
-     * 3. call objc_getClassList again with this buffer.
-     *
-     * Step 3 might return more classes because another thread may have
-     * loaded a new framework/bundle. This means we need a loop to be sure
-     * we'll get all classes.
-     */
-    neededLen = objc_getClassList(NULL, 0);
-    bufferLen = 0;
-    buffer    = NULL;
-
-    while (bufferLen < neededLen) {
-        Class* newBuffer;
-        bufferLen = neededLen;
-
-        newBuffer = PyMem_Realloc(buffer, sizeof(Class) * bufferLen);
-        if (newBuffer == NULL) { // LCOV_BR_EXCL_LINE
-            // LCOV_EXCL_START
-            PyErr_NoMemory();
-            goto error;
-            // LCOV_EXCL_STOP
-        }
-
-        buffer    = newBuffer;
-        newBuffer = NULL;
-        neededLen = objc_getClassList(buffer, bufferLen);
-    }
-    bufferLen = neededLen;
+    unsigned int       bufferLen = 0;
+    unsigned int       i;
 
     result = PyList_New(0);
     if (result == NULL) { // LCOV_BR_EXCL_LINE
         goto error;       // LCOV_EXCL_LINE
     }
 
+    buffer = objc_copyClassList(&bufferLen);
+    if (buffer == NULL) {
+        return result;
+    }
+
     for (i = 0; i < bufferLen; i++) {
         PyObject* pyclass;
 
@@ -506,7 +477,7 @@ PyObjC_GetClassList(bool ignore_invalid_identifiers)
         }
     }
 
-    PyMem_Free(buffer);
+    free(buffer);
     buffer = NULL;
 
     PyObject* tmp = PyList_AsTuple(result);
@@ -517,7 +488,7 @@ PyObjC_GetClassList(bool ignore_invalid_identifiers)
 error:
     // LCOV_EXCL_START
     if (buffer != NULL) {
-        PyMem_Free(buffer);
+        free(buffer);
     }
     Py_XDECREF(result);
     return NULL;
