@@ -835,6 +835,7 @@ END_UNITTEST
 BEGIN_UNITTEST(UnicodeFunctions)
 PyObject* unicode = PyUnicode_FromString("hello world");
 int       ok;
+ASSERT(unicode != NULL);
 
 ok = PyObjC_is_ascii_string(unicode, "hello world");
 ASSERT(ok);
@@ -860,6 +861,18 @@ ASSERT(ok);
 ok = PyObjC_is_ascii_prefix(unicode, "hello world!", 12);
 ASSERT(!ok);
 
+Py_CLEAR(unicode);
+
+unicode = PyUnicode_FromString("Straße");
+ASSERT(unicode != NULL);
+ok = PyObjC_is_ascii_prefix(unicode, "hello", 5);
+ASSERT(!ok);
+
+ok = PyObjC_is_ascii_string(unicode, "hello");
+ASSERT(!ok);
+
+Py_CLEAR(unicode);
+
 END_UNITTEST
 
 BEGIN_UNITTEST(DecimalSize)
@@ -970,7 +983,11 @@ END_UNITTEST
 
 BEGIN_UNITTEST(InvalidObjCPointer)
 
-/* Invalid type encoding */
+/* Invalid type encodings
+ *
+ * XXX: This tests are artificial, but at least
+ *      ensure that some error paths are hit.
+ */
 PyObject* p = PyObjCPointer_New(&p, "^{foo=");
 ASSERT(PyErr_Occurred());
 ASSERT(!p);
@@ -981,6 +998,11 @@ ASSERT(PyErr_Occurred());
 ASSERT(!p);
 PyErr_Clear();
 
+p = PyObjCPointer_New(&p, "X");
+ASSERT(PyErr_Occurred());
+ASSERT(!p);
+PyErr_Clear();
+
 END_UNITTEST
 
 BEGIN_UNITTEST(InvalidRegistryUsage)
@@ -1164,6 +1186,24 @@ BEGIN_UNITTEST(NoKwNames)
 
 END_UNITTEST
 
+BEGIN_UNITTEST(PyObjC_NSMethodSignatureToTypeString_Errors)
+    char buffer[2048];
+    char* result;
+    NSMethodSignature* sig = [NSURL instanceMethodSignatureForSelector:@selector(initByResolvingBookmarkData:options:relativeToURL:bookmarkDataIsStale:error:)];
+    ASSERT(sig != nil);
+
+    result = PyObjC_NSMethodSignatureToTypeString(sig, buffer, 0);
+    ASSERT(result == NULL);
+    ASSERT(PyErr_Occurred());
+    PyErr_Clear();
+
+    result = PyObjC_NSMethodSignatureToTypeString(sig, buffer, 4);
+    ASSERT(result == NULL);
+    ASSERT(PyErr_Occurred());
+    PyErr_Clear();
+
+END_UNITTEST
+
 static PyMethodDef mod_methods[] = {TESTDEF(CheckNSInvoke),
 
                                     TESTDEF(VectorSize),
@@ -1204,6 +1244,7 @@ static PyMethodDef mod_methods[] = {TESTDEF(CheckNSInvoke),
                                     TESTDEF(ValidEncoding),
                                     TESTDEF(CheckArgCount),
                                     TESTDEF(NoKwNames),
+                                    TESTDEF(PyObjC_NSMethodSignatureToTypeString_Errors),
                                     {0, 0, 0, 0}};
 
 int
