@@ -54,6 +54,12 @@ static PyMethodDef func_methods[] = {
      .ml_meth  = (PyCFunction)func_metadata,
      .ml_flags = METH_NOARGS,
      .ml_doc   = "Return a dict that describes the metadata for this function."},
+#if PY_VERSION_HEX > 0x03090000
+    {.ml_name  = "__class_getitem__",
+     .ml_meth  = (PyCFunction)Py_GenericAlias,
+     .ml_flags = METH_O|METH_CLASS,
+     .ml_doc   = "See PEP 585"},
+#endif
     {
         .ml_name = NULL /* SENTINEL */
     }};
@@ -169,8 +175,8 @@ static PyObject* _Nullable func_vectorcall(PyObject* s, PyObject* const* args,
     argbuf_len = align(argbuf_len, sizeof(void*));
     r = PyObjCFFI_CountArguments(self->methinfo, 0, &byref_in_count, &byref_out_count,
                                  &plain_count, &argbuf_len, &variadicAllArgs);
-    if (r == -1) {
-        return NULL;
+    if (r == -1) { // LCOV_BR_EXCL_LINE
+        return NULL; // LCOV_EXCL_LINE
     }
 
     variadicAllArgs |=
@@ -242,9 +248,11 @@ static PyObject* _Nullable func_vectorcall(PyObject* s, PyObject* const* args,
 #pragma clang diagnostic pop
 #endif
 
-        if (r != FFI_OK) {
+        if (r != FFI_OK) { // LCOV_BR_EXCL_LINE
+            // LCOV_EXCL_START
             PyErr_Format(PyExc_RuntimeError, "Cannot setup FFI CIF [%d]", r);
             goto error;
+            // LCOV_EXCL_STOP
         }
         cifptr = &cif;
 
@@ -266,19 +274,14 @@ static PyObject* _Nullable func_vectorcall(PyObject* s, PyObject* const* args,
     }
 
     retval = PyObjCFFI_BuildResult(self->methinfo, 0, argbuf, byref, byref_attr,
-                                   byref_out_count, NULL, 0, values);
+                                   byref_out_count, values);
 
 error:
     if (variadicAllArgs) {
-        if (PyObjCFFI_FreeByRef(Py_SIZE(self->methinfo) + nargsf, byref, byref_attr)
-            < 0) {
-            Py_CLEAR(retval);
-        }
+        PyObjCFFI_FreeByRef(Py_SIZE(self->methinfo) + nargsf, byref, byref_attr);
 
     } else {
-        if (PyObjCFFI_FreeByRef(Py_SIZE(self->methinfo), byref, byref_attr) < 0) {
-            Py_CLEAR(retval);
-        }
+        PyObjCFFI_FreeByRef(Py_SIZE(self->methinfo), byref, byref_attr);
     }
 
     if (argbuf) {
@@ -303,7 +306,7 @@ static PyObject* _Nullable func_vectorcall_simple(PyObject* s, PyObject* const*
     unsigned char argbuf[SHORTCUT_MAX_ARGBUF];
     void*         values[MAX_ARGCOUNT_SIMPLE];
 
-    PyObjC_Assert(self->methinfo->shortcut_signature, NULL);
+    assert(self->methinfo->shortcut_signature);
 
     if (unlikely(kwnames != NULL
                  && (PyTuple_CheckExact(kwnames) && PyTuple_GET_SIZE(kwnames) != 0))) {
@@ -339,7 +342,7 @@ static PyObject* _Nullable func_vectorcall_simple(PyObject* s, PyObject* const*
                      self->methinfo, 0, args, nargsf,
                      align(PyObjCRT_SizeOfReturnType(self->methinfo->rettype->type),
                            sizeof(void*)),
-                     argbuf, sizeof(argbuf), /*arglist,*/ values)
+                     argbuf, sizeof(argbuf), values)
                  == -1)) {
 
         goto error;
@@ -357,7 +360,7 @@ static PyObject* _Nullable func_vectorcall_simple(PyObject* s, PyObject* const*
         goto error;
     }
 
-    return PyObjCFFI_BuildResult_Simple(self->methinfo, argbuf, NULL, 0);
+    return PyObjCFFI_BuildResult_Simple(self->methinfo, argbuf);
 
 error:
     return NULL;
@@ -458,7 +461,7 @@ PyObject* _Nullable PyObjCFunc_WithMethodSignature(PyObject* _Nullable name, voi
 {
     func_object* result;
 
-    PyObjC_Assert(!name || PyUnicode_Check(name), NULL);
+    assert(!name || PyUnicode_Check(name));
 
     result = PyObject_NEW(func_object, (PyTypeObject*)PyObjCFunc_Type);
     if (result == NULL) // LCOV_BR_EXCL_LINE
@@ -490,7 +493,7 @@ PyObject* _Nullable PyObjCFunc_New(PyObject* name, void* func, const char* signa
 {
     func_object* result;
 
-    PyObjC_Assert(!name || PyUnicode_Check(name), NULL);
+    assert(!name || PyUnicode_Check(name));
     if (doc && PyUnicode_GetLength(doc) == 0) {
         /* Ignore empty docstring */
         doc = NULL;
@@ -552,8 +555,8 @@ PyObjCFunc_Setup(PyObject* module)
         return -1;                 // LCOV_EXCL_LINE
     }
 
-    if ( // LCOV_BR_EXCL_LINE
-        PyModule_AddObject(module, "function", PyObjCFunc_Type) == -1) {
+    if (PyModule_AddObject( // LCOV_BR_EXCL_LINE
+            module, "function", PyObjCFunc_Type) == -1) {
         return -1; // LCOV_EXCL_LINE
     }
     Py_INCREF(PyObjCFunc_Type);
