@@ -8,6 +8,9 @@
  *   NSDecimal objects support +, -, *, /, +=, -=, *= and /=, which directly
  *   correspond with NSDecimal* functions with the NSRoundPlain argument.
  *   They also support unary -, unary + and abs, with the obvious semantics.
+ *
+ * The helpers do not support subclassing NSDecimalNumber because subclassing
+ * this type doesn't work in Objective-C either on modern versions of macOS.
  */
 #include "pyobjc.h"
 
@@ -152,7 +155,7 @@ DecimalFromString(NSDecimal* aDecimal, NSString* aString,
     NSDecimalNumber* num;
 
     num       = [[NSDecimalNumber alloc] initWithString:aString];
-    *aDecimal = [num decimalValue];
+    *aDecimal = [num decimalValue]; // LCOV_BR_EXCL_LINE
     [num release];
 }
 
@@ -166,7 +169,7 @@ DecimalFromComponents(NSDecimal* aDecimal, unsigned long long mantissa,
                                            exponent:exponent
                                          isNegative:negative];
 
-    *aDecimal = [num decimalValue];
+    *aDecimal = [num decimalValue]; // LCOV_BR_EXCL_LINE
     [num release];
 }
 
@@ -183,8 +186,8 @@ decimal_new(PyTypeObject* type __attribute__((__unused__)), PyObject* _Nullable
 
     memset(&self->value, 0, sizeof(self->value));
     self->objc_value = nil;
-    if ((args == NULL || PyTuple_Size(args) == 0)
-        && (kwds == NULL || PyDict_Size(kwds) == 0)) {
+    if ((args == NULL || PyTuple_Size(args) == 0) // LCOV_BR_EXCL_LINE
+        && (kwds == NULL || PyDict_Size(kwds) == 0)) { // LCOV_BR_EXCL_LINE
         DecimalFromComponents(&self->value, 0, 0, 0);
         return (PyObject*)self;
     }
@@ -220,7 +223,7 @@ PyObjC_number_to_decimal(PyObject* pyValue, NSDecimal* outResult)
 
     if (PyLong_Check(pyValue)) {
         mantissa = PyLong_AsUnsignedLongLong(pyValue);
-        if (mantissa == (unsigned long long)-1 && PyErr_Occurred()) {
+        if (mantissa == (unsigned long long)-1 && PyErr_Occurred()) { // LCOV_BR_EXCL_LINE
             long long lng;
             PyErr_Clear();
             lng = PyLong_AsLongLong(pyValue);
@@ -286,7 +289,7 @@ PyObjC_number_to_decimal(PyObject* pyValue, NSDecimal* outResult)
         }
 
         @try {
-            DecimalFromString(outResult, stringVal, NULL);
+            DecimalFromString(outResult, stringVal, NULL); // LCOV_BR_EXCL_LINE
 
             // LCOV_EXCL_START
             /* Experiments show that DecimalFromString won't raise, but
@@ -314,7 +317,7 @@ PyObjC_number_to_decimal(PyObject* pyValue, NSDecimal* outResult)
     if (_NSDecimalNumber_Class != NULL
         && PyObject_IsInstance(pyValue, _NSDecimalNumber_Class)) {
         NSDecimalNumber* val = PyObjCObject_GetObject(pyValue);
-        *outResult           = [val decimalValue];
+        *outResult           = [val decimalValue]; // LCOV_BR_EXCL_LINE
         return 0;
     }
 
@@ -334,7 +337,7 @@ decimal_init(PyObject* self, PyObject* _Nullable args, PyObject* _Nullable kwds)
     PyObject*          pyMantissa;
     PyObject*          pyExponent;
     PyObject*          pyNegative;
-    BOOL               negative;
+    int                negative;
     unsigned long long mantissa;
     short int          exponent;
 
@@ -361,7 +364,7 @@ decimal_init(PyObject* self, PyObject* _Nullable args, PyObject* _Nullable kwds)
             }
             Py_BEGIN_ALLOW_THREADS
                 @try {
-                    DecimalFromString(&Decimal_Value(self), stringVal, NULL);
+                    DecimalFromString(&Decimal_Value(self), stringVal, NULL); // LCOV_BR_EXCL_LINE
 
                     // LCOV_EXCL_START
                     // DecimalFromString does not raise, but returns a NaN on
@@ -382,6 +385,9 @@ decimal_init(PyObject* self, PyObject* _Nullable args, PyObject* _Nullable kwds)
     }
 
     negative = PyObject_IsTrue(pyNegative);
+    if (negative == -1) {
+        return -1;
+    }
     if (depythonify_c_value(@encode(short int), pyExponent, &exponent) == -1) {
         return -1;
     }
@@ -390,7 +396,7 @@ decimal_init(PyObject* self, PyObject* _Nullable args, PyObject* _Nullable kwds)
         return -1;
     }
 
-    DecimalFromComponents(&Decimal_Value(self), mantissa, exponent, negative);
+    DecimalFromComponents(&Decimal_Value(self), mantissa, exponent, negative?YES:NO);
 
     return 0;
 }
@@ -401,9 +407,9 @@ decimal_hash(PyObject* self)
     NSString*  strrepr = NSDecimalString(&Decimal_Value(self), nil);
     NSUInteger hash    = [strrepr hash];
 
-    if ((Py_hash_t)hash == (Py_hash_t)-1) {
+    if ((Py_hash_t)hash == (Py_hash_t)-1) { // LCOV_BR_EXCL_LINE
         /* hash -1 is not used by Python */
-        return -2;
+        return -2; // LCOV_EXCL_LINE
     } else {
         return hash;
     }
@@ -413,19 +419,16 @@ static PyObject*
 decimal_richcompare(PyObject* self, PyObject* other, int type)
 {
     NSComparisonResult res;
-    (void)decimal_coerce_compare(&self, &other);
+    int r = decimal_coerce_compare(&self, &other);
+    if (r == 1) {
+        PyErr_Clear();
+    }
 
     if (!Decimal_Check(other)) {
         if (type == Py_EQ) {
-            if (PyErr_Occurred()) {
-                PyErr_Clear();
-            }
             Py_RETURN_FALSE;
 
         } else if (type == Py_NE) {
-            if (PyErr_Occurred()) {
-                PyErr_Clear();
-            }
             Py_RETURN_TRUE;
         }
 
@@ -434,13 +437,9 @@ decimal_richcompare(PyObject* self, PyObject* other, int type)
         return NULL;
     }
 
-    if (PyErr_Occurred()) {
-        return NULL;
-    }
-
     res = NSDecimalCompare(&Decimal_Value(self), &Decimal_Value(other));
 
-    switch (type) {
+    switch (type) { // LCOV_BR_EXCL_LINE
     case Py_LT:
         if(res == NSOrderedAscending) { Py_RETURN_TRUE; } else { Py_RETURN_FALSE; };
     case Py_LE:
@@ -489,13 +488,15 @@ static PyObject* _Nullable decimal_result_to_python(NSCalculationError status,
 {
     /* XXX: Need tests that trigger these errors, including the loss of precision one */
     NSDecimal value2;
-    switch (status) {
+    switch (status) { // LCOV_BR_EXCL_LINE
     case NSCalculationOverflow:
         PyErr_SetString(PyExc_OverflowError, "Numeric overflow");
         return NULL;
     case NSCalculationUnderflow:
+        // LCOV_EXCL_START
         PyErr_SetString(PyExc_OverflowError, "Numeric underflow");
         return NULL;
+        // LCOV_EXCL_STOP
     case NSCalculationDivideByZero:
         PyErr_SetString(PyExc_ZeroDivisionError, "Division by zero");
         return NULL;
@@ -520,6 +521,7 @@ static PyObject* _Nullable decimal_result_to_python(NSCalculationError status,
 #define TRY_COERCE(left, right)                                                          \
     int r = decimal_coerce(&left, &right);                                               \
     if (r == 1) {                                                                        \
+        if (PyErr_Occurred()) PyErr_Clear();                                             \
         Py_RETURN_NOTIMPLEMENTED;                                                        \
     }
 
@@ -537,8 +539,8 @@ static PyObject* _Nullable decimal_result_to_python(NSCalculationError status,
     }
 
 DECIMAL_OPERATOR(decimal_add, NSDecimalAdd)
-DECIMAL_OPERATOR(decimal_subtract, NSDecimalSubtract)
-DECIMAL_OPERATOR(decimal_multiply, NSDecimalMultiply)
+DECIMAL_OPERATOR(decimal_subtract, NSDecimalSubtract) // LCOV_BR_EXCL_LINE
+DECIMAL_OPERATOR(decimal_multiply, NSDecimalMultiply) // LCOV_BR_EXCL_LINE
 DECIMAL_OPERATOR(decimal_divide, NSDecimalDivide)
 
 static PyObject*
@@ -591,7 +593,7 @@ decimal_absolute(PyObject* self)
     NSDecimal          zero;
     DecimalFromComponents(&zero, 0, 0, 0);
 
-    switch (NSDecimalCompare(&zero, &Decimal_Value(self))) {
+    switch (NSDecimalCompare(&zero, &Decimal_Value(self))) { // LCOV_BR_EXCL_LINE
     case NSOrderedSame:
     case NSOrderedAscending:
         /* self >= 0 */
@@ -702,18 +704,11 @@ error:
 static int
 decimal_coerce_compare(PyObject** l, PyObject** r)
 {
-    if (PyFloat_Check(*l)) {
-        NSDecimal tmp;
-        PyObjC_number_to_decimal(*l, &tmp);
-        if (PyObjC_number_to_decimal(*r, &tmp) == -1) {
-            return 1;
-        }
-        *l = Decimal_New(&tmp);
-    }
+    assert(Decimal_Check(*l)); /* documented guarantee */
     if (PyFloat_Check(*r)) {
         NSDecimal tmp;
-        if (PyObjC_number_to_decimal(*r, &tmp) == -1) {
-            return 1;
+        if (PyObjC_number_to_decimal(*r, &tmp) == -1) { // LCOV_BR_EXCL_LINE
+            return 1; // LCOV_EXCL_LINE
         }
         *r = Decimal_New(&tmp);
     }
@@ -725,25 +720,14 @@ decimal_repr(PyObject* self)
 {
     NSString* val = NSDecimalString(&Decimal_Value(self), NULL);
     PyObject* tmp = id_to_python(val);
-    if (tmp == NULL) { // LCOV_BR_EXCL_START
-        return NULL;   // LCOV_EXCL_STOP
+    if (tmp == NULL) { // LCOV_BR_EXCL_LINE
+        return NULL;   // LCOV_EXCL_LINE
     }
     PyObject* repr = PyObject_Str(tmp);
     Py_XDECREF(tmp);
     return repr;
 }
 
-static inline int
-Decimal_Convert(PyObject* self, void* val)
-{
-    if (Decimal_Check(self)) {
-        *(NSDecimal**)val = &Decimal_Value(self);
-        return 1;
-    }
-    PyErr_Format(PyExc_TypeError, "Expecting an NSDecimal, got instance of '%s'",
-                 Py_TYPE(self)->tp_name);
-    return 0;
-}
 
 static PyObject*
 Decimal_New(const NSDecimal* aDecimal)
@@ -809,11 +793,11 @@ static PyObject* _Nullable call_NSDecimalNumber_decimalNumberWithDecimal_(
 
     Py_BEGIN_ALLOW_THREADS
         @try {
-            super.super_class = (Class _Nonnull)object_getClass(PyObjCSelector_GetClass(method));
-            super.receiver    = (id _Nonnull)object_getClass(PyObjCClass_GetClass(self));
+            super.super_class = (Class _Nonnull)object_getClass(PyObjCSelector_GetClass(method)); // LCOV_BR_EXCL_LINE
+            super.receiver    = (id _Nonnull)object_getClass(PyObjCClass_GetClass(self)); // LCOV_BR_EXCL_LINE
 
-            res = ((id(*)(struct objc_super*, SEL, NSDecimal))objc_msgSendSuper)(
-                &super, PyObjCSelector_GetSelector(method), *aDecimal);
+            res = ((id(*)(struct objc_super*, SEL, NSDecimal))objc_msgSendSuper)( // LCOV_BR_EXCL_LINE
+                &super, PyObjCSelector_GetSelector(method), *aDecimal); // LCOV_BR_EXCL_LINE
             // LCOV_EXCL_START
             // AFAIK the method won't raise.
         } @catch (NSObject* localException) {
@@ -823,8 +807,8 @@ static PyObject* _Nullable call_NSDecimalNumber_decimalNumberWithDecimal_(
         // LCOV_EXCL_STOP
     Py_END_ALLOW_THREADS
 
-    if (res == nil && PyErr_Occurred()) {
-        return NULL;
+    if (res == nil && PyErr_Occurred()) { // LCOV_BR_EXCL_LINE
+        return NULL; // LCOV_EXCL_LINE
     }
 
     return id_to_python(res);
@@ -833,6 +817,11 @@ static PyObject* _Nullable call_NSDecimalNumber_decimalNumberWithDecimal_(
 static PyObject* _Nullable call_NSDecimalNumber_initWithDecimal_(
     PyObject* method, PyObject* self, PyObject* const* arguments, size_t nargs)
 {
+    /*
+     * NOTE: This helper does not implement the "init" family protocol (e.g.
+     *       stolen reference to self, new reference returned) because doing
+     *       that causes a hard crash on macOS 15.
+     */
     struct objc_super super;
     NSDecimal*        aDecimal;
     id                res;
@@ -850,11 +839,12 @@ static PyObject* _Nullable call_NSDecimalNumber_initWithDecimal_(
 
     Py_BEGIN_ALLOW_THREADS
         @try {
-            super.super_class = PyObjCSelector_GetClass(method);
-            super.receiver    = (id _Nonnull)PyObjCObject_GetObject(self);
+            super.super_class = PyObjCSelector_GetClass(method); // LCOV_BR_EXCL_LINE
+            super.receiver    = PyObjCObject_GetObject(self); // LCOV_BR_EXCL_LINE
+
+            res = ((id(*)(struct objc_super*, SEL, NSDecimal))objc_msgSendSuper)( // LCOV_BR_EXCL_LINE
+                &super, PyObjCSelector_GetSelector(method), *aDecimal); // LCOV_BR_EXCL_LINE
 
-            res = ((id(*)(struct objc_super*, SEL, NSDecimal))objc_msgSendSuper)(
-                &super, PyObjCSelector_GetSelector(method), *aDecimal);
             // LCOV_EXCL_START
             // AFAIK the method won't raise.
         } @catch (NSObject* localException) {
@@ -864,65 +854,13 @@ static PyObject* _Nullable call_NSDecimalNumber_initWithDecimal_(
         // LCOV_EXCL_STOP
     Py_END_ALLOW_THREADS
 
-    if (res == nil && PyErr_Occurred()) {
-        return NULL;
+    if (res == nil && PyErr_Occurred()) { // LCOV_BR_EXCL_LINE
+        return NULL; // LCOV_EXCL_LINE
     }
 
     return id_to_python(res);
 }
 
-static IMP
-mkimp_NSDecimalNumber_initWithDecimal_(PyObject* callable, PyObjCMethodSignature* methinfo
-                                       __attribute__((__unused__)))
-{
-    Py_INCREF(callable);
-    NSDecimalNumber* (^block)(NSDecimalNumber*, NSDecimal) =
-        ^(NSDecimalNumber* _Nullable self, NSDecimal aDecimal) {
-          NSDecimalNumber* rv;
-          PyObject*        result = NULL;
-          PyObject*        v      = NULL;
-          PyObject*        pyself = NULL;
-          int              cookie = 0;
-
-          PyGILState_STATE state = PyGILState_Ensure();
-
-          pyself = PyObjCObject_NewTransient(self, &cookie);
-          if (pyself == NULL)
-              goto error;
-
-          v = Decimal_New(&aDecimal);
-          if (v == NULL)
-              goto error;
-
-          PyObject* arglist[3] = {NULL, pyself, v};
-          result               = PyObject_Vectorcall((PyObject*)callable, arglist + 1,
-                                                     2 | PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
-          Py_DECREF(v);
-          v = NULL;
-          PyObjCObject_ReleaseTransient(pyself, cookie);
-          pyself = NULL;
-          if (result == NULL) {
-              goto error;
-          }
-
-          if (depythonify_python_object(result, &rv) == -1) {
-              Py_DECREF(result);
-              goto error;
-          }
-          Py_DECREF(result);
-          PyGILState_Release(state);
-          return rv;
-
-      error:
-          Py_XDECREF(v);
-          if (pyself) {
-              PyObjCObject_ReleaseTransient(pyself, cookie);
-          }
-          PyObjCErr_ToObjCWithGILState(&state);
-          return (NSDecimalNumber*)nil;
-        };
-    return imp_implementationWithBlock(block);
-}
 
 static PyObject* _Nullable call_NSDecimalNumber_decimalValue(PyObject*        method,
                                                              PyObject*        self,
@@ -938,128 +876,30 @@ static PyObject* _Nullable call_NSDecimalNumber_decimalValue(PyObject*        me
 
     Py_BEGIN_ALLOW_THREADS
         @try {
-            super.super_class = PyObjCSelector_GetClass(method);
-            super.receiver    = (id _Nonnull)PyObjCObject_GetObject(self);
+            super.super_class = PyObjCSelector_GetClass(method); // LCOV_BR_EXCL_LINE
+            super.receiver    = PyObjCObject_GetObject(self); // LCOV_BR_EXCL_LINE
 
 #if defined(__arm64__)
-            aDecimal = ((NSDecimal(*)(struct objc_super*, SEL))objc_msgSendSuper)(
-                &super, PyObjCSelector_GetSelector(method));
+            aDecimal = ((NSDecimal(*)(struct objc_super*, SEL))objc_msgSendSuper)( // LCOV_BR_EXCL_LINE
+                &super, PyObjCSelector_GetSelector(method)); // LCOV_BR_EXCL_LINE
 #else
-            ((void (*)(void*, struct objc_super*, SEL))objc_msgSendSuper_stret)(
-                &aDecimal, &super, PyObjCSelector_GetSelector(method));
+            ((void (*)(void*, struct objc_super*, SEL))objc_msgSendSuper_stret)( // LCOV_BR_EXCL_LINE
+                &aDecimal, &super, PyObjCSelector_GetSelector(method)); // LCOV_BR_EXCL_LINE
 #endif
 
-        } @catch (NSObject* localException) {
-            PyObjCErr_FromObjC(localException);
+        } @catch (NSObject* localException) { // LCOV_EXCL_LINE
+            PyObjCErr_FromObjC(localException); // LCOV_EXCL_LINE
         }
     Py_END_ALLOW_THREADS
 
-    if (PyErr_Occurred()) {
-        return NULL;
+    if (PyErr_Occurred()) { // LCOV_BR_EXCL_LINE
+        return NULL; // LCOV_EXCL_LINE
     }
 
     return Decimal_New(&aDecimal);
 }
 
-static IMP
-mkimp_NSDecimalNumber_decimalNumberWithDecimal_(PyObject*              callable,
-                                                PyObjCMethodSignature* methinfo
-                                                __attribute__((__unused__)))
-{
-    Py_INCREF(callable);
-    NSDecimalNumber* (^block)(Class, NSDecimal) =
-        ^(Class _Nullable self, NSDecimal aDecimal) {
-          NSDecimalNumber* rv;
-          PyObject*        result = NULL;
-          PyObject*        v      = NULL;
-          PyObject*        pyself = NULL;
 
-          PyGILState_STATE state = PyGILState_Ensure();
-
-          pyself = PyObjCClass_New(self);
-          if (pyself == NULL)
-              goto error;
-
-          v = Decimal_New(&aDecimal);
-          if (v == NULL)
-              goto error;
-
-          PyObject* arglist[3] = {NULL, pyself, v};
-
-          result = PyObject_Vectorcall((PyObject*)callable, arglist + 1,
-                                       2 | PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
-          Py_DECREF(v);
-          v = NULL;
-          Py_DECREF(pyself);
-          pyself = NULL;
-          if (result == NULL) {
-              goto error;
-          }
-
-          if (depythonify_python_object(result, &rv) == -1) {
-              Py_DECREF(result);
-              goto error;
-          }
-          Py_DECREF(result);
-          PyGILState_Release(state);
-          return rv;
-
-      error:
-          Py_XDECREF(v);
-          Py_XDECREF(pyself);
-          PyObjCErr_ToObjCWithGILState(&state);
-          return (NSDecimalNumber*)nil;
-        };
-    return imp_implementationWithBlock(block);
-}
-
-static IMP
-mkimp_NSDecimalNumber_decimalValue(PyObject* callable, PyObjCMethodSignature* methinfo
-                                   __attribute__((__unused__)))
-{
-    Py_INCREF(callable);
-
-    NSDecimal (^block)(id) = ^(id _Nullable self) {
-      NSDecimal* res = NULL;
-
-      PyObject* result = NULL;
-      PyObject* v      = NULL;
-
-      PyGILState_STATE state = PyGILState_Ensure();
-
-      v = id_to_python(self);
-      if (v == NULL)
-          goto error;
-
-      PyObject* arglist[2] = {NULL, v};
-      result               = PyObject_Vectorcall((PyObject*)callable, arglist + 1,
-                                                 1 | PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
-      Py_DECREF(v);
-      v = NULL;
-      if (result == NULL)
-          goto error;
-
-      Decimal_Convert(result, &res);
-      if (res == NULL) {
-          Py_DECREF(result);
-          goto error;
-      }
-
-      Py_DECREF(result);
-      PyGILState_Release(state);
-      return *res;
-
-  error:
-      Py_XDECREF(v);
-      PyObjCErr_ToObjCWithGILState(&state);
-#pragma clang diagnostic push
-#pragma clang diagnostic ignored "-Wunreachable-code"
-
-      __builtin_unreachable(); // LCOV_EXCL_LINE
-#pragma clang diagnostic pop
-    };
-    return imp_implementationWithBlock(block);
-}
 
 int
 PyObjC_setup_nsdecimal(PyObject* m)
@@ -1074,7 +914,7 @@ PyObjC_setup_nsdecimal(PyObject* m)
     }
     Py_INCREF(Decimal_Type);
 
-    if (@encode(NSDecimal)[1] == '?') {
+    if (@encode(NSDecimal)[1] == '?') { // LCOV_BR_EXCL_LINE
         Decimal_Encoding[0] = '{';
         Decimal_Encoding[1] = '\0';
         strlcat(Decimal_Encoding, "_NSDecimal", sizeof(Decimal_Encoding));
@@ -1087,7 +927,7 @@ PyObjC_setup_nsdecimal(PyObject* m)
 
     if (PyObjC_RegisterMethodMapping( // LCOV_BR_EXCL_LINE
             classNSDecimalNumber, @selector(initWithDecimal:),
-            call_NSDecimalNumber_initWithDecimal_, mkimp_NSDecimalNumber_initWithDecimal_)
+            call_NSDecimalNumber_initWithDecimal_, PyObjCUnsupportedMethod_IMP)
         < 0) {
         return -1; // LCOV_EXCL_LINE
     }
@@ -1095,10 +935,11 @@ PyObjC_setup_nsdecimal(PyObject* m)
     Class classNSDecimalNumberPlaceholder =
         objc_lookUpClass("NSDecimalNumberPlaceholder");
     if (classNSDecimalNumberPlaceholder != Nil) { // LCOV_BR_EXCL_LINE
-        if (PyObjC_RegisterMethodMapping(classNSDecimalNumberPlaceholder,
-                                         @selector(initWithDecimal:),
-                                         call_NSDecimalNumber_initWithDecimal_,
-                                         mkimp_NSDecimalNumber_initWithDecimal_)
+        if (PyObjC_RegisterMethodMapping( // LCOV_BR_EXCL_LINE
+                     classNSDecimalNumberPlaceholder,
+                     @selector(initWithDecimal:),
+                     call_NSDecimalNumber_initWithDecimal_,
+                     PyObjCUnsupportedMethod_IMP)
             < 0) {
 
             return -1; // LCOV_EXCL_LINE
@@ -1108,14 +949,14 @@ PyObjC_setup_nsdecimal(PyObject* m)
     if (PyObjC_RegisterMethodMapping(classNSDecimalNumber, // LCOV_BR_EXCL_LINE
                                      @selector(decimalNumberWithDecimal:),
                                      call_NSDecimalNumber_decimalNumberWithDecimal_,
-                                     mkimp_NSDecimalNumber_decimalNumberWithDecimal_)
+                                     PyObjCUnsupportedMethod_IMP)
         < 0) {
         return -1; // LCOV_EXCL_LINE
     }
 
     if (PyObjC_RegisterMethodMapping( // LCOV_BR_EXCL_LINE
             classNSNumber, @selector(decimalValue), call_NSDecimalNumber_decimalValue,
-            mkimp_NSDecimalNumber_decimalValue)
+            PyObjCUnsupportedMethod_IMP)
         < 0) {
         return -1; // LCOV_EXCL_LINE
     }
