@@ -46,23 +46,26 @@ static PyObject* _Nullable find_selector(PyObject* self, const char* name,
             unbound_instance_method = 1;
         }
 
+        /* XXX: Check if PyObjCClass_GetClass can return Nil */
+        if (objc_object == nil) {
+            PyErr_Format(PyExc_AttributeError, "<nil> doesn't have attribute %s", name);
+            return NULL;
+        }
+
     } else {
-        PyObjC_Assert(PyObjCObject_Check(self), NULL);
+        assert(PyObjCObject_Check(self));
 
         /* Objective-C class methods cannot be accessed though the
          * instance, class_method will never be true
          */
-        PyObjC_Assert(!class_method, NULL);
+        assert(!class_method);
 
         class_object = (PyObject*)Py_TYPE(self);
 
         objc_object = PyObjCObject_GetObject(self);
+        assert(objc_object != nil);
     }
 
-    if (objc_object == nil) {
-        PyErr_Format(PyExc_AttributeError, "<nil> doesn't have attribute %s", name);
-        return NULL;
-    }
 
     if (strcmp(object_getClassName(objc_object), "_NSZombie") == 0) { // LCOV_BR_EXCL_LINE
         /* Impossible to hit in regular testing */
@@ -85,9 +88,9 @@ static PyObject* _Nullable find_selector(PyObject* self, const char* name,
     Py_BEGIN_ALLOW_THREADS
         @try {
             if (unbound_instance_method) {
-                methsig = [objc_object instanceMethodSignatureForSelector:sel];
+                methsig = [objc_object instanceMethodSignatureForSelector:sel]; // LCOV_BR_EXCL_LINE
             } else {
-                methsig = [objc_object methodSignatureForSelector:sel];
+                methsig = [objc_object methodSignatureForSelector:sel]; // LCOV_BR_EXCL_LINE
             }
 
         } @catch (NSObject* localException) { // LCOV_EXCL_LINE
@@ -160,9 +163,11 @@ static PyObject* _Nullable find_selector(PyObject* self, const char* name,
                     return meta;
                 } else {
                     methinfo = PyObjCSelector_GetMetadata(meta);
-                    if (methinfo == NULL) {
+                    if (methinfo == NULL) { // LCOV_BR_EXCL_LINE
+                        // LCOV_EXCL_START
                         Py_DECREF(meta);
                         return NULL;
+                        // LCOV_EXCL_STOP
                     }
                     flattened = (char*)methinfo->signature;
 
@@ -184,8 +189,10 @@ static PyObject* _Nullable find_selector(PyObject* self, const char* name,
          * the NSMethodSignature is invalid, or if the encoded
          * signature would not fit buffer.
          */
+        // LCOV_EXCL_START
         Py_CLEAR(methinfo);
-        return NULL; // LCOV_EXCL_LINE
+        return NULL;
+        // LCOV_EXCL_STOP
     }
 
     Py_CLEAR(methinfo);
@@ -209,14 +216,11 @@ static PyObject* _Nullable make_dict(PyObject* self, int class_method)
         }
 
     } else {
-        PyObjC_Assert(PyObjCObject_Check(self), NULL);
-        PyObjC_Assert(!class_method, NULL);
+        assert(PyObjCObject_Check(self));
+        assert(!class_method);
 
         id obj = PyObjCObject_GetObject(self);
-
-        if (obj == NULL) {
-            return PyDict_New();
-        }
+        assert(obj != nil);
 
         cls        = object_getClass(obj);
         objc_class = cls;
@@ -311,7 +315,7 @@ static PyObject* _Nullable make_dict(PyObject* self, int class_method)
 
             Py_CLEAR(py_name);
             Py_DECREF(v);
-        }
+        } // LCOV_BR_EXCL_LINE
 
         free(methods);
     }
@@ -359,6 +363,13 @@ methacc_traverse(PyObject* _self, visitproc visit, void* _Nullable arg)
     return 0;
 }
 
+// LCOV_EXCL_START
+// Instances of this type cannot be part of a loop: This type has
+// references to an ObjC type of instance. Instances only have
+// references to their type, and ObjC types are immortal.
+//
+// Even when an ObjC class has (Python) attributes those are stored
+// on the actual ObjC object, not on the ``PyObjCObject*`` value.
 static int
 methacc_clear(PyObject* _self)
 {
@@ -372,15 +383,16 @@ methacc_clear(PyObject* _self)
 
     return 0;
 }
+// LCOV_EXCL_STOP
 
 static PyObject* _Nullable methacc_getattro(PyObject* _self, PyObject* name)
 {
     PyObjCMethodAccessor* self   = (PyObjCMethodAccessor*)_self;
     PyObject*             result = NULL;
 
-    PyObjC_Assert(PyObjCObject_Check(self->base) || PyObjCClass_Check(self->base), NULL);
+    assert(PyObjCObject_Check(self->base) || PyObjCClass_Check(self->base));
 
-    if (PyUnicode_Check(name)) {
+    if (PyUnicode_Check(name)) { // LCOV_BR_EXCL_LINE
         if (PyObjC_Unicode_Fast_Bytes(name) == NULL) { // LCOV_BR_EXCL_LINE
             return NULL;                               // LCOV_EXCL_LINE
         }
@@ -421,7 +433,7 @@ static PyObject* _Nullable methacc_getattro(PyObject* _self, PyObject* name)
     }
 
     if (self->class_method) {
-        PyObjC_Assert(PyObjCClass_Check(self->base), NULL);
+        assert(PyObjCClass_Check(self->base));
         result = PyObject_GetAttr(self->base, name);
 
     } else {
@@ -449,9 +461,11 @@ static PyObject* _Nullable methacc_getattro(PyObject* _self, PyObject* name)
                 PyObject* v;
 
                 int r = PyDict_GetItemRef(dict, name, &v);
-                if (r == -1) {
+                if (r == -1) { // LCOV_BR_EXCL_LINE
+                    // LCOV_EXCL_START
                     Py_CLEAR(dict);
                     return NULL;
+                    // LCOV_EXCL_STOP
                 } else if (r == 1) {
                     if (PyObjCSelector_Check(v)) {
                         /* Found it, use the
@@ -519,7 +533,7 @@ static PyObject* _Nullable methacc_getattro(PyObject* _self, PyObject* name)
 
     if (!self->class_method && PyObjCClass_Check(self->base)) {
         /* Unbound instance method */
-        PyObjC_Assert(((PyObjCSelector*)result)->sel_self == NULL, NULL);
+        assert(((PyObjCSelector*)result)->sel_self == NULL);
         return result;
     } else {
         /* Bound instance or class method
@@ -602,9 +616,9 @@ static PyObject* PyObjCMethodAccessor_Type;
 PyObject* _Nullable PyObjCMethodAccessor_New(PyObject* base, int class_method)
 {
     PyObjCMethodAccessor* result;
-    PyObjC_Assert(PyObjCObject_Check(base) || PyObjCClass_Check(base), NULL);
+    assert(PyObjCObject_Check(base) || PyObjCClass_Check(base));
     if (class_method) {
-        PyObjC_Assert(PyObjCClass_Check(base), NULL);
+        assert(PyObjCClass_Check(base));
     }
 
     result =
