@@ -17,7 +17,7 @@ typedef struct {
 
 ffi_cif* _Nullable PyObjCIMP_GetCIF(PyObject* self)
 {
-    PyObjC_Assert(PyObjCIMP_Check(self), NULL);
+    assert(PyObjCIMP_Check(self));
 
     ffi_cif* result;
     Py_BEGIN_CRITICAL_SECTION(self);
@@ -30,7 +30,7 @@ ffi_cif* _Nullable PyObjCIMP_GetCIF(PyObject* self)
 int
 PyObjCIMP_SetCIF(PyObject* self, ffi_cif* _Nullable cif)
 {
-    PyObjC_Assert(PyObjCIMP_Check(self), -1);
+    assert(PyObjCIMP_Check(self));
 
     Py_BEGIN_CRITICAL_SECTION(self);
     ((PyObjCIMPObject*)self)->cif = cif;
@@ -41,14 +41,14 @@ PyObjCIMP_SetCIF(PyObject* self, ffi_cif* _Nullable cif)
 
 SEL _Nullable PyObjCIMP_GetSelector(PyObject* self)
 {
-    PyObjC_Assert(PyObjCIMP_Check(self), NULL);
+    assert(PyObjCIMP_Check(self));
 
     return ((PyObjCIMPObject*)self)->selector;
 }
 
 IMP _Nullable PyObjCIMP_GetIMP(PyObject* self)
 {
-    PyObjC_Assert(PyObjCIMP_Check(self), NULL);
+    assert(PyObjCIMP_Check(self));
 
     return ((PyObjCIMPObject*)self)->imp;
 }
@@ -56,14 +56,14 @@ IMP _Nullable PyObjCIMP_GetIMP(PyObject* self)
 int
 PyObjCIMP_GetFlags(PyObject* self)
 {
-    PyObjC_Assert(PyObjCIMP_Check(self), -1);
+    assert(PyObjCIMP_Check(self));
 
     return ((PyObjCIMPObject*)self)->flags;
 }
 
 PyObjCMethodSignature* _Nullable PyObjCIMP_GetSignature(PyObject* self)
 {
-    PyObjC_Assert(PyObjCIMP_Check(self), NULL);
+    assert(PyObjCIMP_Check(self));
 
     Py_INCREF(((PyObjCIMPObject*)self)->signature);
     return ((PyObjCIMPObject*)self)->signature;
@@ -87,9 +87,6 @@ static PyObject* _Nullable imp_vectorcall(PyObject* _self,
 {
     PyObjCIMPObject* self = (PyObjCIMPObject*)_self;
     PyObject*        pyself;
-    PyObjC_CallFunc  execute = NULL;
-    PyObject*        res;
-    PyObject*        pyres;
 
     if (PyObjC_CheckNoKwnames(_self, kwnames) == -1) {
         return NULL;
@@ -106,41 +103,13 @@ static PyObject* _Nullable imp_vectorcall(PyObject* _self,
         PyErr_SetString(PyExc_TypeError, "Missing argument: self");
         return NULL;
     }
-    PyObjC_Assert(args != NULL, NULL);
+    assert(args != NULL);
 
     pyself = args[0];
-    PyObjC_Assert(pyself != NULL, NULL);
+    assert(pyself != NULL);
+    assert(self->callfunc != NULL);
 
-    execute = self->callfunc;
-
-    pyres = res = execute((PyObject*)self, pyself, args + 1, nargsf - 1);
-
-    if (pyres != NULL && PyTuple_Check(pyres) && PyTuple_GET_SIZE(pyres) > 1
-        && PyTuple_GET_ITEM(pyres, 0) == pyself) {
-        pyres = pyself;
-    }
-
-    if (PyObjCObject_Check(pyself)
-        && (((PyObjCObject*)pyself)->flags & PyObjCObject_kUNINITIALIZED)) {
-        if (pyself != pyres && !PyErr_Occurred()) {
-            PyObjCObject_ClearObject(pyself);
-        }
-    }
-
-    if (pyres && PyObjCObject_Check(res)) {
-        if (self->flags & PyObjCSelector_kRETURNS_UNINITIALIZED) {
-            ((PyObjCObject*)pyres)->flags |= PyObjCObject_kUNINITIALIZED;
-
-        } else if (((PyObjCObject*)pyres)->flags & PyObjCObject_kUNINITIALIZED) {
-            ((PyObjCObject*)pyres)->flags &= ~PyObjCObject_kUNINITIALIZED;
-            if (pyself && pyself != pyres && PyObjCObject_Check(pyself)
-                && !PyErr_Occurred()) {
-                PyObjCObject_ClearObject(pyself);
-            }
-        }
-    }
-
-    return res;
+    return self->callfunc((PyObject*)self, pyself, args + 1, nargsf - 1);
 }
 
 #if PY_VERSION_HEX >= 0x03090000
@@ -149,12 +118,9 @@ static PyObject* _Nullable imp_vectorcall_simple(PyObject* _self,
                                                  size_t nargsf,
                                                  PyObject* _Nullable kwnames)
 {
-    PyObjCIMPObject* self = (PyObjCIMPObject*)_self;
     PyObject*        pyself;
-    PyObject*        res;
-    PyObject*        pyres;
 
-    PyObjC_Assert(self->signature->shortcut_signature, NULL);
+    assert(((PyObjCIMPObject*)_self)->signature->shortcut_signature);
 
     if (PyObjC_CheckNoKwnames(_self, kwnames) == -1) {
         return NULL;
@@ -172,39 +138,12 @@ static PyObject* _Nullable imp_vectorcall_simple(PyObject* _self,
         return NULL;
     }
 
-    PyObjC_Assert(args != NULL, NULL);
+    assert(args != NULL);
 
     pyself = args[0];
-    PyObjC_Assert(pyself != NULL, NULL);
+    assert(pyself != NULL);
 
-    pyres = res = PyObjCFFI_Caller_Simple(_self, pyself, args + 1, nargsf - 1);
-
-    if (pyres != NULL && PyTuple_Check(pyres) && PyTuple_GET_SIZE(pyres) > 1
-        && PyTuple_GET_ITEM(pyres, 0) == pyself) {
-        pyres = pyself;
-    }
-
-    if (PyObjCObject_Check(pyself)
-        && (((PyObjCObject*)pyself)->flags & PyObjCObject_kUNINITIALIZED)) {
-        if (pyself != pyres && !PyErr_Occurred()) {
-            PyObjCObject_ClearObject(pyself);
-        }
-    }
-
-    if (pyres && PyObjCObject_Check(res)) {
-        if (self->flags & PyObjCSelector_kRETURNS_UNINITIALIZED) {
-            ((PyObjCObject*)pyres)->flags |= PyObjCObject_kUNINITIALIZED;
-
-        } else if (((PyObjCObject*)pyres)->flags & PyObjCObject_kUNINITIALIZED) {
-            ((PyObjCObject*)pyres)->flags &= ~PyObjCObject_kUNINITIALIZED;
-            if (pyself && pyself != pyres && PyObjCObject_Check(pyself)
-                && !PyErr_Occurred()) {
-                PyObjCObject_ClearObject(pyself);
-            }
-        }
-    }
-
-    return res;
+    return PyObjCFFI_Caller_Simple(_self, pyself, args + 1, nargsf - 1);
 }
 #endif
 
@@ -247,8 +186,8 @@ static PyObject* _Nullable imp_signature(PyObject* _self,
                                          void*     closure __attribute__((__unused__)))
 {
     PyObjCIMPObject* self = (PyObjCIMPObject*)_self;
-    PyObjC_Assert(self->signature != NULL, NULL);
-    PyObjC_Assert(self->signature->signature != NULL, NULL);
+    assert(self->signature != NULL);
+    assert(self->signature->signature != NULL);
     return PyBytes_FromString(self->signature->signature);
 }
 
@@ -280,15 +219,14 @@ PyDoc_STRVAR(
     "True if this is method returns a a freshly allocated object (uninitialized)\n"
     "\n"
     "NOTE: This field is used by the implementation.");
-static PyObject*
-imp_is_alloc(PyObject* _self, void* closure __attribute__((__unused__)))
+static PyObject* _Nullable
+imp_is_alloc(PyObject* self __attribute__((__unused__)), void* closure __attribute__((__unused__)))
 {
-    PyObjCIMPObject* self = (PyObjCIMPObject*)_self;
-    if ((self->flags & PyObjCSelector_kRETURNS_UNINITIALIZED) != 0) {
-        Py_RETURN_TRUE;
-    } else {
-        Py_RETURN_FALSE;
+    if (PyErr_Warn(PyObjCExc_DeprecationWarning, "isAlloc is always false") < 0) {
+        return NULL;
     }
+
+    Py_RETURN_FALSE;
 }
 
 static PyGetSetDef imp_getset[] = {{
@@ -346,14 +284,6 @@ static PyObject* _Nullable imp_metadata(PyObject* self)
         return NULL;       // LCOV_EXCL_LINE
     }
 
-    if (((PyObjCIMPObject*)self)->flags & PyObjCSelector_kRETURNS_UNINITIALIZED) {
-        r = PyDict_SetItem(result, PyObjCNM_return_unitialized_object, Py_True);
-        if (r == -1) {         // LCOV_BR_EXCL_LINE
-            Py_DECREF(result); // LCOV_EXCL_LINE
-            return NULL;       // LCOV_EXCL_LINE
-        }
-    }
-
     return result;
 }
 
@@ -421,8 +351,8 @@ static PyObject* _Nullable PyObjCIMP_New(IMP imp, SEL selector, PyObjC_CallFunc
 {
     PyObjCIMPObject* result;
 
-    PyObjC_Assert(callfunc != NULL, NULL);
-    PyObjC_Assert(signature != NULL, NULL);
+    assert(callfunc != NULL);
+    assert(signature != NULL);
 
     result = PyObject_New(PyObjCIMPObject, (PyTypeObject*)PyObjCIMP_Type);
     if (result == NULL) // LCOV_BR_EXCL_LINE
@@ -438,8 +368,8 @@ static PyObject* _Nullable PyObjCIMP_New(IMP imp, SEL selector, PyObjC_CallFunc
     result->flags = flags;
 
 #if PY_VERSION_HEX >= 0x03090000
-    if (signature && signature->shortcut_signature && (callfunc == PyObjCFFI_Caller)) {
-        PyObjC_Assert(signature->shortcut_signature, NULL);
+    if (signature && signature->shortcut_signature && (callfunc == PyObjCFFI_Caller)) { // LCOV_BR_EXCL_LINE
+        assert(signature->shortcut_signature);
         result->vectorcall = imp_vectorcall_simple;
     } else {
         result->vectorcall = imp_vectorcall;
@@ -462,7 +392,7 @@ static PyObject* _Nullable call_instanceMethodForSelector_(
     if (PyObjC_CheckArgCount(method, 1, 1, nargs) == -1)
         return NULL;
 
-    PyObjC_Assert(args != NULL, NULL);
+    assert(args != NULL);
 
     sel = args[0];
 
@@ -470,30 +400,38 @@ static PyObject* _Nullable call_instanceMethodForSelector_(
         return NULL;
     }
 
-    if (!PyObjCClass_Check(self)) {
+    if (!PyObjCClass_Check(self)) { // LCOV_BR_EXCL_LINE
+        /* AFAIK it is not possible to get an unbound objc.selector
+         * for a class method.
+         */
+        // LCOV_EXCL_START
         PyErr_Format(PyExc_TypeError,
                      "Expecting instance of 'objc.objc_class' as 'self', "
                      "got '%s'",
                      Py_TYPE(self)->tp_name);
         return NULL;
+        // LCOV_EXCL_STOP
     }
 
     Py_BEGIN_ALLOW_THREADS
         @try {
-            retval = ((IMP(*)(Class, SEL, SEL))objc_msgSend)(
-                PyObjCClass_GetClass(self), PyObjCSelector_GetSelector(method), selector);
+            retval = ((IMP(*)(Class, SEL, SEL))objc_msgSend)( // LCOV_BR_EXCL_LINE
+                PyObjCClass_GetClass(self), PyObjCSelector_GetSelector(method), selector); // LCOV_BR_EXCL_LINE
 
-        } @catch (NSObject* localException) {
-            PyObjCErr_FromObjC(localException);
-            retval = NULL;
+        } @catch (NSObject* localException) { // LCOV_EXCL_LINE
+            PyObjCErr_FromObjC(localException); // LCOV_EXCL_LINE
+            retval = NULL; // LCOV_EXCL_LINE
         }
     Py_END_ALLOW_THREADS
 
-    if (retval == NULL) {
+    if (retval == NULL) { // LCOV_BR_EXCL_LINE
+        /* AFAIK the method in practice never returns NULL */
+        // LCOV_EXCL_START
         if (PyErr_Occurred()) {
             return NULL;
         }
         Py_RETURN_NONE;
+        // LCOV_EXCL_STOP
     }
 
     attr = PyObjCClass_FindSelector(self, selector, NO);
@@ -523,8 +461,8 @@ static PyObject* _Nullable call_instanceMethodForSelector_(
     }
 
     PyObjCMethodSignature* methinfo = PyObjCSelector_GetMetadata(attr);
-    if (methinfo == NULL) {
-        return NULL;
+    if (methinfo == NULL) { // LCOV_BR_EXCL_LINE
+        return NULL; // LCOV_EXCL_LINE
     }
 
     res = PyObjCIMP_New(retval, selector, ((PyObjCNativeSelector*)attr)->sel_call_func,
@@ -548,7 +486,7 @@ static PyObject* _Nullable call_methodForSelector_(PyObject* method, PyObject* s
     if (PyObjC_CheckArgCount(method, 1, 1, nargs) == -1)
         return NULL;
 
-    PyObjC_Assert(args != NULL, NULL);
+    assert(args != NULL);
 
     sel = args[0];
 
@@ -560,26 +498,28 @@ static PyObject* _Nullable call_methodForSelector_(PyObject* method, PyObject* s
         super.receiver = (Class _Nonnull)PyObjCClass_GetClass(self);
 
     } else {
-        super.receiver = (id _Nonnull)PyObjCObject_GetObject(self);
+        super.receiver = PyObjCObject_GetObject(self);
     }
     super.super_class = (Class _Nonnull)object_getClass(super.receiver);
 
     Py_BEGIN_ALLOW_THREADS
         @try {
-            retval = ((IMP(*)(struct objc_super*, SEL, SEL))objc_msgSendSuper)(
-                &super, PyObjCSelector_GetSelector(method), selector);
+            retval = ((IMP(*)(struct objc_super*, SEL, SEL))objc_msgSendSuper)( // LCOV_BR_EXCL_LINE
+                &super, PyObjCSelector_GetSelector(method), selector); // LCOV_BR_EXCL_LINE
 
-        } @catch (NSObject* localException) {
-            PyObjCErr_FromObjC(localException);
-            retval = NULL;
+        } @catch (NSObject* localException) { // LCOV_EXCL_LINE
+            PyObjCErr_FromObjC(localException); // LCOV_EXCL_LINE
+            retval = NULL; // LCOV_EXCL_LINE
         }
     Py_END_ALLOW_THREADS
 
-    if (retval == NULL) {
+    if (retval == NULL) { // LCOV_BR_EXCL_LINE
+        // LCOV_EXCL_START
         if (PyErr_Occurred()) {
             return NULL;
         }
         Py_RETURN_NONE;
+        // LCOV_EXCL_STOP
     }
 
     if (PyObjCClass_Check(self)) {
@@ -613,9 +553,11 @@ static PyObject* _Nullable call_methodForSelector_(PyObject* method, PyObject* s
     }
 
     PyObjCMethodSignature* methinfo = PyObjCSelector_GetMetadata(attr);
-    if (methinfo == NULL) {
+    if (methinfo == NULL) { // LCOV_BR_EXCL_LINE
+        // LCOV_EXCL_START
         Py_DECREF(attr);
         return NULL;
+        // LCOV_EXCL_STOP
     }
 
     res = PyObjCIMP_New(retval, selector, ((PyObjCNativeSelector*)attr)->sel_call_func,
