@@ -146,8 +146,8 @@ sig_dealloc(PyObject* _self)
     }
 
     for (i = 0; i < Py_SIZE(self); i++) {
-        if (self->argtype[i] == NULL)
-            continue;
+        if (self->argtype[i] == NULL) // LCOV_BR_EXCL_LINE
+            continue; // LCOV_EXCL_LINE
         if (self->argtype[i]->tmpl)
             continue;
 
@@ -222,7 +222,7 @@ determine_if_shortcut(PyObjCMethodSignature* methinfo)
     /* TODO: simple pass-by-reference objects args should work (NSError** arguments) */
     /* FIXME: structs/unions/... also use byref */
 
-    PyObjC_Assert(methinfo, -1);
+    assert(methinfo);
     methinfo->shortcut_signature   = NO;
     methinfo->shortcut_argbuf_size = 0;
     methinfo->shortcut_result_size = 0;
@@ -243,10 +243,7 @@ determine_if_shortcut(PyObjCMethodSignature* methinfo)
         return 0;
     }
 
-#ifdef PyObjC_DEBUG
-    if (PyObjCMethodSignature_Validate(methinfo) == -1) // LCOV_BR_EXCL_LINE
-        return -1;                                      // LCOV_EXCL_LINE
-#endif /* PyObjC_DEBUG */
+    PyObjCMethodSignature_Validate(methinfo);
 
     for (Py_ssize_t i = 0; i < Py_SIZE(methinfo); i++) {
         switch (*methinfo->argtype[i]->type) {
@@ -282,9 +279,11 @@ determine_if_shortcut(PyObjCMethodSignature* methinfo)
     }
 
     Py_ssize_t result_size = PyObjCRT_SizeOfReturnType(methinfo->rettype->type);
-    if (result_size == -1) {
+    if (result_size == -1) { // LCOV_BR_EXCL_LINE
+        // LCOV_EXCL_START
         PyErr_Clear();
         return 0;
+        // LCOV_EXCL_STOP
     }
     if (result_size > 128) {
         return 0;
@@ -426,7 +425,7 @@ static PyObjCMethodSignature* _Nullable new_methodsignature(const char* signatur
     const char*            cur;
     PyObjCMethodSignature* retval;
 
-    PyObjC_Assert(signature != NULL, NULL);
+    assert(signature != NULL);
 
     /* Skip return-type */
     cur = PyObjCRT_SkipTypeSpec(signature);
@@ -484,6 +483,7 @@ static PyObjCMethodSignature* _Nullable new_methodsignature(const char* signatur
     retval->variadic              = NO;
     retval->null_terminated_array = NO;
     retval->free_result           = NO;
+    retval->initializer           = NO;
     retval->shortcut_signature    = NO;
     retval->shortcut_argbuf_size  = 0;
     retval->shortcut_result_size  = 0;
@@ -492,7 +492,7 @@ static PyObjCMethodSignature* _Nullable new_methodsignature(const char* signatur
     retval->rettype               = (struct _PyObjC_ArgDescr* _Nonnull)NULL;
 
     cur = PyObjCRT_SkipTypeQualifiers(retval->signature);
-    PyObjC_Assert(cur != NULL, NULL);
+    assert(cur != NULL);
     if (unlikely(cur[0] == _C_ID && cur[1] == _C_UNDEF)) {
         retval->rettype = (__typeof__(retval->rettype))&block_template;
     } else if (unlikely(cur[0] == _C_PTR)) {
@@ -528,14 +528,16 @@ static PyObjCMethodSignature* _Nullable new_methodsignature(const char* signatur
         /* Ignore type specifiers for methods returning void. Mostly needed
          * to avoid crapping out one (oneway void) methods.
          */
-        PyObjC_Assert(retval->signature != NULL, NULL);
-        if (setup_type(retval->rettype, cur) < 0) {
+        assert(retval->signature != NULL);
+        if (setup_type(retval->rettype, cur) < 0) { // LCOV_BR_EXCL_LINE
+            // LCOV_EXCL_START
             Py_DECREF(retval);
             return NULL;
+            // LCOV_EXCL_STOP
         }
-        PyObjC_Assert(retval->rettype->type != NULL, NULL);
+        assert(retval->rettype->type != NULL);
     }
-    PyObjC_Assert(retval->rettype->type != NULL, NULL);
+    assert(retval->rettype->type != NULL);
 
     cur = PyObjCRT_SkipTypeSpec(retval->signature);
     if (cur && *cur == '"') {
@@ -568,12 +570,14 @@ static PyObjCMethodSignature* _Nullable new_methodsignature(const char* signatur
                 return NULL;
                 // LCOV_EXCL_STOP
             }
-            PyObjC_Assert(cur != NULL, NULL);
-            if (setup_type(retval->argtype[nargs], cur) < 0) {
+            assert(cur != NULL);
+            if (setup_type(retval->argtype[nargs], cur) < 0) { // LCOV_BR_EXCL_LINE
+                // LCOV_EXCL_START
                 Py_DECREF(retval);
                 return NULL;
+                // LCOV_EXCL_STOP
             }
-            PyObjC_Assert(retval->argtype[nargs]->type != NULL, NULL);
+            assert(retval->argtype[nargs]->type != NULL);
         }
 
         cur = PyObjCRT_SkipTypeSpec(cur);
@@ -589,11 +593,8 @@ static PyObjCMethodSignature* _Nullable new_methodsignature(const char* signatur
         nargs++;
     }
 
-#ifdef PyObjC_DEBUG
-    PyObjC_Assert(Py_SIZE(retval) == nargs, NULL);
-    if (PyObjCMethodSignature_Validate(retval) == -1) // LCOV_BR_EXCL_LINE
-        return NULL;                                  // LCOV_EXCL_LINE
-#endif                                                /* PyObjC_DEBUG */
+    assert(Py_SIZE(retval) == nargs);
+    PyObjCMethodSignature_Validate(retval);
 
     if (determine_if_shortcut(retval) < 0) { // LCOV_BR_EXCL_LINE
         // LCOV_EXCL_START
@@ -609,7 +610,6 @@ char* _Nullable PyObjC_NSMethodSignatureToTypeString(NSMethodSignature* sig, cha
                                                      size_t buflen)
 {
     char*      result = buf;
-    char*      end;
     NSUInteger arg_count = [sig numberOfArguments];
     NSUInteger i;
     size_t     r;
@@ -620,29 +620,12 @@ char* _Nullable PyObjC_NSMethodSignatureToTypeString(NSMethodSignature* sig, cha
         return NULL;
     }
 
-    /* Using SkipTypeSpec validates the buffer */
-    end = (char*)PyObjCRT_SkipTypeSpec(buf);
-    if (end == NULL) {
-        return NULL;
-    }
-    *end = '\0';
-    buflen -= (end - buf);
-    buf = end;
-
     for (i = 0; i < arg_count; i++) {
-        r = strlcpy(buf, [sig getArgumentTypeAtIndex:i], buflen);
+        r = strlcat(buf, [sig getArgumentTypeAtIndex:i], buflen);
         if (r >= buflen) {
             PyErr_Format(PyObjCExc_InternalError, "NSMethodsignature too large (%ld)", r);
             return NULL;
         }
-
-        /* Using SkipTypeSpec validates the buffer */
-        end = (char*)PyObjCRT_SkipTypeSpec(buf);
-        if (end == NULL) {
-            return NULL;
-        }
-        buflen -= (end - buf);
-        buf = end;
     }
 
     return result;
@@ -661,6 +644,7 @@ static int
 setup_descr(struct _PyObjC_ArgDescr* descr, PyObject* _Nullable meta, BOOL is_native)
 {
     PyObject* d;
+    int r;
     char      typeModifier = 0;
 
     if (meta == Py_None) {
@@ -668,28 +652,26 @@ setup_descr(struct _PyObjC_ArgDescr* descr, PyObject* _Nullable meta, BOOL is_na
     }
 
     if (meta != NULL && !PyDict_Check(meta)) {
-        PyObject* r = PyObject_Repr(meta);
-        if (r == NULL) {
-            return -1;
-        }
-
-        PyErr_Format(PyExc_TypeError, "metadata of type %s: %U", Py_TYPE(meta)->tp_name,
-                     r);
-        Py_DECREF(r);
+        PyErr_Format(PyExc_TypeError, "metadata of type %s: %R", Py_TYPE(meta)->tp_name,
+                     meta);
 
         return -1;
     }
 
-    PyObjC_Assert(meta == NULL || PyDict_Check(meta), -1);
+    assert(meta == NULL || PyDict_Check(meta));
+    assert(descr == NULL || descr->allowNULL);
 
-    PyObjC_Assert(descr == NULL || descr->allowNULL, -1);
     if (meta) {
-        switch (PyDict_GetItemRef(meta, PyObjCNM_null_accepted, &d)) {
+        switch (PyDict_GetItemRef(meta, PyObjCNM_null_accepted, &d)) { // LCOV_BR_EXCL_LINE
         case -1:
-            return -1;
+            return -1; // LCOV_EXCL_LINE
         /* case 0: pass */
        case 1:
-            if (!PyObject_IsTrue(d)) {
+            r = PyObject_IsTrue(d);
+            if (r == -1) {
+                return -1;
+            }
+            if (!r) {
                 if (descr == NULL || descr->tmpl)  {
                     Py_DECREF(d);
                     return -2;
@@ -701,19 +683,23 @@ setup_descr(struct _PyObjC_ArgDescr* descr, PyObject* _Nullable meta, BOOL is_na
     }
 
     if (meta) {
-        switch (PyDict_GetItemRef(meta, PyObjCNM_already_retained, &d)) {
+        switch (PyDict_GetItemRef(meta, PyObjCNM_already_retained, &d)) { // LCOV_BR_EXCL_LINE
         case -1:
-            return -1;
+            return -1; // LCOV_EXCL_LINE
         case 0:
             if (descr == NULL || (descr->tmpl && descr->alreadyRetained))
                 return -2;
             // descr may be loaded into read-only memory, so only
             // write if truly necessary
-            if (descr->alreadyRetained)
-                descr->alreadyRetained = NO;
+            if (descr->alreadyRetained) // LCOV_BR_EXCL_LINE
+                descr->alreadyRetained = NO; // LCOV_EXCL_LINE
             break;
         case 1:
-            if (PyObject_IsTrue(d)) {
+            r = PyObject_IsTrue(d);
+            if (r == -1) {
+                return -1;
+            }
+            if (r) {
                 if (descr == NULL || (descr->tmpl && !descr->alreadyRetained)) {
                     Py_DECREF(d);
                     return -2;
@@ -728,15 +714,19 @@ setup_descr(struct _PyObjC_ArgDescr* descr, PyObject* _Nullable meta, BOOL is_na
         }
     }
 
-    PyObjC_Assert(descr == NULL || !descr->alreadyCFRetained, -1);
+    assert(descr == NULL || !descr->alreadyCFRetained);
     if (meta) {
-        switch (PyDict_GetItemRef(meta, PyObjCNM_already_cfretained, &d)) {
+        switch (PyDict_GetItemRef(meta, PyObjCNM_already_cfretained, &d)) { // LCOV_BR_EXCL_LINE
         case -1:
             return -1;                       // LCOV_EXCL_LINE
 
         /* case 0: pass */
         case 1:
-            if (PyObject_IsTrue(d)) {
+            r = PyObject_IsTrue(d);
+            if (r == -1) {
+                return -1;
+            }
+            if (r) {
                 if (descr == NULL || descr->tmpl) {
                     Py_DECREF(d);
                     return -2;
@@ -747,14 +737,18 @@ setup_descr(struct _PyObjC_ArgDescr* descr, PyObject* _Nullable meta, BOOL is_na
         }
     }
 
-    PyObjC_Assert(descr == NULL || !descr->callableRetained, -1);
+    assert(descr == NULL || !descr->callableRetained);
     if (meta) {
-        switch (PyDict_GetItemRef(meta, PyObjCNM_callable_retained, &d)) {
+        switch (PyDict_GetItemRef(meta, PyObjCNM_callable_retained, &d)) { // LCOV_BR_EXCL_LINE
         case -1:
             return -1;                       // LCOV_EXCL_LINE
         /* case 0: pass */
        case 1:
-            if (PyObject_IsTrue(d)) {
+            r = PyObject_IsTrue(d);
+            if (r == -1) {
+                return -1;
+            }
+            if (r) {
                 if (descr == NULL || descr->tmpl) {
                     Py_DECREF(d);
                     return -2;
@@ -765,9 +759,9 @@ setup_descr(struct _PyObjC_ArgDescr* descr, PyObject* _Nullable meta, BOOL is_na
         }
     }
 
-    PyObjC_Assert(descr == NULL || descr->sel_type == NULL, -1);
+    assert(descr == NULL || descr->sel_type == NULL);
     if (meta) {
-        switch (PyDict_GetItemRef(meta, PyObjCNM_sel_of_type, &d)) {
+        switch (PyDict_GetItemRef(meta, PyObjCNM_sel_of_type, &d)) { // LCOV_BR_EXCL_LINE
         case -1:
             return -1;                       // LCOV_EXCL_LINE
         /* case 0: pass */
@@ -809,7 +803,7 @@ setup_descr(struct _PyObjC_ArgDescr* descr, PyObject* _Nullable meta, BOOL is_na
     }
 
     if (meta) {
-        switch (PyDict_GetItemRef(meta, PyObjCNM_callable, &d)) {
+        switch (PyDict_GetItemRef(meta, PyObjCNM_callable, &d)) { // LCOV_BR_EXCL_LINE
         case -1:
             return -1;                       // LCOV_EXCL_LINE
         /* case 0: pass */
@@ -826,7 +820,7 @@ setup_descr(struct _PyObjC_ArgDescr* descr, PyObject* _Nullable meta, BOOL is_na
             PyObject* a = NULL;
             Py_ssize_t i, len;
 
-            switch (PyDict_GetItemRef(d, PyObjCNM_arguments, &a)) {
+            switch (PyDict_GetItemRef(d, PyObjCNM_arguments, &a)) { // LCOV_BR_EXCL_LINE
             case -1:
                 Py_DECREF(d);
                 return -1;                       // LCOV_EXCL_LINE
@@ -866,14 +860,18 @@ setup_descr(struct _PyObjC_ArgDescr* descr, PyObject* _Nullable meta, BOOL is_na
         }
     }
 
-    PyObjC_Assert(descr == NULL || !descr->arraySizeInRetval, -1);
+    assert(descr == NULL || !descr->arraySizeInRetval);
     if (meta) {
-        switch (PyDict_GetItemRef(meta, PyObjCNM_c_array_length_in_result, &d)) {
+        switch (PyDict_GetItemRef(meta, PyObjCNM_c_array_length_in_result, &d)) { // LCOV_BR_EXCL_LINE
         case -1:
             return -1;                       // LCOV_EXCL_LINE
         /* case 0: pass */
         case 1:
-            if (PyObject_IsTrue(d)) {
+            r = PyObject_IsTrue(d);
+            if (r == -1) {
+                return -1;
+            }
+            if (r) {
                 if (descr == NULL || descr->tmpl) {
                     Py_DECREF(d);
                     return -2;
@@ -885,14 +883,18 @@ setup_descr(struct _PyObjC_ArgDescr* descr, PyObject* _Nullable meta, BOOL is_na
         }
     }
 
-    PyObjC_Assert(descr == NULL || !descr->printfFormat, -1);
+    assert(descr == NULL || !descr->printfFormat);
     if (meta) {
-        switch (PyDict_GetItemRef(meta, PyObjCNM_printf_format, &d)) {
+        switch (PyDict_GetItemRef(meta, PyObjCNM_printf_format, &d)) { // LCOV_BR_EXCL_LINE
         case -1:
             return -1;                       // LCOV_EXCL_LINE
         /*case 0: pass */
         case 1:
-            if (PyObject_IsTrue(d)) {
+            r = PyObject_IsTrue(d);
+            if (r == -1) {
+                return -1;
+            }
+            if (r) {
                 if (descr == NULL || descr->tmpl) {
                     Py_DECREF(d);
                     return -2;
@@ -905,12 +907,16 @@ setup_descr(struct _PyObjC_ArgDescr* descr, PyObject* _Nullable meta, BOOL is_na
     }
 
     if (meta) {
-        switch (PyDict_GetItemRef(meta, PyObjCNM_c_array_delimited_by_null, &d)) {
+        switch (PyDict_GetItemRef(meta, PyObjCNM_c_array_delimited_by_null, &d)) { // LCOV_BR_EXCL_LINE
         case -1:
             return -1;                       // LCOV_EXCL_LINE
         /* case 0: pass */
         case 1:
-            if (PyObject_IsTrue(d)) {
+            r = PyObject_IsTrue(d);
+            if (r == -1) {
+                return -1;
+            }
+            if (r) {
                 if (descr == NULL || descr->tmpl) {
                     Py_DECREF(d);
                     return -2;
@@ -923,7 +929,7 @@ setup_descr(struct _PyObjC_ArgDescr* descr, PyObject* _Nullable meta, BOOL is_na
     }
 
     if (meta) {
-        switch (PyDict_GetItemRef(meta, PyObjCNM_c_array_of_fixed_length, &d)) {
+        switch (PyDict_GetItemRef(meta, PyObjCNM_c_array_of_fixed_length, &d)) { // LCOV_BR_EXCL_LINE
         case -1:
             return -1;                       // LCOV_EXCL_LINE
         /* case 0: pass */
@@ -947,12 +953,16 @@ setup_descr(struct _PyObjC_ArgDescr* descr, PyObject* _Nullable meta, BOOL is_na
     }
 
     if (meta) {
-        switch (PyDict_GetItemRef(meta, PyObjCNM_c_array_of_variable_length, &d)) {
+        switch (PyDict_GetItemRef(meta, PyObjCNM_c_array_of_variable_length, &d)) { // LCOV_BR_EXCL_LINE
         case -1:
             return -1;                       // LCOV_EXCL_LINE
         /* case 0: pass */
         case 1:
-            if (PyObject_IsTrue(d)) {
+            r = PyObject_IsTrue(d);
+            if (r == -1) {
+                return -1;
+            }
+            if (r) {
                 if (descr == NULL || descr->tmpl) {
                     Py_DECREF(d);
                     return -2;
@@ -967,12 +977,16 @@ setup_descr(struct _PyObjC_ArgDescr* descr, PyObject* _Nullable meta, BOOL is_na
     }
 
     if (meta) {
-        switch (PyDict_GetItemRef(meta, PyObjCNM_deref_result_pointer, &d)) {
+        switch (PyDict_GetItemRef(meta, PyObjCNM_deref_result_pointer, &d)) { // LCOV_BR_EXCL_LINE
         case -1:
             return -1;                       // LCOV_EXCL_LINE
         /* case 0: pass */
         case 1:
-            if (PyObject_IsTrue(d)) {
+            r = PyObject_IsTrue(d);
+            if (r == -1) {
+                return -1;
+            }
+            if (r) {
                 if (descr == NULL || descr->tmpl) {
                     Py_DECREF(d);
                     return -2;
@@ -987,7 +1001,7 @@ setup_descr(struct _PyObjC_ArgDescr* descr, PyObject* _Nullable meta, BOOL is_na
     }
 
     if (meta) {
-        switch (PyDict_GetItemRef(meta, PyObjCNM_c_array_length_in_arg, &d)) {
+        switch (PyDict_GetItemRef(meta, PyObjCNM_c_array_length_in_arg, &d)) { // LCOV_BR_EXCL_LINE
         case -1:
             return -1;                       // LCOV_EXCL_LINE
         /* case 0: pass */
@@ -1061,7 +1075,7 @@ setup_descr(struct _PyObjC_ArgDescr* descr, PyObject* _Nullable meta, BOOL is_na
     }
 
     if (meta) {
-        switch (PyDict_GetItemRef(meta, PyObjCNM_type_modifier, &d)) {
+        switch (PyDict_GetItemRef(meta, PyObjCNM_type_modifier, &d)) { // LCOV_BR_EXCL_LINE
         case -1:
             return -1;                       // LCOV_EXCL_LINE
         /* case 0: pass */
@@ -1080,7 +1094,7 @@ setup_descr(struct _PyObjC_ArgDescr* descr, PyObject* _Nullable meta, BOOL is_na
                 }
 
                 typeModifier = *PyBytes_AsString(bytes);
-                PyObjC_Assert(!PyErr_Occurred(), -1);
+                assert(!PyErr_Occurred());
                 Py_CLEAR(bytes);
             } else if (PyBytes_Check(d)) {
                 if (descr == NULL || descr->tmpl) {
@@ -1089,14 +1103,14 @@ setup_descr(struct _PyObjC_ArgDescr* descr, PyObject* _Nullable meta, BOOL is_na
                 }
 
                 typeModifier = *PyBytes_AsString(d);
-                PyObjC_Assert(!PyErr_Occurred(), -1);
+                assert(!PyErr_Occurred());
             }
             Py_CLEAR(d);
         }
     }
 
     if (meta) {
-        switch (PyDict_GetItemRef(meta, PyObjCNM_type, &d)) {
+        switch (PyDict_GetItemRef(meta, PyObjCNM_type, &d)) { // LCOV_BR_EXCL_LINE
         case -1:
             return -1;                       // LCOV_EXCL_LINE
         /* case 0: pass */
@@ -1130,15 +1144,15 @@ setup_descr(struct _PyObjC_ArgDescr* descr, PyObject* _Nullable meta, BOOL is_na
             d = NULL;
 
         } else {
+            PyErr_Format(PyExc_TypeError, "key %R value %R is not a (byte) string", PyObjCNM_type, d);
             Py_XDECREF(d);
-            PyObjC_Assert(0, -1);
             return -1;
         }
 
         const char* type = PyBytes_AsString(bytes);
 
         /* XXX: This assertion is not really useful and needs to be more clear */
-        PyObjC_Assert(!is_native || descr->type != NULL, -1);
+        assert(!is_native || descr->type != NULL);
 
         if (is_native && !PyObjC_signatures_compatible(descr->type, type)) {
             /* The new signature is not compatible enough, ignore the
@@ -1159,7 +1173,6 @@ setup_descr(struct _PyObjC_ArgDescr* descr, PyObject* _Nullable meta, BOOL is_na
             // LCOV_EXCL_STOP
         }
 
-        /*PyObjC_Assert(*withoutModifiers != _C_ARY_B, -1);*/
         if (typeModifier != '\0') {
             /* Skip existing modifiers, we're overriding those */
             strlcpy(tp + 1, withoutModifiers, bufsize-1);
@@ -1167,7 +1180,7 @@ setup_descr(struct _PyObjC_ArgDescr* descr, PyObject* _Nullable meta, BOOL is_na
         } else {
             strlcpy(tp, type, bufsize);
         }
-        PyObjC_Assert(tp != NULL, -1);
+        assert(tp != NULL);
         descr->typeOverride = YES;
         descr->type         = tp;
         Py_XDECREF(bytes);
@@ -1184,7 +1197,7 @@ setup_descr(struct _PyObjC_ArgDescr* descr, PyObject* _Nullable meta, BOOL is_na
     } else if (descr != NULL && descr->type != NULL) {
         /* XXX: Is this case still needed? */
         const char* withoutModifiers = PyObjCRT_SkipTypeQualifiers(descr->type);
-        PyObjC_Assert(*withoutModifiers != _C_ARY_B, -1);
+        assert(*withoutModifiers != _C_ARY_B);
         if (descr->type[0] == _C_PTR && descr->type[1] == _C_VOID
             && descr->ptrType == PyObjC_kPointerPlain) {
 
@@ -1227,6 +1240,7 @@ process_metadata_dict(PyObjCMethodSignature* methinfo, PyObject* _Nullable metad
                       BOOL                   is_native)
 {
     PyObject* v;
+    int r;
 
     if (metadata != NULL && !PyDict_Check(metadata)) {
         PyErr_Format(PyExc_TypeError,
@@ -1240,7 +1254,7 @@ process_metadata_dict(PyObjCMethodSignature* methinfo, PyObject* _Nullable metad
         PyObject* av;
         int r;
 
-        switch (PyDict_GetItemRef(metadata, PyObjCNM_retval, &retval)) {
+        switch (PyDict_GetItemRef(metadata, PyObjCNM_retval, &retval)) { // LCOV_BR_EXCL_LINE
         case -1:
             return -1;                            // LCOV_EXCL_LINE
         /* case 0: pass */
@@ -1261,16 +1275,22 @@ process_metadata_dict(PyObjCMethodSignature* methinfo, PyObject* _Nullable metad
                     Py_DECREF(retval);
                     return -1;
                 }
-                PyObjC_Assert(r != -2, -1);
+                assert(r != -2);
             }
 
-            switch (PyDict_GetItemRef(metadata, PyObjCNM_free_result, &av)) {
+            switch (PyDict_GetItemRef(metadata, PyObjCNM_free_result, &av)) { // LCOV_BR_EXCL_LINE
             case -1:
+                // LCOV_EXCL_START
                 Py_DECREF(retval);
-                return -1;                        // LCOV_EXCL_LINE
+                return -1;
+                // LCOV_EXCL_STOP
             /* case 0: pass */
             case 1:
-                if (PyObject_IsTrue(av)) {
+                r = PyObject_IsTrue(av);
+                if (r == -1) {
+                    return -1;
+                }
+                if (r) {
                     methinfo->free_result = YES;
                 }
                 Py_CLEAR(av);
@@ -1282,7 +1302,7 @@ process_metadata_dict(PyObjCMethodSignature* methinfo, PyObject* _Nullable metad
     if (metadata) {
         PyObject* args;
 
-        switch (PyDict_GetItemRef(metadata, PyObjCNM_arguments, &args)) {
+        switch (PyDict_GetItemRef(metadata, PyObjCNM_arguments, &args)) { // LCOV_BR_EXCL_LINE
         case -1:
             return -1;                          // LCOV_EXCL_LINE
         /* case 0: pass */
@@ -1298,7 +1318,7 @@ process_metadata_dict(PyObjCMethodSignature* methinfo, PyObject* _Nullable metad
                 int       r;
 
                 if (args) {
-                    switch(PyDict_GetItemRef(args, k, &d)) {
+                    switch(PyDict_GetItemRef(args, k, &d)) { // LCOV_BR_EXCL_LINE
                     case -1:
                         // LCOV_EXCL_START
                         Py_DECREF(k);
@@ -1312,13 +1332,10 @@ process_metadata_dict(PyObjCMethodSignature* methinfo, PyObject* _Nullable metad
 
                 } else {
                     /* No metadata, hence no need to call setup_descr */
-                    PyObjC_Assert(methinfo->argtype[i] == NULL, -1);
+                    assert(methinfo->argtype[i] == NULL);
                     continue;
                 }
 
-                // PyObjC_Assert(
-                //     methinfo->argtype[i] == NULL || methinfo->argtype[i]->allowNULL,
-                //     -1);
                 r = setup_descr(methinfo->argtype[i], d, is_native);
                 if (r == -1) {
                     Py_XDECREF(d);
@@ -1341,14 +1358,14 @@ process_metadata_dict(PyObjCMethodSignature* methinfo, PyObject* _Nullable metad
                         Py_DECREF(args);
                         return -1;
                     }
-                    PyObjC_Assert(r != -2, -1);
+                    assert(r != -2);
                 }
                 Py_CLEAR(d);
             }
         }
         Py_CLEAR(args);
 
-        switch (PyDict_GetItemRef(metadata, PyObjCNM_suggestion, &v)) {
+        switch (PyDict_GetItemRef(metadata, PyObjCNM_suggestion, &v)) { // LCOV_BR_EXCL_LINE
         case -1:
             return -1;                       // LCOV_EXCL_LINE
         /* case 0: pass */
@@ -1357,7 +1374,7 @@ process_metadata_dict(PyObjCMethodSignature* methinfo, PyObject* _Nullable metad
             v = NULL;
         }
 
-        switch (PyDict_GetItemRef(metadata, PyObjCNM_deprecated, &v)) {
+        switch (PyDict_GetItemRef(metadata, PyObjCNM_deprecated, &v)) { // LCOV_BR_EXCL_LINE
         case -1:;
             return -1;                       // LCOV_EXCL_LINE
         /* case 0: pass */
@@ -1372,19 +1389,23 @@ process_metadata_dict(PyObjCMethodSignature* methinfo, PyObject* _Nullable metad
         }
 
         methinfo->null_terminated_array = NO;
-        switch(PyDict_GetItemRef(metadata, PyObjCNM_c_array_delimited_by_null, &v)) {
+        switch(PyDict_GetItemRef(metadata, PyObjCNM_c_array_delimited_by_null, &v)) { // LCOV_BR_EXCL_LINE
         case -1:
             return -1;                       // LCOV_EXCL_LINE
         /*case 0: pass */
         case 1:
-            if (PyObject_IsTrue(v)) {
+            r = PyObject_IsTrue(v);
+            if (r == -1) {
+                return -1;
+            }
+            if (r) {
                 methinfo->null_terminated_array = YES;
             }
             Py_CLEAR(v);
         }
 
         methinfo->arrayArg = -1;
-        switch (PyDict_GetItemRef(metadata, PyObjCNM_c_array_length_in_arg, &v)) {
+        switch (PyDict_GetItemRef(metadata, PyObjCNM_c_array_length_in_arg, &v)) { // LCOV_BR_EXCL_LINE
         case -1:
             return -1;                       // LCOV_EXCL_LINE
         /* case 0: pass */
@@ -1401,12 +1422,16 @@ process_metadata_dict(PyObjCMethodSignature* methinfo, PyObject* _Nullable metad
 
         methinfo->variadic = NO;
 
-        switch (PyDict_GetItemRef(metadata, PyObjCNM_variadic, &v)) {
+        switch (PyDict_GetItemRef(metadata, PyObjCNM_variadic, &v)) { // LCOV_BR_EXCL_LINE
         case -1:
             return -1;                       // LCOV_EXCL_LINE
         /* case 0: pass */
         case 1:
-            if (PyObject_IsTrue(v)) {
+            r = PyObject_IsTrue(v);
+            if (r == -1) {
+                return -1;
+            }
+            if (r) {
                 methinfo->variadic = YES;
 
                 if ((methinfo->suggestion == NULL) && (!methinfo->null_terminated_array)
@@ -1450,11 +1475,11 @@ static PyObjCMethodSignature* _Nullable compiled_metadata(PyObject* metadata)
     Py_ssize_t             pos;
     Py_ssize_t             i;
 
-    PyObjC_Assert(metadata != NULL, NULL);
-    PyObjC_Assert(PyDict_Check(metadata), NULL);
+    assert(metadata != NULL);
+    assert(PyDict_Check(metadata));
 
     PyObject* arguments;
-    switch (PyDict_GetItemRef(metadata, PyObjCNM_arguments, &arguments)) {
+    switch (PyDict_GetItemRef(metadata, PyObjCNM_arguments, &arguments)) { // LCOV_BR_EXCL_LINE
     case -1:
         return NULL;                             // LCOV_EXCL_LINE
     case 0:
@@ -1497,6 +1522,7 @@ static PyObjCMethodSignature* _Nullable compiled_metadata(PyObject* metadata)
     result->variadic              = NO;
     result->null_terminated_array = NO;
     result->free_result           = NO;
+    result->initializer           = NO;
     result->shortcut_signature    = NO;
     result->shortcut_argbuf_size  = 0;
     result->shortcut_result_size  = 0;
@@ -1508,10 +1534,12 @@ static PyObjCMethodSignature* _Nullable compiled_metadata(PyObject* metadata)
         result->argtype[i] = NULL;
     }
 
-    switch (PyDict_GetItemRef(metadata, PyObjCNM_full_signature, &value)) {
+    switch (PyDict_GetItemRef(metadata, PyObjCNM_full_signature, &value)) { // LCOV_BR_EXCL_LINE
     case -1:
+        // LCOV_EXCL_START
         Py_DECREF(result);
         return NULL;
+        // LCOV_EXCL_STOP
     /* case 0: pass */
     case 1:
         if (PyBytes_Check(value)) {
@@ -1544,9 +1572,9 @@ PyObjC_registerMetaData(PyObject* class_name, PyObject* selector, PyObject* meta
     PyObject* compiled;
     int       r;
 
-    PyObjC_Assert(registry != NULL, -1);
-    PyObjC_Assert(PyBytes_Check(class_name), -1);
-    PyObjC_Assert(PyBytes_Check(selector), -1);
+    assert(registry != NULL);
+    assert(PyBytes_Check(class_name));
+    assert(PyBytes_Check(selector));
     if (!PyDict_Check(metadata)) {
         PyErr_SetString(PyExc_TypeError, "metadata should be a dictionary");
         return -1;
@@ -1575,7 +1603,7 @@ PyObjCMethodSignature* _Nullable PyObjCMethodSignature_WithMetaData(
 {
     PyObjCMethodSignature* methinfo;
 
-    PyObjC_Assert(signature != NULL, NULL);
+    assert(signature != NULL);
 
     methinfo = new_methodsignature(signature);
     if (methinfo == NULL) {
@@ -1668,13 +1696,13 @@ static struct _PyObjC_ArgDescr* _Nullable merge_descr(struct _PyObjC_ArgDescr* d
 
     if (meta->modifier != '\0') {
         const char* withoutModifiers = PyObjCRT_SkipTypeQualifiers(descr->type);
-        PyObjC_Assert(*withoutModifiers != _C_ARY_B, NULL);
+        assert(*withoutModifiers != _C_ARY_B);
         if (descr->type[0] == _C_PTR && descr->type[1] == _C_VOID
             && descr->ptrType == PyObjC_kPointerPlain) {
 
             /* Plain old void*, ignore type modifiers */
 
-        } else {
+        } else { // LCOV_EXCL_LINE
             size_t bufsize = strlen(withoutModifiers) + 2;
             char* tp      = PyMem_Malloc(bufsize);
             char* to_free = NULL;
@@ -1693,7 +1721,7 @@ static struct _PyObjC_ArgDescr* _Nullable merge_descr(struct _PyObjC_ArgDescr* d
             }
 
             /* Skip existing modifiers, we're overriding those */
-            PyObjC_Assert(tp != NULL, NULL);
+            assert(tp != NULL);
             strlcpy(tp + 1, withoutModifiers, bufsize-1);
             tp[0]               = meta->modifier;
             descr->typeOverride = YES;
@@ -1725,6 +1753,7 @@ process_metadata_object(PyObjCMethodSignature* methinfo, PyObjCMethodSignature*
     methinfo->variadic              = metadata->variadic;
     methinfo->null_terminated_array = metadata->null_terminated_array;
     methinfo->free_result           = metadata->free_result;
+    methinfo->initializer           = metadata->initializer;
     methinfo->arrayArg              = metadata->arrayArg;
     methinfo->deprecated            = metadata->deprecated;
 
@@ -1816,7 +1845,7 @@ PyObjCMethodSignature* _Nullable PyObjCMethodSignature_ForSelector(
     PyObject*              metadata;
 
     metadata = PyObjC_FindInRegistry(registry, cls, sel);
-    PyObjC_Assert(metadata == NULL || PyObjCMethodSignature_Check(metadata), NULL);
+    assert(metadata == NULL || PyObjCMethodSignature_Check(metadata));
 
     if (metadata != NULL && ((PyObjCMethodSignature*)metadata)->signature) {
         methinfo = new_methodsignature(((PyObjCMethodSignature*)metadata)->signature);
@@ -1834,6 +1863,7 @@ PyObjCMethodSignature* _Nullable PyObjCMethodSignature_ForSelector(
         return NULL;
     }
 
+
     if (isClassMethod) {
         const char* nm = sel_getName(sel);
         if (strncmp(nm, "new", 3) == 0 && ((nm[3] == 0) || isupper(nm[3]))) {
@@ -1851,10 +1881,24 @@ PyObjCMethodSignature* _Nullable PyObjCMethodSignature_ForSelector(
         }
     }
 
-#ifdef PyObjC_DEBUG
-    if (PyObjCMethodSignature_Validate(methinfo) == -1) // LCOV_BR_EXCL_LINE
-        return NULL;                                    // LCOV_EXCL_LINE
-#endif                                                  /* PyObjC_DEBUG */
+    if (methinfo->rettype != NULL && methinfo->rettype->type != NULL && methinfo->rettype->type[0] == _C_ID) {
+        const char* nm = sel_getName(sel);
+        // The check below matches the calculation of the default objc_method_family in clang
+        // (other than the special casing of NSAutoreleasePool)
+        if (sel_isEqual(sel, @selector(init)) ||
+            ((strncmp(nm, "init", 4) == 0) && isupper(nm[4]))) {
+            /* Mark 'init' methods, except for NSAutoreleasePool
+             * which crashes when the reference count is manipulated
+             * manually.
+             */
+            if (cls != [NSAutoreleasePool class]) {
+                methinfo->initializer = YES;
+            }
+        }
+    }
+
+
+    PyObjCMethodSignature_Validate(methinfo);
 
     Py_XDECREF(metadata);
     return methinfo;
@@ -2000,18 +2044,15 @@ static PyObject* _Nullable argdescr2dict(struct _PyObjC_ArgDescr* descr)
         r = PyDict_SetItem(result, PyObjCNM_null_accepted, descr->allowNULL?Py_True:Py_False);
         if (r == -1)    // LCOV_BR_EXCL_LINE
             goto error; // LCOV_EXCL_LINE
-    } else if (PyErr_Occurred()) {
-        /* Invalid metadata, XXX don't like this pattern */
-        goto error;
     }
 
     return result;
 
 error:
-    // LCOV_BR_EXCL_START
+    // LCOV_EXCL_START
     Py_DECREF(result);
     return NULL;
-    // LCOV_BR_EXCL_STOP
+    // LCOV_EXCL_STOP
 }
 
 PyObject* _Nullable PyObjCMethodSignature_AsDict(PyObjCMethodSignature* methinfo)
@@ -2032,6 +2073,19 @@ PyObject* _Nullable PyObjCMethodSignature_AsDict(PyObjCMethodSignature* methinfo
         if (r == -1)    // LCOV_BR_EXCL_LINE
             goto error; // LCOV_EXCL_LINE
     }
+
+    if (methinfo->free_result) {
+        r = PyDict_SetItem(result, PyObjCNM_free_result, methinfo->free_result?Py_True:Py_False);
+        if (r == -1)    // LCOV_BR_EXCL_LINE
+            goto error; // LCOV_EXCL_LINE
+    }
+
+    if (methinfo->initializer) {
+        r = PyDict_SetItem(result, PyObjCNM_initializer, methinfo->initializer?Py_True:Py_False);
+        if (r == -1)    // LCOV_BR_EXCL_LINE
+            goto error; // LCOV_EXCL_LINE
+    }
+
     if (methinfo->variadic) {
         r = PyDict_SetItem(result, PyObjCNM_variadic, methinfo->variadic?Py_True:Py_False);
         if (r == -1)    // LCOV_BR_EXCL_LINE
@@ -2105,13 +2159,15 @@ PyObject* _Nullable PyObjCMethodSignature_AsDict(PyObjCMethodSignature* methinfo
         }
 
         PyTuple_SET_ITEM(v, i, t);
-    }
+    } // LCOV_BR_EXCL_LINE
 
     return result;
 
+    // LCOV_EXCL_START
 error:
     Py_XDECREF(result);
     return NULL;
+    // LCOV_EXCL_STOP
 }
 
 PyObject* _Nullable PyObjC_copyMetadataRegistry(void)
