@@ -110,7 +110,8 @@ calc_current_version(void)
         }
 
         [pool release];
-    } // LCOV_EXCL_STOP
+        // LCOV_EXCL_STOP
+    }  // LCOV_EXCL_LINE
 }
 
 PyObjC_FINAL_CLASS @interface OC_NSAutoreleasePoolCollector : NSObject
@@ -215,7 +216,7 @@ static PyObject* _Nullable repythonify(PyObject* self __attribute__((__unused__)
 }
 
 PyDoc_STRVAR(macos_available_doc,
-             "macos_available(major, minor, patch=0)\n" CLINIC_SEP "\n"
+             "macos_available(major, minor=0, patch=0)\n" CLINIC_SEP "\n"
              "Return true if the current macOS release is "
              "at least the provided version");
 
@@ -225,10 +226,10 @@ static PyObject* _Nullable macos_available(PyObject* self __attribute__((__unuse
 {
     static char* keywords[] = {"major", "minor", "patch", NULL};
     long         major;
-    long         minor;
+    long         minor = 0;
     long         patch = 0;
 
-    if (!PyArg_ParseTupleAndKeywords(args, kwds, "ll|l", keywords, &major, &minor,
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "l|ll", keywords, &major, &minor,
                                      &patch)) {
         return NULL;
     }
@@ -417,7 +418,7 @@ static PyObject* _Nullable recycle_autorelease_pool(PyObject* self
 }
 
 PyDoc_STRVAR(getClassList_doc,
-             "getClassList(ignore_invalid_identifiers=False)\n" CLINIC_SEP "\n"
+             "getClassList(ignore_invalid_identifiers=False, /)\n" CLINIC_SEP "\n"
              "Return a list with all Objective-C classes known to the runtime.\n");
 static PyObject* _Nullable getClassList(PyObject* self __attribute__((__unused__)),
                                         PyObject* args)
@@ -527,8 +528,15 @@ static PyObject* _Nullable loadBundle(PyObject* self __attribute__((__unused__))
      * when possible.
      */
 
-    if (scanClasses != NULL && !PyObject_IsTrue(scanClasses)) {
-        return pythonify_c_value(@encode(NSBundle*), &bundle);
+    if (scanClasses != NULL) {
+        int r =  PyObject_IsTrue(scanClasses);
+        if (r == -1) {
+            return NULL;
+        }
+
+        if (!r) {
+            return pythonify_c_value(@encode(NSBundle*), &bundle);
+        }
     }
 
     class_list = PyObjC_GetClassList(1);
@@ -798,7 +806,7 @@ PyDoc_STRVAR(PyObjC_loadBundleFunctions_doc,
              "The signature is the Objective-C type specifier for the function \n"
              "signature.");
 PyDoc_STRVAR(PyObjC_loadFunctionList_doc,
-             "loadFunctionList(list, module_globals, functionInfo, "
+             "loadFunctionList(function_list, module_globals, functionInfo, "
              "skip_undefined=True)\n" CLINIC_SEP "\n"
              "Load the specified functions. List should be a capsule object containing\n"
              "an array of { char*, function } structs.");
@@ -862,6 +870,27 @@ static PyObject* _Nullable idSignatures(PyObject* self __attribute__((__unused__
     return PyObjCPointer_GetIDEncodings();
 }
 
+PyDoc_STRVAR(sizeOfType_doc,
+        "_sizeOfType(type, /)\n" CLINIC_SEP "\n"
+        "Returns the size of the type referred to by the encoding.");
+static PyObject* _Nullable sizeOfType(PyObject* self __attribute__((__unused__)), PyObject* _Nullable args)
+{
+    char* encoding;
+    Py_ssize_t size;
+
+    if (!PyArg_ParseTuple(args, "y", &encoding)) {
+        return NULL;
+    }
+
+    size = PyObjCRT_SizeOfType(encoding);
+    if (size == -1) {
+        return NULL;
+    }
+
+    return PyLong_FromSsize_t(size);
+}
+
+
 PyDoc_STRVAR(protocolsForClass_doc,
              "protocolsForClass(cls)\n" CLINIC_SEP "\n"
              "Returns a list of Protocol objects that the class claims\n"
@@ -889,7 +918,7 @@ static PyObject* _Nullable protocolsForClass(PyObject* self __attribute__((__unu
     protocol_list = class_copyProtocolList(cls, &protocol_count);
 
     /* Documented API: the list will only be NULL if the count is 0 */
-    PyObjC_Assert(protocol_count == 0 || protocol_list != NULL, NULL);
+    assert(protocol_count == 0 || protocol_list != NULL);
 
     for (i = 0; i < protocol_count; i++) {
         PyObject* protocol = PyObjCFormalProtocol_ForProtocol(protocol_list[i]);
@@ -964,8 +993,8 @@ static PyObject* _Nullable registerMetaData(PyObject* self __attribute__((__unus
                                      &metadata)) {
         return NULL;
     }
-    PyObjC_Assert(PyBytes_Check(class_name), NULL);
-    PyObjC_Assert(PyBytes_Check(selector), NULL);
+    assert(PyBytes_Check(class_name));
+    assert(PyBytes_Check(selector));
 
     if (PyObjC_registerMetaData(class_name, selector, metadata) < 0) {
         return NULL;
@@ -1085,9 +1114,11 @@ static PyObject* _Nullable createStructType(PyObject* self __attribute__((__unus
                 goto error_cleanup;
             }
 
-            if (fieldnames[i] == NULL) {
+            if (fieldnames[i] == NULL) { // LCOV_BR_EXCL_LINE
+                // LCOV_EXCL_START
                 PyErr_NoMemory();
                 goto error_cleanup;
+                // LCOV_EXCL_STOP
             }
         }
         field_count = PyTuple_GET_SIZE(pyfieldnames);
@@ -1097,7 +1128,7 @@ static PyObject* _Nullable createStructType(PyObject* self __attribute__((__unus
         fieldnames  = NULL;
     }
 
-    retval = PyObjC_RegisterStructType(typestr, name, docstr, NULL, field_count,
+    retval = PyObjC_RegisterStructType(typestr, name, docstr, field_count,
                                        (const char**)fieldnames, pack);
     if (retval == NULL)
         goto error_cleanup;
@@ -1160,7 +1191,7 @@ static PyObject* _Nullable registerCFSignature(PyObject* self __attribute__((__u
     CFTypeID     typeId;
     char*        tollfreeName = NULL;
 
-    if (!PyArg_ParseTupleAndKeywords(args, kwds, "syO|s", keywords, &name, &encoding,
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "syO|z", keywords, &name, &encoding,
                                      &pTypeId, &tollfreeName)) {
         return NULL;
     }
@@ -1202,8 +1233,8 @@ static PyObject* _Nullable registerCFSignature(PyObject* self __attribute__((__u
             return NULL;
         }
 
-        if (PyObjCPointerWrapper_RegisterID(name, encoding) == -1) {
-            return NULL;
+        if (PyObjCPointerWrapper_RegisterID(name, encoding) == -1) { // LCOV_BR_EXCL_LINE
+            return NULL; // LCOV_EXCL_LINE
         }
 
         /* Don't have to do anything with the cfTypeId: because
@@ -1224,13 +1255,13 @@ static PyObject* _Nullable _updatingMetadata(PyObject* self __attribute__((__unu
                                              PyObject* _Nullable kwds)
 {
     static char* keywords[] = {"flag", NULL};
-    PyObject*    flag;
+    int    flag;
 
-    if (!PyArg_ParseTupleAndKeywords(args, kwds, "O", keywords, &flag)) {
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "p", keywords, &flag)) {
         return NULL;
     }
 
-    if (PyObject_IsTrue(flag)) {
+    if (flag) {
         PyObjC_UpdatingMetaData = YES;
 
     } else {
@@ -1258,12 +1289,14 @@ static PyObject* _Nullable PyObjC_objc_sync_enter(PyObject* self
 
     Py_END_ALLOW_THREADS
 
-    if (rv == OBJC_SYNC_SUCCESS) {
+    if (rv == OBJC_SYNC_SUCCESS) { // LCOV_BR_EXCL_LINE
         Py_RETURN_NONE;
     }
 
+    // LCOV_EXCL_START
     PyErr_Format(PyObjCExc_LockError, "objc_sync_enter failed: %d", rv);
     return NULL;
+    // LCOV_EXCL_STOP
 }
 
 static PyObject* _Nullable PyObjC_objc_sync_exit(PyObject* self
@@ -1323,15 +1356,17 @@ static PyObject* _Nullable _makeClosure(PyObject* self __attribute__((__unused__
 
     if (PyObjCFunction_Check(closureFor)) {
         methinfo = (PyObjCMethodSignature*)PyObjCFunc_GetMethodSignature(closureFor);
-        if (methinfo == NULL) {
-            return NULL;
+        if (methinfo == NULL) { // LCOV_BR_EXCL_LINE
+            return NULL; // LCOV_EXCL_LINE
         }
 
     } else if (PyObjCSelector_Check(closureFor)) {
         methinfo = PyObjCSelector_GetMetadata(closureFor);
-        if (methinfo == NULL) {
-            PyObjC_Assert(PyErr_Occurred(), NULL);
+        if (methinfo == NULL) { // LCOV_BR_EXCL_LNE
+            // LCOV_EXCL_START
+            assert(PyErr_Occurred());
             return NULL;
+            // LCOV_EXCL_STOP
         }
 
     } else {
@@ -1381,10 +1416,12 @@ static PyObject* _Nullable _makeClosure(PyObject* self __attribute__((__unused__
     }
 
     PyObject* capsule = PyCapsule_New((void*)closure_func, "objc.__imp__", _callback_cleanup);
-    if (capsule == NULL) {
+    if (capsule == NULL) { // LCOV_BR_EXCL_LINE
+        // LCOV_EXCL_START
         PyObjCFFI_FreeIMP((IMP)closure_func);
         Py_CLEAR(methinfo);
         return NULL;
+        // LCOV_EXCL_STOP
     }
 
     PyObject* result = Py_BuildValue(
@@ -1451,13 +1488,13 @@ static PyObject* _Nullable PyObjC_setAssociatedObject(PyObject* self
         @try {
             objc_setAssociatedObject(object, (void*)key, value, policy);
 
-        } @catch (NSObject* localException) {
-            PyObjCErr_FromObjC(localException);
+        } @catch (NSObject* localException) { // LCOV_EXCL_LINE
+            PyObjCErr_FromObjC(localException); // LCOV_EXCL_LINE
         }
     Py_END_ALLOW_THREADS
 
-    if (PyErr_Occurred())
-        return NULL;
+    if (PyErr_Occurred()) // LCOV_BR_EXCL_LINE
+        return NULL; // LCOV_EXCL_LINE
 
     Py_RETURN_NONE;
 }
@@ -1486,14 +1523,14 @@ static PyObject* _Nullable PyObjC_getAssociatedObject(PyObject* self
         @try {
             value = objc_getAssociatedObject(object, (void*)key);
 
-        } @catch (NSObject* localException) {
-            value = nil;
-            PyObjCErr_FromObjC(localException);
+        } @catch (NSObject* localException) { // LCOV_EXCL_LINE
+            value = nil; // LCOV_EXCL_LINE
+            PyObjCErr_FromObjC(localException); // LCOV_EXCL_LINE
         }
     Py_END_ALLOW_THREADS
 
-    if (PyErr_Occurred())
-        return NULL;
+    if (PyErr_Occurred()) // LCOV_BR_EXCL_LINE
+        return NULL; // LCOV_EXCL_LINE
 
     return id_to_python(value);
 }
@@ -1522,13 +1559,13 @@ static PyObject* _Nullable PyObjC_removeAssociatedObjects(PyObject* self
         @try {
             objc_removeAssociatedObjects(object);
 
-        } @catch (NSObject* localException) {
-            PyObjCErr_FromObjC(localException);
+        } @catch (NSObject* localException) { // LCOV_EXCL_LINE
+            PyObjCErr_FromObjC(localException); // LCOV_EXCL_LINE
         }
     Py_END_ALLOW_THREADS
 
-    if (PyErr_Occurred())
-        return NULL;
+    if (PyErr_Occurred()) // LCOV_BR_EXCL_LINE
+        return NULL; // LCOV_EXCL_LINE
 
     Py_RETURN_NONE;
 }
@@ -1614,13 +1651,16 @@ static PyObject* _Nullable block_signature(PyObject* mod __attribute__((__unused
     }
 
     id objc_block = PyObjCObject_GetObject(block);
-    if (objc_block == nil) {
-        PyErr_SetString(PyObjCExc_Error, "Cannot get block signature of 'nil' block");
-        return NULL;
-    }
+    assert(objc_block != nil);
+
     const char* sig = PyObjCBlock_GetSignature(objc_block);
-    if (sig == NULL) {
+    if (sig == NULL) { // LCOV_BR_EXCL_LINE
+        /* AFAIK it is not possible to get a block object
+         * without a signature in modern compilers.
+         */
+        // LCOV_EXCL_START
         Py_RETURN_NONE;
+        // LCOV_EXCL_STOP
     }
 
     return PyBytes_FromString(sig);
@@ -1667,7 +1707,7 @@ static PyObject* _Nullable mod_dyld_shared_cache_contains_path(
     /* This uses an availability check for 10.16 just in case
      * we're loaded in a Python that was compiled with an old SDK.
      */
-    if (@available(macOS 10.16, *)) {
+    if (@available(macOS 10.16, *)) { // LCOV_BR_EXCL_LINE
         const char* path = PyUnicode_AsUTF8(object);
         if (path == NULL) {
             return NULL;
@@ -1680,7 +1720,10 @@ static PyObject* _Nullable mod_dyld_shared_cache_contains_path(
             Py_RETURN_FALSE;
         }
     } else {
-        Py_RETURN_FALSE;
+        /* coverage tests are run on a modern system, hence
+         * the lcov annotation.
+         */
+        Py_RETURN_FALSE; // LCOV_EXCL_LINE
     }
 }
 
@@ -1758,7 +1801,7 @@ static PyObject* _Nullable mod_registerVectorType(PyObject* _Nullable mod
     int r = PyObjCRT_RegisterVectorType(PyBytes_AsString(typestr), object);
     Py_DECREF(typestr);
     if (r == -1) {
-        PyObjC_Assert(PyErr_Occurred(), NULL);
+        assert(PyErr_Occurred());
         return NULL;
     } else {
         Py_RETURN_NONE;
@@ -1786,7 +1829,7 @@ static PyObject* _Nullable mod_registeredMetadataForSelector(PyObject* _Nullable
     Class cls = PyObjCClass_GetClass(class);
     if (cls == Nil) { // LCOV_BR_EXCL_LINE
         // LCOV_EXCL_START
-        PyObjC_Assert(PyErr_Occurred(), NULL);
+        assert(PyErr_Occurred());
         return NULL;
         // LCOV_EXCL_STOP
     }
@@ -1925,6 +1968,10 @@ static PyMethodDef mod_methods[] = {
      .ml_meth  = (PyCFunction)idSignatures,
      .ml_flags = METH_NOARGS,
      .ml_doc   = idSignatures_doc},
+    {.ml_name  = "_sizeOfType",
+     .ml_meth  = (PyCFunction)sizeOfType,
+     .ml_flags = METH_VARARGS,
+     .ml_doc   = sizeOfType_doc},
     {.ml_name  = "_updatingMetadata",
      .ml_meth  = (PyCFunction)_updatingMetadata,
      .ml_flags = METH_VARARGS | METH_KEYWORDS,
@@ -2150,6 +2197,7 @@ static setup_function _Nullable setup_functions[] = {
     PyObjCVarList_Setup,
     PyObjCSuper_Setup,
     PyObjCMethodSignature_Setup,
+    PyObjCStruct_Init,
     PyObjCUnicode_Setup,
     PyObjCInitNULL,
     PyObjCInstanceVariable_Setup,
@@ -2167,7 +2215,6 @@ static setup_function _Nullable setup_functions[] = {
     PyObjCCFType_Setup,
     PyObjCBlock_Setup,
     PyObjCFSRef_Setup,
-    PyObjC_SockAddr_Setup,
 
     PyObjCAPI_Register, /* Must be last */
     NULL};
