@@ -1,9 +1,6 @@
 /*
  * Implementation of objective-C object wrapper
  *
- * NOTE: We're using CFRetain and CFRelease to manage the retaincount of the Objective-C
- * objects because that will do the right thing when Garbage Collection is involved.
- *
  * XXX: Free-threading: Access to 'flags' is not yet thread safe, need to audit all
  *      paths that change flags.
  */
@@ -103,32 +100,27 @@ static PyObject* _Nullable object_repr(PyObject* _self)
         // LCOV_EXCL_STOP
     }
 
-    if ((flags & PyObjCObject_kUNINITIALIZED) == 0) {
-        /* Try to call the method 'description', which is the ObjC
-         * equivalent of __repr__. If that fails we'll fall back to
-         * the default repr.
-         * Don't call 'description' for uninitialized objects, that
-         * is undefined behaviour and will crash the interpreter sometimes.
-         *
-         * Free threaded: the UNINITIALIZED flags is never re-set once cleared.
-         */
-        PyObject* args[2] = {NULL, (PyObject*)self};
-        res               = PyObject_VectorcallMethod(PyObjCNM_description, args + 1,
-                                                      1 | PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
-        if (res == NULL) {
-            PyErr_Clear();
-            /* Fall though to default */
-        } else if (res == Py_None) {
-            Py_DECREF(res);
-            /* Fall though to default */
-        } else {
-            return res;
-        }
+    /* Try to call the method 'description', which is the ObjC
+     * equivalent of __repr__. If that fails we'll fall back to
+     * the default repr.
+     * Don't call 'description' for uninitialized objects, that
+     * is undefined behaviour and will crash the interpreter sometimes.
+     *
+     * Free threaded: the UNINITIALIZED flags is never re-set once cleared.
+     */
+    PyObject* args[2] = {NULL, (PyObject*)self};
+    res               = PyObject_VectorcallMethod(PyObjCNM_description, args + 1,
+                                                  1 | PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
+    if (res == NULL) {
+        PyErr_Clear();
+    } else if (res == Py_None) {
+        Py_DECREF(res);
+    } else {
+        return res;
     }
-    Py_BEGIN_CRITICAL_SECTION(_self);
+
     res = PyUnicode_FromFormat("<%s objective-c instance %p>", Py_TYPE(self)->tp_name,
                                 self->objc_object);
-    Py_END_CRITICAL_SECTION();
 
     return res;
 }
@@ -153,55 +145,32 @@ object_dealloc(PyObject* obj)
      */
     PyObject *ptype, *pvalue, *ptraceback;
     PyErr_Fetch(&ptype, &pvalue, &ptraceback);
+    id objc_object = ((PyObjCObject*)obj)->objc_object;
 
+    PyObjC_UnregisterPythonProxy(objc_object, obj);
 
-    if (PyObjCObject_IsBlock(obj)) {
-        SET_FIELD(((PyObjCBlockObject*)obj)->signature, NULL);
-    }
-
-    if (PyObjCObject_GetFlags(obj) != PyObjCObject_kDEALLOC_HELPER
-        && PyObjCObject_GetObject(obj) != nil) {
+    if (PyObjCObject_GetFlags(obj) != PyObjCObject_kDEALLOC_HELPER) {
         /* Release the proxied object, we don't have to do this when
          * there is no proxied object.
          */
-        PyObjC_UnregisterPythonProxy(PyObjCObject_GetObject(obj), obj);
-
         if ((((PyObjCObject*)obj)->flags & PyObjCObject_kSHOULD_NOT_RELEASE)) {
             /* pass */
 
-        } else if (((PyObjCObject*)obj)->flags & PyObjCObject_kUNINITIALIZED) {
-            /* Freeing of an uninitialized object, just leak because
-             * there is no reliable manner to free such objects.
-             *
-             * - [obj release] doesn't work because some classes
-             *   cause crashes for uninitialized objects
-             * - [[obj init] release] also doesn't work because
-             *   not all classes implement -init
-             * - [obj dealloc] doesn't work for class
-             *   clusters like NSArray.
-             */
-            char buf[256];
-            snprintf(buf, sizeof(buf), "leaking an uninitialized object of type %s",
-                     Py_TYPE(obj)->tp_name);
-            ((PyObjCObject*)obj)->objc_object = nil;
-            if (PyErr_Warn(PyObjCExc_UnInitDeallocWarning, buf) == -1) {
-                PyErr_WriteUnraisable((PyObject*)Py_TYPE(obj));
-            }
-
         } else {
             Py_BEGIN_ALLOW_THREADS
                 @try {
-                    CFRelease(((PyObjCObject*)obj)->objc_object);
+                    [objc_object release];
 
                 } @catch (NSObject* localException) {
                     NSLog(@"PyObjC: Exception during dealloc of proxy: %@",
                           localException);
                 }
             Py_END_ALLOW_THREADS;
-            ((PyObjCObject*)obj)->objc_object = nil;
         }
     }
-
+    if (PyObjCObject_IsBlock(obj)) {
+        SET_FIELD(((PyObjCBlockObject*)obj)->signature, NULL);
+    }
     Py_TYPE(obj)->tp_free(obj);
 
     PyErr_Restore(ptype, pvalue, ptraceback);
@@ -214,13 +183,7 @@ object_verify_type(PyObject* obj)
     id obj_inst;
 
     obj_inst = PyObjCObject_OBJECT(obj);
-    if (unlikely(obj_inst == nil)) { // LCOV_BR_EXCL_LINE
-        /* the (currently sole) caller of this
-         * function already checked that obj_inst is
-         * not nil.
-         */
-        return 0; // LCOV_EXCL_LINE
-    }
+    assert(obj_inst != nil);
 
     if (PyObjCClass_IsCFWrapper(Py_TYPE(obj))) {
         /* Object is a CoreFoundation type, ISA swiffling doesn't
@@ -236,7 +199,7 @@ object_verify_type(PyObject* obj)
          * object because that might cause havoc.
          */
 
-    } else {
+    } else { // LCOV_EXCL_LINE
         /* Special hack for KVO on MacOS X, when an object is observed it's
          * ISA is changed by the runtime. We change the python type as well.
          *
@@ -303,66 +266,18 @@ object_verify_type(PyObject* obj)
     return 0;
 }
 
-static int
-object_verify_not_nil(PyObject* obj, PyObject* name)
-{
-    /* XXX: This should be safe in a free-threaded build, the 'objc_object'
-     *      is only set to 'nil' when working with an uninitialized value
-     *      (either deallocating it, or when an init method returns a
-     *      value different than 'self'.
-     *
-     *      In either case the proxy should not be used in different threads
-     *      (e.g. ``x = [NSArray alloc]; [x init]; [x init]`` is buggy both
-     *      as written and when one of the init calls is in a different thread.
-     *
-     *      For UNINITIALIZED values there will always be a race between using
-     *      the value in one thread, and another thread invoking the initializer.
-     *      That's even true when the GIL is enabled because we release the GIL
-     *      when calling a method.
-     *
-     *      E.g.:
-     *
-     *              THREAD 1                     THREAD 2
-     *              --------                     --------
-     *
-     *              gValue = NSArray.alloc()
-     *
-     *              gValue.init()
-     *                                            gValue.init()
-     *                                            actually invoke [gValue init]
-     *                                            clear objc_object because the result
-     *                                               from init is not self.
-     *
-     *              actually invoke [gValue init]
-     *              BOOM!
-     *
-     *      The only way to avoid this is to lock 'self' when using an UNINITIALIZED
-     *      value.
-     *
-     *      This needs to be documented properly, but is otherwise in "don't do that then"
-     *      territory.
-     */
-    if (PyObjCObject_OBJECT(obj) == nil) {
-        PyErr_Format(PyExc_AttributeError,
-                     "cannot access attribute '%U' of NIL '%.50s' object", name,
-                     Py_TYPE(obj)->tp_name);
-        return -1;
-    }
-    return 0;
-}
-
 static PyObject* _Nullable* _Nullable _get_dictptr(PyObject* obj)
 {
     /* XXX: See above: should be safe with a free-threaded build
      */
     Py_ssize_t dictoffset;
-    id         obj_object;
+    id         objc_object;
     dictoffset = PyObjCClass_DictOffset((PyObject*)Py_TYPE(obj));
     if (dictoffset == 0)
         return NULL;
-    obj_object = PyObjCObject_OBJECT(obj);
-    PyObjC_Assert(obj_object != nil, NULL);
-    return (PyObject**)(((char*)obj_object) + dictoffset);
+    objc_object = PyObjCObject_OBJECT(obj);
+    assert(objc_object != nil);
+    return (PyObject**)(((char*)objc_object) + dictoffset);
 }
 
 /* returns a new reference */
@@ -386,7 +301,7 @@ static inline PyObject* _Nullable _type_lookup(PyTypeObject* tp, PyObject* name,
         return NULL;   // LCOV_EXCL_LINE
     }
 
-    PyObjC_Assert(PyTuple_Check(mro), NULL);
+    assert(PyTuple_Check(mro));
     n = PyTuple_GET_SIZE(mro);
     for (i = 0; i < n; i++) {
         base = PyTuple_GET_ITEM(mro, i);
@@ -410,8 +325,10 @@ static inline PyObject* _Nullable _type_lookup(PyTypeObject* tp, PyObject* name,
 #endif
 
             if (PyObjCClass_CheckMethodList(base, 0) < 0) { // LCOV_BR_EXCL_LINE
+                // LCOV_EXCL_START
                 Py_CLEAR(mro);
-                return NULL;                                // LCOV_EXCL_LINE
+                return NULL;
+                // LCOV_EXCL_STOP
             }
 
             dict = PyType_GetDict((PyTypeObject*)base);
@@ -432,7 +349,7 @@ static inline PyObject* _Nullable _type_lookup(PyTypeObject* tp, PyObject* name,
             // LCOV_EXCL_STOP
         }
 
-        PyObjC_Assert(dict && PyDict_Check(dict), NULL);
+        assert(dict && PyDict_Check(dict));
 
         int r = PyDict_GetItemRef(dict, name, &descr);
         Py_CLEAR(dict);
@@ -462,8 +379,10 @@ static inline PyObject* _Nullable _type_lookup(PyTypeObject* tp, PyObject* name,
             PyObject* hidden = PyObjCClass_HiddenSelector(base, sel, NO);
             if (!hidden) {
                 if (PyErr_Occurred()) { // LCOV_BR_EXCL_LINE
+                    // LCOV_EXCL_START
                     Py_CLEAR(mro);
-                    return NULL;        // LCOV_EXCL_LINE
+                    return NULL;
+                    // LCOV_EXCL_STOP
                 }
 
                 descr = PyObjCClass_TryResolveSelector(base, name, sel);
@@ -500,7 +419,7 @@ static inline PyObject* _Nullable _type_lookup_harder(PyTypeObject* tp, PyObject
         return NULL;   // LCOV_EXCL_LINE
     }
 
-    PyObjC_Assert(PyTuple_Check(mro), NULL);
+    assert(PyTuple_Check(mro));
     n = PyTuple_GET_SIZE(mro);
 
     for (i = 0; i < n; i++) {
@@ -628,7 +547,7 @@ static PyObject* _Nullable object_getattro(PyObject* obj, PyObject* name)
     PyObject**    dictptr;
     const char*   namestr;
 
-    PyObjC_Assert(name != NULL, NULL);
+    assert(name != NULL);
 
     if (!PyUnicode_Check(name)) { // LCOV_BR_EXCL_LINE
         /* Can only happen when someone calls the tp_getattro slot
@@ -646,10 +565,6 @@ static PyObject* _Nullable object_getattro(PyObject* obj, PyObject* name)
         return NULL;       // LCOV_EXCL_LINE
     }
 
-    if (object_verify_not_nil(obj, name) == -1) {
-        goto done;
-    }
-
     if (object_verify_type(obj) == -1) {
         goto done;
     }
@@ -682,14 +597,17 @@ static PyObject* _Nullable object_getattro(PyObject* obj, PyObject* name)
 
     if (PyObjC_is_ascii_string(name, "__del__")) {
         res = PyObjCClass_GetDelMethod((PyObject*)Py_TYPE(obj));
-
-        descrgetfunc f = Py_TYPE(res)->tp_descr_get;
-        if (f != NULL) {
-            PyObject* tmp = f(res, obj, (PyObject*)Py_TYPE(obj));
-            Py_CLEAR(res);
-            res = tmp;
+        if (res != NULL) {
+            descrgetfunc f = Py_TYPE(res)->tp_descr_get;
+            if (f != NULL) {
+                PyObject* tmp = f(res, obj, (PyObject*)Py_TYPE(obj));
+                Py_CLEAR(res);
+                res = tmp;
+            }
+        } else {
+            PyErr_Format(PyExc_AttributeError, "'%.50s' object has no attribute '%.400s'",
+                 tp->tp_name, namestr);
         }
-
         goto done;
     }
 
@@ -701,7 +619,7 @@ static PyObject* _Nullable object_getattro(PyObject* obj, PyObject* name)
 
         if (PyObjC_is_ascii_string(name, "__dict__")) {
 
-            res = *dictptr;
+            res = *(PyObject* PyObjC_ATOMIC *)dictptr;
             if (res == NULL) {
                 res = PyDict_New();
 
@@ -711,11 +629,11 @@ static PyObject* _Nullable object_getattro(PyObject* obj, PyObject* name)
                  */
                 PyObject* to_clear = NULL;
                 Py_BEGIN_CRITICAL_SECTION(obj);
-                if (*dictptr == NULL) {
-                    *dictptr = res;
+                if (*(PyObject* PyObjC_ATOMIC *)dictptr == NULL) {
+                    *(PyObject* PyObjC_ATOMIC *)dictptr = res;
                 } else {
                     to_clear = res;
-                    res = *dictptr;
+                    res = *(PyObject* PyObjC_ATOMIC *)dictptr;
                 }
                 Py_END_CRITICAL_SECTION();
                 Py_CLEAR(to_clear);
@@ -765,7 +683,7 @@ static PyObject* _Nullable object_getattro(PyObject* obj, PyObject* name)
             PyErr_SetString(PyObjCExc_Error, "Descriptor getter returned NULL "
                                              "without raising an exception");
             // LCOV_EXCL_STOP
-        }
+        } // LCOV_EXCL_LINE
         goto done;
     }
 
@@ -818,32 +736,26 @@ object_setattro(PyObject* obj, PyObject* name, PyObject* _Nullable value)
         return -1;
     }
 
-    if (object_verify_not_nil(obj, name) == -1) {
-        return -1;
-    }
-
     obj_inst = PyObjCObject_GetObject(obj);
 
     obj_name = nil;
     if (((PyObjCClassObject*)tp)->useKVO) {
-        if ((PyObjCObject_GetFlags(obj) & PyObjCObject_kUNINITIALIZED) == 0) {
-            if (!PyObjC_is_ascii_prefix(name, "_", 1)) {
-                obj_name =
-                    [NSString stringWithUTF8String:(const char* _Nonnull)PyObjC_Unicode_Fast_Bytes(name)];
+        if (!PyObjC_is_ascii_prefix(name, "_", 1)) {
+            obj_name =
+                [NSString stringWithUTF8String:(const char* _Nonnull)PyObjC_Unicode_Fast_Bytes(name)];
 
-                if (obj_name == nil) {
-                    PyErr_SetString(PyObjCExc_Error, "Cannot convert attribute name to NSString");
-                    return -1;
-                }
+            if (obj_name == nil) {
+                PyErr_SetString(PyObjCExc_Error, "Cannot convert attribute name to NSString");
+                return -1;
+            }
 
-                @try {
-                    [(NSObject*)obj_inst willChangeValueForKey:obj_name];
-                } @catch (NSObject* localException) {
-                    PyObjCErr_FromObjC(localException);
-                }
-                if (PyErr_Occurred()) {
-                    return -1;
-                }
+            @try {
+                [(NSObject*)obj_inst willChangeValueForKey:obj_name];
+            } @catch (NSObject* localException) {
+                PyObjCErr_FromObjC(localException);
+            }
+            if (PyErr_Occurred()) {
+                return -1;
             }
         }
     }
@@ -942,14 +854,8 @@ static PyObject* _Nullable objc_get_real_class(PyObject* self,
     id        objc_object;
     PyObject* ret;
 
-    /* XXX: free-threading: see earlier discussion about using PyObjCObject_OBJECT
-     *      without locking.
-     */
     objc_object = PyObjCObject_OBJECT(self);
-    if (objc_object == nil) {
-        PyErr_SetString(PyObjCExc_Error, "Cannot access ISA of nil");
-        return NULL;
-    }
+    assert(objc_object != nil);
 
     /* object_getClass only return Nil if its argument is nil */
     ret = PyObjCClass_New((Class _Nonnull)object_getClass(objc_object));
@@ -969,7 +875,7 @@ static PyObject* _Nullable objc_get_real_class(PyObject* self,
         }
 #endif
         Py_END_CRITICAL_SECTION();
-    }
+    } // LCOV_EXCL_LINE
     return ret;
 }
 
@@ -997,15 +903,12 @@ static PyObject* _Nullable obj_get_blocksignature(PyObject* self, void* closure
     if (PyObjCObject_IsBlock(self)) {
         PyObjCMethodSignature* v = PyObjCObject_GetBlockSignature(self);
         if (v != NULL) {
+            Py_INCREF(v);
             return (PyObject*)v;
         } else {
             id objc_self = PyObjCObject_GetObject(self);
-            if (objc_self == nil) { // LCOV_BR_EXCL_LINE
-                // LCOV_EXCL_START
-                PyErr_SetString(PyObjCExc_Error, "Cannot get block signature of a 'nil' block");
-                return NULL;
-                // LCOV_EXCL_STOP
-            }
+            assert(objc_self != nil);
+
             const char* typestr = PyObjCBlock_GetSignature(objc_self);
             if (typestr != NULL) {
                 v = PyObjCMethodSignature_WithMetaData(typestr, NULL, YES);
@@ -1093,28 +996,18 @@ static PyObject* _Nullable meth_sizeof(PyObject* self __attribute__((__unused__)
      * This doesn't return the correct size for collections (such as NSArray),
      * but is better than the default implementation
      */
-    if (PyObjCObject_GetObject(self) == nil) {
-        return PyLong_FromSize_t(Py_TYPE(self)->tp_basicsize);
+    id objc_value = PyObjCObject_GetObject(self);
+    assert(objc_value != nil);
 
-    } else {
-        return PyLong_FromSize_t(
+    return PyLong_FromSize_t(
             Py_TYPE(self)->tp_basicsize
-            + class_getInstanceSize(object_getClass(PyObjCObject_GetObject(self))));
-    }
+            + class_getInstanceSize(object_getClass(objc_value)));
 }
 
 static PyObject*
 meth_is_magic(PyObject* self)
 {
-    int is_magic;
-
-    if (PyObjCObject_GetObject(self) == nil) { // LCOV_BR_EXCL_LINE
-        is_magic = 0; // LCOV_EXCL_LINE
-    } else { // LCOV_EXCL_LINE
-        is_magic = PyObjCObject_IsMagic(self);
-    }
-
-    if (is_magic) {
+    if (PyObjCObject_IsMagic(self)) {
         Py_RETURN_TRUE;
     } else {
         Py_RETURN_FALSE;
@@ -1124,7 +1017,7 @@ meth_is_magic(PyObject* self)
 static PyObject* _Nullable as_cobject(PyObject* self)
 {
     id self_value = PyObjCObject_OBJECT(self);
-    PyObjC_Assert(self_value != nil, NULL);
+    assert(self_value != nil);
 
     return PyCapsule_New(self_value, "objc.__object__", NULL);
 }
@@ -1133,7 +1026,7 @@ static PyObject* _Nullable as_cobject(PyObject* self)
 static PyObject* _Nullable as_ctypes_voidp(PyObject* self)
 {
     id self_value = PyObjCObject_OBJECT(self);
-    PyObjC_Assert(self_value != nil, NULL);
+    assert(self_value != nil);
     return PyObjC_MakeCVoidP(self_value);
 }
 
@@ -1161,10 +1054,13 @@ static PyObject* _Nullable meth_dir(PyObject* self)
         PyObject* args[] = { NULL, result, *dictptr };
         PyObject* tmp = PyObject_VectorcallMethod(PyObjCNM_update, args+1, 2|PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
         if (tmp == NULL) { // LCOV_BR_EXCL_LINE
-            return NULL; // LCOV_EXCL_LINE
+            // LCOV_EXCL_START
+            Py_CLEAR(result);
+            return NULL;
+            // LCOV_EXCL_STOP
         }
         Py_CLEAR(tmp);
-    }
+    } // LCOV_BR_EXCL_LINE
 
     if (PyObjCObject_IsMagic(self)) {
         /* "magic cookie" objects don't have Objective-C methods */
@@ -1291,7 +1187,7 @@ PyObject* _Nullable _PyObjCObject_NewDeallocHelper(id objc_object)
     PyObject*     res;
     PyTypeObject* cls_type;
 
-    PyObjC_Assert(objc_object != nil, NULL);
+    assert(objc_object != nil);
     cls_type =
         (PyTypeObject*)PyObjCClass_New((Class _Nonnull)object_getClass(objc_object));
     if (cls_type == NULL) { // LCOV_BR_EXCL_LINE
@@ -1318,7 +1214,11 @@ PyObject* _Nullable _PyObjCObject_NewDeallocHelper(id objc_object)
 void
 _PyObjCObject_FreeDeallocHelper(PyObject* obj)
 {
+#if PY_VERSION_HEX >= 0x030e0000
+    if (PyUnstable_Object_IsUniquelyReferenced(obj) != 1) {
+#else
     if (Py_REFCNT(obj) != 1) {
+#endif
         /* Someone revived this object. Objective-C doesn't like
          * this at all, therefore warn the user about this and
          * zero out the instance.
@@ -1329,36 +1229,23 @@ _PyObjCObject_FreeDeallocHelper(PyObject* obj)
                  Py_TYPE(obj)->tp_name);
 
         id objc_object = PyObjCObject_GetObject(obj);
-        if (objc_object != nil) {
+        assert(objc_object != nil);
 
-            if (((PyObjCObject*)obj)->flags & PyObjCObject_kSHOULD_NOT_RELEASE) {
-                /* pass */
+        PyObjC_UnregisterPythonProxy(objc_object, obj);
 
-            } else if (((PyObjCObject*)obj)->flags & PyObjCObject_kUNINITIALIZED) {
-                /* pass */
-
-            } else {
-                CFRelease(objc_object);
-            }
-
-            PyObjC_UnregisterPythonProxy(objc_object, obj);
-        }
-        ((PyObjCObject*)obj)->objc_object = nil;
-
-        Py_DECREF(obj);
+        /* Object is set to ``[NSNull null]`` to maintain the invariant */
+        ((PyObjCObject*)obj)->objc_object = NSNull_null;
 
         if (PyErr_Warn(PyObjCExc_ObjCRevivalWarning, buf) == -1) {
             PyErr_WriteUnraisable(obj);
         }
-
-        return;
     }
     Py_DECREF(obj);
 }
 
 PyObject* _Nullable PyObjCObject_New(id objc_object, int flags, int retain)
 {
-    PyObjC_Assert(objc_object != nil, NULL);
+    assert(objc_object != nil);
 
     /* object_getClass only returns Nil if its argument is nil */
     Class         cls = (Class _Nonnull)object_getClass(objc_object);
@@ -1410,9 +1297,9 @@ PyObject* _Nullable PyObjCObject_New(id objc_object, int flags, int retain)
     }
 
     if (retain) {
-        if (strcmp(object_getClassName(objc_object), "NSAutoreleasePool") != 0) {
+        if (object_getClass(objc_object) != NSAutoreleasePool_class) {
             /* NSAutoreleasePool doesn't like retain */
-            CFRetain(objc_object);
+            [objc_object retain];
         }
     }
 
@@ -1434,42 +1321,43 @@ PyObject* _Nullable PyObjCObject_FindSelector(PyObject* object, SEL selector)
     return PyObjCClass_FindSelector((PyObject*)Py_TYPE(object), selector, NO);
 }
 
-id _Nullable PyObjCObject_GetObject(PyObject* object)
+id PyObjCObject_GetObject(PyObject* object)
 {
-    PyObjC_Assert(PyObjCObject_Check(object), nil);
     id result = PyObjCObject_OBJECT(object);
     return result;
 }
 
 unsigned int PyObjCObject_GetFlags(PyObject* object)
 {
-    PyObjC_Assert(PyObjCObject_Check(object), 0xffffffff);
     return PyObjCObject_FLAGS(object);
 }
 
 bool PyObjCObject_IsBlock(PyObject* object)
 {
-    PyObjC_Assert(PyObjCObject_Check(object), false);
+    assert(PyObjCObject_Check(object));
     return (PyObjCObject_FLAGS(object) & PyObjCObject_kBLOCK) != 0;
 }
 
 bool PyObjCObject_IsMagic(PyObject* object)
 {
-    PyObjC_Assert(PyObjCObject_Check(object), false);
+    assert(PyObjCObject_Check(object));
     return (PyObjCObject_FLAGS(object) & PyObjCObject_kMAGIC_COOKIE) != 0;
 }
 
 PyObjCMethodSignature* _Nullable PyObjCObject_GetBlockSignature(PyObject* object)
 {
-    PyObjC_Assert(PyObjCObject_IsBlock(object), NULL);
-    PyObjCMethodSignature* result = (((PyObjCBlockObject*)(object))->signature);
+    PyObjCMethodSignature* result;
+    assert(PyObjCObject_IsBlock(object));
+    Py_BEGIN_CRITICAL_SECTION(object);
+    result = (((PyObjCBlockObject*)(object))->signature);
     Py_XINCREF(result);
+    Py_END_CRITICAL_SECTION();
     return result;
 }
 
 PyObjCMethodSignature* _Nullable PyObjCObject_SetBlockSignature(PyObject* object, PyObjCMethodSignature* methinfo)
 {
-    PyObjC_Assert(PyObjCObject_IsBlock(object), NULL);
+    assert(PyObjCObject_IsBlock(object));
     PyObjCMethodSignature* result;
 
     Py_BEGIN_CRITICAL_SECTION(object);
@@ -1485,14 +1373,6 @@ PyObjCMethodSignature* _Nullable PyObjCObject_SetBlockSignature(PyObject* object
 
 
 
-void
-PyObjCObject_ClearObject(PyObject* object)
-{
-    assert(PyObjCObject_Check(object));
-    PyObjC_UnregisterPythonProxy(((PyObjCObject*)object)->objc_object, object);
-    ((PyObjCObject*)object)->objc_object = nil;
-}
-
 PyObject* _Nullable PyObjCObject_GetAttr(PyObject* obj, PyObject* name)
 {
     return object_getattro(obj, name);
