@@ -7,11 +7,12 @@
 NS_ASSUME_NONNULL_BEGIN
 
 NSNull* NSNull_null;
+Class NSAutoreleasePool_class;
 
 PyObject* PyObjCExc_Error;
 PyObject* PyObjCExc_NoSuchClassError;
 PyObject* PyObjCExc_InternalError;
-PyObject* PyObjCExc_UnInitDeallocWarning;
+PyObject* PyObjCExc_UnInitDeallocWarning; /* XXX: Drop in PyObjC 12 */
 PyObject* PyObjCExc_ObjCRevivalWarning;
 PyObject* PyObjCExc_LockError;
 PyObject* PyObjCExc_BadPrototypeError;
@@ -64,6 +65,7 @@ PyObject* PyObjCNM_callable_retained;
 PyObject* PyObjCNM_deprecated;
 PyObject* PyObjCNM_deref_result_pointer;
 PyObject* PyObjCNM_free_result;
+PyObject* PyObjCNM_initializer;
 PyObject* PyObjCNM_full_signature;
 PyObject* PyObjCNM_null_accepted;
 PyObject* PyObjCNM_printf_format;
@@ -96,12 +98,14 @@ PyObject* PyObjCNM_pyobjcSetValue_;
 PyObject* PyObjCNM_tzinfo;
 PyObject* PyObjCNM_update;
 PyObject* PyObjCNM_co_consts;
+PyObject* PyObjCNM___call__;
 
 
 int
 PyObjCUtil_Init(PyObject* module)
 {
     NSNull_null = [[NSNull null] retain];
+    NSAutoreleasePool_class = [NSAutoreleasePool class];
 
 #define NEW_EXC(identifier, name, base_class)                                            \
     identifier = PyErr_NewException("objc." name, base_class, NULL);                     \
@@ -114,7 +118,7 @@ PyObjCUtil_Init(PyObject* module)
     NEW_EXC(PyObjCExc_Error, "error", NULL);
     NEW_EXC(PyObjCExc_NoSuchClassError, "nosuchclass_error", PyObjCExc_Error);
     NEW_EXC(PyObjCExc_InternalError, "internal_error", PyObjCExc_Error);
-    NEW_EXC(PyObjCExc_UnInitDeallocWarning, "UninitializedDeallocWarning", PyExc_Warning);
+    NEW_EXC(PyObjCExc_UnInitDeallocWarning, "UninitializedDeallocWarning", PyExc_Warning); /* XXX */
     NEW_EXC(PyObjCExc_ObjCRevivalWarning, "RevivedObjectiveCObjectWarning",
             PyExc_Warning);
     NEW_EXC(PyObjCExc_LockError, "LockError", PyObjCExc_Error);
@@ -177,6 +181,7 @@ PyObjCUtil_Init(PyObject* module)
     NEW_STR(PyObjCNM_deprecated, "deprecated");
     NEW_STR(PyObjCNM_deref_result_pointer, "deref_result_pointer");
     NEW_STR(PyObjCNM_free_result, "free_result");
+    NEW_STR(PyObjCNM_initializer, "initializer");
     NEW_STR(PyObjCNM_full_signature, "full_signature");
     NEW_STR(PyObjCNM_null_accepted, "null_accepted");
     NEW_STR(PyObjCNM_printf_format, "printf_format");
@@ -209,6 +214,7 @@ PyObjCUtil_Init(PyObject* module)
     NEW_STR(PyObjCNM_tzinfo, "tzinfo");
     NEW_STR(PyObjCNM_update, "update");
     NEW_STR(PyObjCNM_co_consts, "co_consts");
+    NEW_STR(PyObjCNM___call__, "__call__");
 
 #undef NEW_STR
 
@@ -244,8 +250,8 @@ PyObjCErr_FromObjC(NSObject* localException)
     PyObject*     exception;
     PyObject*     v;
     PyObject*     exc_type;
-    PyObject*     exc_value;
-    PyObject*     exc_traceback;
+    PyObject*     exc_value = NULL;
+    PyObject*     exc_traceback = NULL;
     PyObject*     c_localException_name;
     PyObject*     c_localException_reason;
 
@@ -282,16 +288,35 @@ PyObjCErr_FromObjC(NSObject* localException)
                 if (val) {
                     id temp;
                     exc_type = id_to_python(val);
+                    if (exc_type == NULL) { // LCOV_BR_EXCL_LINE
+                        PyObjC_GIL_RETURNVOID; // LCOV_EXCL_LINE
+                    } // LCOV_EXCL_LINE
 
-                    temp          = [userInfo objectForKey:@"__pyobjc_exc_value__"];
-                    exc_value     = temp != NULL ? id_to_python(temp) : NULL;
-                    temp          = [userInfo objectForKey:@"__pyobjc_exc_traceback__"];
-                    exc_traceback = temp != NULL ? id_to_python(temp) : NULL;
-
-                    if (exc_type != NULL) {
-                        PyErr_Restore(exc_type, exc_value, exc_traceback);
+                    temp = [userInfo objectForKey:@"__pyobjc_exc_value__"];
+                    if (temp != nil) {
+                        exc_value     = id_to_python(temp);
+                        if (exc_value == NULL) { // LCOV_BR_EXCL_LINE
+                            // LCOV_EXCL_START
+                            Py_CLEAR(exc_type);
+                            PyObjC_GIL_RETURNVOID;
+                            // LCOV_EXCL_STOP
+                        } // LCOV_EXCL_LINE
                     }
 
+                    temp = [userInfo objectForKey:@"__pyobjc_exc_traceback__"];
+                    if (temp != nil) {
+                        exc_traceback     = id_to_python(temp);
+                        if (exc_traceback == NULL) { // LCOV_BR_EXCL_LINE
+                            // LCOV_EXCL_START
+                            Py_CLEAR(exc_type);
+                            Py_CLEAR(exc_value);
+                            PyObjC_GIL_RETURNVOID;
+                            // LCOV_EXCL_STOP
+                        } // LCOV_EXCL_LINE
+                    }
+
+                    PyErr_Restore(exc_type, exc_value, exc_traceback);
+
                     PyObjC_GIL_RETURNVOID;
                 } // LCOV_EXCL_LINE
             }
@@ -351,7 +376,7 @@ PyObjCErr_FromObjC(NSObject* localException)
             if (reason != NULL) {
                 PyErr_Format(exception, "%s - %s", name ? name : "<null>", reason);
             } else {
-                PyErr_Format(exception, name ? name : "<null>");
+                PyErr_SetString(exception, name ? name : "<null>"); // LCOV_BR_EXCL_LINE
             }
             PyErr_Fetch(&exc_type, &exc_value, &exc_traceback);
             PyErr_NormalizeException(&exc_type, &exc_value, &exc_traceback);
@@ -381,7 +406,7 @@ static NSException* _Nullable python_exception_to_objc(void)
     NSException*         val;
     NSMutableDictionary* userInfo = nil;
 
-    PyObjC_Assert(PyErr_Occurred(), nil);
+    assert(PyErr_Occurred());
 
     PyErr_Fetch(&exc_type, &exc_value, &exc_traceback);
     if (exc_type == NULL) { // LCOV_BR_EXCL_LINE
@@ -477,9 +502,6 @@ static NSException* _Nullable python_exception_to_objc(void)
     userInfo = [NSMutableDictionary dictionaryWithCapacity:3];
 
     /* XXX: For recent enough versions of Python we don't need to store all three */
-    // PyObjC_Assert(exc_type != NULL, nil);
-    // PyObjC_Assert(exc_value != NULL, nil);
-    // PyObjC_Assert(exc_traceback != NULL, nil);
     [userInfo setObject:[[[OC_PythonObject alloc] initWithPyObject:exc_type] autorelease]
                  forKey:@"__pyobjc_exc_type__"];
 
@@ -585,21 +607,6 @@ PyObjC_FreeCArray(int code, Py_buffer* view)
     }
 }
 
-static inline PyTypeObject* _Nullable fetch_array_type(void)
-{
-    static PyTypeObject* array_type = NULL;
-
-    if (array_type != NULL) {
-        return array_type;
-
-    } else {
-        array_type = (PyTypeObject*)PyObjC_ImportName("array.ArrayType");
-        return array_type;
-    }
-}
-
-#define array_check(obj) PyObject_TypeCheck(obj, fetch_array_type())
-
 static char struct_elem_code(const char* typestr);
 
 static char
@@ -617,7 +624,7 @@ array_typestr(PyObject* array)
         return '\0'; // LCOV_EXCL_LINE
     }
 
-    if (PyUnicode_Check(typecode)) {
+    if (PyUnicode_Check(typecode)) { // LCOV_BR_EXCL_LINE
         bytes = PyUnicode_AsEncodedString(typecode, NULL, NULL);
         if (bytes == NULL) { // LCOV_BR_EXCL_LINE
             /* Should not happen: array.array typecode is an
@@ -757,7 +764,7 @@ struct_elem_code(const char* start_typestr)
 
         typestr = PyObjCRT_SkipTypeSpec(typestr);
     }
-    if (*typestr != _C_STRUCT_E) {
+    if (typestr == NULL || *typestr != _C_STRUCT_E) {
         return '\0';
     }
 
@@ -787,7 +794,7 @@ code_compatible(char array_code, char type_code)
         case _C_BOOL:
         case _C_NSBOOL:
             return YES;
-        }
+        } // LCOV_EXCL_LINE
 
     case 'B':
         switch (type_code) {
@@ -799,10 +806,12 @@ code_compatible(char array_code, char type_code)
              * compatibility.
              */
             return YES;
-        }
+        } // LCOV_EXCL_LINE
 
     case 'u':
+#if PY_VERSION_HEX >= 0x030d0000
     case 'w':
+#endif
         return type_code == _C_INT;
 
     case 'h':
@@ -827,7 +836,7 @@ code_compatible(char array_code, char type_code)
         case _C_LNG:
         case _C_LNG_LNG:
             return YES;
-        }
+        } // LCOV_EXCL_LINE
 
     case 'L':
     case 'Q':
@@ -835,7 +844,7 @@ code_compatible(char array_code, char type_code)
         case _C_ULNG:
         case _C_ULNG_LNG:
             return YES;
-        }
+        } // LCOV_EXCL_LINE
 
     case 'f':
         return type_code == _C_FLT;
@@ -844,7 +853,12 @@ code_compatible(char array_code, char type_code)
         return type_code == _C_DBL;
 
     }
-    return NO;
+
+    /* Should not get here, the switch
+     * above is up-to-date with the array.array
+     * type codes as of Python 3.13
+     */
+    return NO; // LCOV_EXCL_LINE
 }
 
 /*
@@ -926,13 +940,15 @@ PyObjC_PythonToCArray(BOOL writable, BOOL exactSize, const char* elementType,
 
                 PyBuffer_Release(view);
 
-                if (byte_array == NULL) {
-                    return -1;
+                if (byte_array == NULL) { // LCOV_BR_EXCL_LINE
+                    return -1; // LCOV_EXCL_LINE
                 }
 
-                if (PyObject_GetBuffer(byte_array, view, PyBUF_CONTIG) == -1) {
+                if (PyObject_GetBuffer(byte_array, view, PyBUF_CONTIG) == -1) { // LCOV_BR_EXCL_LINE
+                    // LCOV_EXCL_START
                     Py_DECREF(byte_array);
                     return -1;
+                    // LCOV_EXCL_STOP
                 }
 
                 Py_DECREF(byte_array); /* Reference is kept by the view */
@@ -1047,7 +1063,7 @@ PyObjC_PythonToCArray(BOOL writable, BOOL exactSize, const char* elementType,
          */
     }
 
-    if (array_check(pythonList)) {
+    if (PyObjC_ArrayTypeCheck(pythonList)) {
         /* An array.array. Only convert if the typestr describes an
          * simple type of the same type as the array, or a struct/array
          * containing only elements of the type of the array.
@@ -1234,8 +1250,8 @@ PyObject* _Nullable PyObjC_CArrayToPython(const char* elementType, const void* a
     Py_ssize_t eltsize;
 
     eltsize = PyObjCRT_SizeOfType(elementType);
-    if (eltsize == -1) {
-        return NULL;
+    if (eltsize == -1) { // LCOV_BR_EXCL_LINE
+        return NULL; // LCOV_EXCL_LINE
     }
 
     if (eltsize == 1 || eltsize == 0) {
@@ -1248,7 +1264,7 @@ PyObject* _Nullable PyObjC_CArrayToPython(const char* elementType, const void* a
             /* Special case for buffer-like objects */
             return PyBytes_FromStringAndSize(array, size);
         }
-    }
+    } // LCOV_EXCL_LINE
 
     if (*elementType == _C_UNICHAR) {
         int byteorder = 0;
@@ -1339,12 +1355,13 @@ PyObject* _Nullable PyObjC_CArrayToPython2(const char* elementType, const void*
     Py_ssize_t eltsize;
 
     if (size == -1) {
+        /* XXX: Can this happen? */
         size = 0;
     }
 
     eltsize = PyObjCRT_SizeOfType(elementType);
-    if (eltsize == -1) {
-        return NULL;
+    if (eltsize == -1) { // LCOV_BR_EXCL_LINE
+        return NULL; // LCOV_EXCL_LINE
     }
 
     if (eltsize == 1 || eltsize == 0) {
@@ -1411,7 +1428,7 @@ PyObjCClass_Convert(PyObject* object, void* pvar)
     }
 
     *(Class*)pvar = PyObjCClass_GetClass(object);
-    if (*(Class*)pvar == NULL) // LCOC_BR_EXCL_LINE
+    if (*(Class*)pvar == NULL) // LCOV_BR_EXCL_LINE
         return 0; // LCOV_EXCL_LINE
     return 1;
 }
@@ -1444,39 +1461,6 @@ PyObjC_is_ascii_prefix(PyObject* unicode_string, const char* ascii_string, size_
     return strncmp((const char*)(PyUnicode_DATA(unicode_string)), ascii_string, n) == 0;
 }
 
-PyObject* _Nullable PyObjC_ImportName(const char* name)
-{
-    PyObject* py_name;
-    PyObject* mod;
-    char*     c = strrchr(name, '.');
-
-    if (c == NULL) {
-        /* Toplevel module */
-        py_name = PyUnicode_FromString(name);
-        if (py_name == NULL) { // LCOV_BR_EXCL_LINE
-            return NULL; // LCOV_EXCL_LINE
-        }
-        mod     = PyImport_Import(py_name);
-        Py_DECREF(py_name);
-        return mod;
-
-    } else {
-        py_name = PyUnicode_FromStringAndSize(name, c - name);
-        if (py_name == NULL) { // LCOV_BR_EXCL_LINE
-            return NULL; // LCOV_EXCL_LINE
-        }
-        mod     = PyImport_Import(py_name);
-        Py_DECREF(py_name);
-        if (mod == NULL) {
-            return NULL;
-        }
-
-        PyObject* v = PyObject_GetAttrString(mod, c + 1);
-        Py_DECREF(mod);
-        return v;
-    }
-}
-
 PyObject* _Nullable PyObjC_AdjustSelf(PyObject* object)
 {
     if (PyType_Check(object)
@@ -1604,24 +1588,6 @@ PyObjC_CheckNoKwnames(PyObject* callable, PyObject* _Nullable kwnames)
     return -1;
 }
 
-PyObject* _Nullable PyObjC_MakeCVoidP(void* ptr)
-{
-    PyObject* c_void_p = PyObjC_get_c_void_p();
-    if (c_void_p == NULL) {
-        return NULL;
-    }
-
-    PyObject* pyptr = PyLong_FromVoidPtr(ptr);
-    if (pyptr == NULL) {
-        return NULL;
-    }
-    PyObject* args[2] = {NULL, pyptr};
-    PyObject* res =
-        PyObject_Vectorcall(c_void_p, args + 1, 1 | PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
-    Py_DECREF(pyptr);
-    return res;
-}
-
 bool
 version_is_deprecated(int version)
 {
@@ -1792,46 +1758,5 @@ PyObjCSequence_Tuple(PyObject* value, const char* context)
     return result;
 }
 
-/*
- * XXX: Make this a static variable set during module init
- */
-PyObject* _Nullable PyObjC_get_c_void_p(void)
-{
-#ifdef Py_GIL_DISABLED
-    static PyMutex c_void_p_mutex = { 0 };
-#endif
-    static PyObject* c_void_p = NULL;
-    if (c_void_p == NULL) {
-#ifdef Py_GIL_DISABLED
-        PyMutex_Lock(&c_void_p_mutex);
-        if (c_void_p != NULL) {
-            PyMutex_Unlock(&c_void_p_mutex);
-            return c_void_p;
-        }
-#endif
-        PyObject* mod_ctypes = PyImport_ImportModule("ctypes");
-        if (mod_ctypes == NULL) {
-            /* ctypes is not available */
-#ifdef Py_GIL_DISABLED
-            PyMutex_Unlock(&c_void_p_mutex);
-#endif
-            return NULL;
-        }
-
-        c_void_p = PyObject_GetAttrString(mod_ctypes, "c_void_p");
-        Py_DECREF(mod_ctypes);
-        if (c_void_p == NULL) {
-            /* invalid or incomplete module */
-#ifdef Py_GIL_DISABLED
-            PyMutex_Unlock(&c_void_p_mutex);
-#endif
-            return NULL;
-        }
-#ifdef Py_GIL_DISABLED
-        PyMutex_Unlock(&c_void_p_mutex);
-#endif
-    }
-    return c_void_p;
-}
 
 NS_ASSUME_NONNULL_END
