@@ -8,6 +8,9 @@
  */
 #include "pyobjc.h"
 
+#include <netdb.h>
+
+
 NS_ASSUME_NONNULL_BEGIN
 
 /*
@@ -76,7 +79,7 @@ static PyObject* PyObjCOptions_Type;
     }
 
 #define INT_PROP(NAME, VAR, DFLT)                                                        \
-    PyObjC_ATOMIC int VAR = DFLT;                                                                      \
+    PyObjC_ATOMIC int VAR = DFLT;                                                        \
     MUTEX_FOR(VAR)                                                                       \
                                                                                          \
     static PyObject* NAME##_get(PyObject* s __attribute__((__unused__)),                 \
@@ -105,7 +108,7 @@ static PyObject* PyObjCOptions_Type;
     }
 
 #define BOOL_PROP(NAME, VAR, DFLT)                                                       \
-    PyObjC_ATOMIC BOOL VAR = DFLT;                                                                     \
+    PyObjC_ATOMIC BOOL VAR = DFLT;                                                       \
                                                                                          \
     static PyObject* NAME##_get(PyObject* s __attribute__((__unused__)),                 \
                                 void*     c __attribute__((__unused__)))                 \
@@ -125,7 +128,11 @@ static PyObject* PyObjCOptions_Type;
                             "Cannot delete option '" STR(NAME) "'");                     \
             return -1;                                                                   \
         }                                                                                \
-        VAR = PyObject_IsTrue(newVal) ? YES : NO;                                        \
+        int b = PyObject_IsTrue(newVal);                                                 \
+        if (b == -1) {                                                                   \
+            return -1;                                                                   \
+        }                                                                                \
+        VAR = b ? YES : NO;                                                              \
         return 0;                                                                        \
     }
 
@@ -222,6 +229,11 @@ OBJECT_PROP_STATIC(_transformAttribute, PyObjC_transformAttribute)
 OBJECT_PROP_STATIC(_processClassDict, PyObjC_processClassDict)
 OBJECT_PROP_STATIC(_setDunderNew, PyObjC_setDunderNew)
 OBJECT_PROP_STATIC(_genericNewClass, PyObjC_genericNewClass)
+OBJECT_PROP_STATIC(_ArrayType, PyObjC_ArrayType)
+OBJECT_PROP_STATIC(_deepcopy, PyObjC_deepcopyFunc)
+OBJECT_PROP_STATIC(_socket_error, PyObjC_socket_error)
+OBJECT_PROP_STATIC(_socket_gaierror, PyObjC_socket_gaierror)
+OBJECT_PROP_STATIC(_c_void_p, PyObjC_c_void_p)
 
 static PyObject*
 bundle_hack_get(PyObject* s __attribute__((__unused__)),
@@ -367,6 +379,16 @@ static PyGetSetDef options_getset[] = {
            "Private helper used for setting __new__ of a new Python subclass"),
     GETSET(_genericNewClass,
            "Class of the generic __new__ implementation"),
+    GETSET(_ArrayType,
+           "set to array.ArrayType"),
+    GETSET(_deepcopy,
+           "set to copy.deepcopy"),
+    GETSET(_socket_error,
+           "set to socket.error"),
+    GETSET(_socket_gaierror,
+           "set to socket.gaierror"),
+    GETSET(_c_void_p,
+           "set to ctypes.c_void_p"),
     {
         .name = "deprecation_warnings",
         .get  = deprecation_warnings_get,
@@ -921,7 +943,7 @@ PyObject* _Nullable PyObjC_GetCallableSignature(PyObject* callable, void* _Nulla
 
 int PyObjC_CallClassExtender(PyObject* cls)
 {
-    PyObjC_Assert(PyObjCClass_Check(cls), -1);
+    assert(PyObjCClass_Check(cls));
 
     LOCK(PyObjC_ClassExtender);
     PyObject* func = PyObjC_ClassExtender;
@@ -969,9 +991,51 @@ int PyObjC_CallClassExtender(PyObject* cls)
              */
             Py_INCREF(k);
             Py_INCREF(v);
-            if (PyType_Type.tp_setattro(cls, k, v) == -1) {
-                PyErr_Clear();
+
+#ifdef Py_GIL_DISABLED
+            /* free-threading: First check if the attribute is already set
+             * to the "new" value, and avoid resetting the attribute to the
+             * same value.
+             *
+             * In python 3.13 and 3.14 there is a race condition in
+             * the implementation of updating type slots.
+             *
+             * That race condition is fixed in 3.15, but that makes updating
+             * slots expensive, making it important to avoid spurious updates.
+             */
+            PyObject* c;
+            int r = PyDict_GetItemRef(((PyTypeObject*)cls)->tp_dict, k, &c);
+            if (r == -1) { // LCOV_BR_EXCL_LINE
+                // LCOV_EXCL_START
+                Py_CLEAR(k);
+                Py_CLEAR(v);
+                Py_CLEAR(dict);
+                return -1;
+                // LCOV_EXCL_STOP
             }
+
+            if (c == NULL) {
+                r = 0;
+            } else {
+                r = PyObject_RichCompareBool(c, v, Py_EQ);
+            }
+            switch (r) {
+            case -1:
+                Py_CLEAR(k);
+                Py_CLEAR(v);
+                Py_CLEAR(dict);
+                return -1;
+            case 0:
+#endif
+                if (PyType_Type.tp_setattro(cls, k, v) == -1) {
+                    PyErr_Clear();
+                }
+#ifdef Py_GIL_DISABLED
+            /* case 1: */
+                /* pass */
+            }
+            Py_CLEAR(c);
+#endif
             Py_CLEAR(k);
             Py_CLEAR(v);
 
@@ -988,6 +1052,53 @@ int PyObjC_CallClassExtender(PyObject* cls)
     return 0;
 }
 
+void PyObjCErr_SetGAIError(int error)
+{
+    PyObject* type;
+
+    if (error == EAI_SYSTEM) { // LCOV_BR_EXCL_LINE
+        /* This can happen, but haven't found a way to trigger
+         * this in testing.
+         */
+        // LCOV_EXCL_START
+        LOCK(PyObjC_socket_error);
+        type = PyObjC_socket_error;
+        Py_INCREF(type);
+        UNLOCK(PyObjC_socket_error);
+        PyErr_SetFromErrno(type);
+        Py_DECREF(type);
+        return;
+        // LCOV_EXCL_STOP
+    }
+
+    LOCK(PyObjC_socket_gaierror);
+    type = PyObjC_socket_gaierror;
+    Py_INCREF(type);
+    UNLOCK(PyObjC_socket_gaierror);
+
+    PyObject* v = Py_BuildValue("is", error, gai_strerror(error));
+    if (v != NULL) { // LCOV_BR_EXCL_LINE
+        PyErr_SetObject(type, v);
+        Py_DECREF(v);
+    }
+    Py_DECREF(type);
+}
+
+PyObject* _Nullable PyObjCErr_SetSocketError(const char* message)
+{
+    PyObject* type;
+    LOCK(PyObjC_socket_error);
+    type = PyObjC_socket_error;
+    Py_INCREF(type);
+    UNLOCK(PyObjC_socket_error);
+
+    PyErr_SetString(type, message);
+    Py_DECREF(type);
+    return NULL;
+}
+
+
+
 /*
  * Returns NULL without setting an exception when
  * the option is not set.
@@ -1106,6 +1217,70 @@ int PyObjC_IsGenericNew(PyObject* value)
     return r;
 }
 
+int PyObjC_ArrayTypeCheck(PyObject* value)
+{
+    LOCK(PyObjC_genericNewClass);
+    PyObject* type = PyObjC_ArrayType;
+    Py_INCREF(type);
+    UNLOCK(PyObjC_genericNewClass);
+
+    if (type == Py_None) {
+        return 0;
+    }
+
+    int r = PyObject_TypeCheck(value, (PyTypeObject*)type);
+    Py_DECREF(type);
+    if (r == -1) { // LCOV_BR_EXCL_LINE
+        // LCOV_EXCL_START
+        PyErr_Clear();
+        return 0;
+        // LCOV_EXCL_STOP
+    }
+    return r;
+}
+
+PyObject* _Nullable PyObjC_MakeCVoidP(void* ptr)
+{
+    PyObject* type;
+
+    LOCK(PyObjC_c_void_p);
+    type = PyObjC_c_void_p;
+    Py_INCREF(type);
+    UNLOCK(PyObjC_c_void_p);
+
+    PyObject* pyptr = PyLong_FromVoidPtr(ptr);
+    if (pyptr == NULL) { // LCOV_BR_EXCL_LINE
+        return NULL; // LCOV_EXCL_LINE
+    }
+    PyObject* args[2] = {NULL, pyptr};
+    PyObject* res =
+        PyObject_Vectorcall(type, args + 1, 1 | PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
+    Py_DECREF(pyptr);
+    Py_DECREF(type);
+    return res;
+}
+
+PyObject* _Nullable PyObjC_deepcopy(PyObject* value, PyObject* _Nullable memo)
+{
+    PyObject* func;
+    LOCK(PyObjC_deepcopyFunc);
+    func = PyObjC_deepcopyFunc;
+    Py_INCREF(func);
+    UNLOCK(PyObjC_deepcopyFunc);
+
+    if (func == Py_None) {
+        PyErr_SetString(PyObjCExc_Error, "options._deepcopy is not set");
+        Py_DECREF(func);
+        return NULL;
+    }
+
+    PyObject* args[3] = {NULL, value, memo};
+    PyObject* result       = PyObject_Vectorcall(
+            func, args + 1, (memo?2:1) | PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
+    Py_DECREF(func);
+    return result;
+}
+
 extern PyObject* _Nullable PyObjC_ProcessClassDict(const char* name, PyObject* class_dict,
                                                    PyObject* meta_dict, PyObject* py_superclass,
                                                    PyObject* protocols, PyObject* hiddenSelectors,
@@ -1132,7 +1307,7 @@ extern PyObject* _Nullable PyObjC_ProcessClassDict(const char* name, PyObject* c
         // LCOV_EXCL_STOP
     }
 
-    PyObjC_Assert(!PyErr_Occurred(), NULL);
+    assert(!PyErr_Occurred());
     PyObject* args[] = {NULL,      py_name, class_dict,      meta_dict,           py_superclass,
                         protocols, hiddenSelectors, hiddenClassSelectors};
     PyObject* rv     = PyObject_Vectorcall(func, args + 1,
@@ -1140,7 +1315,7 @@ extern PyObject* _Nullable PyObjC_ProcessClassDict(const char* name, PyObject* c
     Py_DECREF(func);
     Py_DECREF(py_name);
 
-    PyObjC_Assert(rv == NULL || !PyErr_Occurred(), NULL);
+    assert(rv == NULL || !PyErr_Occurred());
 
     return rv;
 }
@@ -1238,6 +1413,12 @@ PyObjC_SetupOptions(PyObject* m)
     INIT(PyObjC_processClassDict);
     INIT(PyObjC_setDunderNew);
     INIT(PyObjC_genericNewClass);
+    INIT(PyObjC_genericNewClass);
+    INIT(PyObjC_ArrayType);
+    INIT(PyObjC_deepcopyFunc);
+    INIT(PyObjC_socket_error);
+    INIT(PyObjC_socket_gaierror);
+    INIT(PyObjC_c_void_p);
 #undef INIT
 
     // LCOV_BR_EXCL_START
