@@ -25,7 +25,7 @@ static PyObject* _Nullable pysel_new(PyTypeObject* type, PyObject* _Nullable arg
 
 PyObjCMethodSignature* _Nullable PyObjCSelector_GetMetadata(PyObject* _self)
 {
-    PyObjC_Assert(PyObjCSelector_Check(_self), NULL);
+    assert(PyObjCSelector_Check(_self));
 
     PyObjCSelector* self = (PyObjCSelector*)_self;
 
@@ -43,8 +43,8 @@ PyObjCMethodSignature* _Nullable PyObjCSelector_GetMetadata(PyObject* _self)
             self->sel_selector, self->sel_python_signature,
             PyObjCNativeSelector_Check(self));
 
-        if (methinfo == NULL)
-            return NULL;
+        if (methinfo == NULL) // LCOV_BR_EXCL_LINE
+            return NULL; // LCOV_EXCL_LINE
 
         Py_BEGIN_CRITICAL_SECTION(self);
 #ifdef Py_GIL_DISABLED
@@ -94,8 +94,8 @@ static PyObject* _Nullable sel_metadata(PyObject* self)
 {
     int                    r;
     PyObjCMethodSignature* mi = PyObjCSelector_GetMetadata(self);
-    if (mi == NULL) {
-        return NULL;
+    if (mi == NULL) { // LCOV_BR_EXCL_LINE
+        return NULL; // LCOV_EXCL_LINE
     }
 
     PyObject* result = PyObjCMethodSignature_AsDict(mi);
@@ -127,15 +127,6 @@ static PyObject* _Nullable sel_metadata(PyObject* self)
         // LCOV_EXCL_STOP
     }
 
-    if (((PyObjCSelector*)self)->sel_flags & PyObjCSelector_kRETURNS_UNINITIALIZED) {
-        r = PyDict_SetItem(result, PyObjCNM_return_uninitialized_object, Py_True);
-        if (r == -1) { // LCOV_BR_EXCL_LINE
-            // LCOV_EXCL_START
-            Py_DECREF(result);
-            return NULL;
-            // LCOV_EXCL_STOP
-        }
-    }
     return result;
 }
 
@@ -171,8 +162,8 @@ static PyObject* _Nullable base_signature(PyObject* self,
                                           void*     closure __attribute__((__unused__)))
 {
     PyObjCMethodSignature* methinfo = PyObjCSelector_GetMetadata(self);
-    if (methinfo == NULL) {
-        return NULL;
+    if (methinfo == NULL) { // LCOV_BR_EXCL_LINE
+        return NULL; // LCOV_EXCL_LINE
     }
 
     PyObject* result = PyBytes_FromString(methinfo->signature);
@@ -259,7 +250,11 @@ base_hidden_setter(PyObject* _self, PyObject* newVal,
         return -1;
     }
 
-    if (PyObject_IsTrue(newVal)) {
+    int hidden = PyObject_IsTrue(newVal);
+    if (hidden == -1) {
+        return -1;
+    }
+    if (hidden) {
         Py_BEGIN_CRITICAL_SECTION(_self);
         ((PyObjCSelector*)_self)->sel_flags |= PyObjCSelector_kHIDDEN;
         Py_END_CRITICAL_SECTION();
@@ -559,9 +554,6 @@ static PyObject* _Nullable objcsel_vectorcall_simple(
 {
     PyObjCNativeSelector*  self   = (PyObjCNativeSelector*)_self;
     PyObject*              pyself = self->base.sel_self;
-    PyObject*              res;
-    PyObject*              pyres;
-    PyObjCMethodSignature* methinfo;
 
     if (PyObjC_CheckNoKwnames(_self, kwnames) == -1) {
         return NULL;
@@ -573,40 +565,43 @@ static PyObject* _Nullable objcsel_vectorcall_simple(
         }
 
         pyself = args[0];
-        PyObjC_Assert(pyself != NULL, NULL);
+        assert(pyself != NULL);
 
         args   = args + 1;
         nargsf = PyVectorcall_NARGS(nargsf) - 1;
     }
 
-    methinfo = PyObjCSelector_GetMetadata(_self);
-    if (methinfo == NULL) {
-        return NULL;
-    }
-
-    if (version_is_deprecated(methinfo->deprecated)) {
-        char  buf[128];
-        Class cls = PyObjCSelector_GetClass(_self);
-        SEL   sel = PyObjCSelector_GetSelector(_self);
-
-        snprintf(buf, 128, "%c[%s %s] is a deprecated API (macOS %d.%d)",
-                 PyObjCSelector_IsClassMethod(_self) ? '+' : '-', class_getName(cls),
-                 sel_getName(sel), methinfo->deprecated / 100,
-                 methinfo->deprecated % 100);
-        if (PyErr_Warn(PyObjCExc_DeprecationWarning, buf) < 0) {
-            Py_CLEAR(methinfo);
-            return NULL;
+    {
+        PyObjCMethodSignature* methinfo = PyObjCSelector_GetMetadata(_self);
+        if (methinfo == NULL) { // LCOV_BR_EXCL_LINE
+            return NULL; // LCOV_EXCL_LINE
         }
+
+        if (version_is_deprecated(methinfo->deprecated)) {
+            char  buf[128];
+            Class cls = PyObjCSelector_GetClass(_self);
+            SEL   sel = PyObjCSelector_GetSelector(_self);
+
+            snprintf(buf, 128, "%c[%s %s] is a deprecated API (macOS %d.%d)",
+                     PyObjCSelector_IsClassMethod(_self) ? '+' : '-', class_getName(cls),
+                     sel_getName(sel), methinfo->deprecated / 100,
+                     methinfo->deprecated % 100);
+            if (PyErr_Warn(PyObjCExc_DeprecationWarning, buf) < 0) {
+                Py_CLEAR(methinfo);
+                return NULL;
+            }
+        }
+        Py_CLEAR(methinfo);
     }
+
     if (!(self->base.sel_flags & PyObjCSelector_kCLASS_METHOD)) {
         /*
          * Class methods should always be bound, therefore we only
          * have to validate the type of self for instance methods.
          */
-        PyObjC_Assert(self->base.sel_class != Nil, NULL);
+        assert(self->base.sel_class != Nil);
         PyObject* myClass = PyObjCClass_New(self->base.sel_class);
         if (myClass == NULL) { // LCOV_BR_EXCL_LINE
-            Py_CLEAR(methinfo);
             return NULL;       // LCOV_EXCL_LINE
         }
 
@@ -616,7 +611,6 @@ static PyObject* _Nullable objcsel_vectorcall_simple(
                                                [NSString class])))) {
 
             Py_DECREF(myClass);
-            Py_CLEAR(methinfo);
             PyErr_Format(PyExc_TypeError,
                          "Expecting instance of %s as self, got one "
                          "of %s",
@@ -625,34 +619,7 @@ static PyObject* _Nullable objcsel_vectorcall_simple(
         }
         Py_DECREF(myClass);
     }
-
-    pyres = res = PyObjCFFI_Caller_SimpleSEL((PyObject*)self, pyself, args, nargsf);
-    if (pyres != NULL && PyTuple_Check(pyres) && PyTuple_GET_SIZE(pyres) >= 1
-        && PyTuple_GET_ITEM(pyres, 0) == pyself) {
-
-        pyres = pyself;
-    }
-
-    if (PyObjCObject_Check(pyself)
-        && (((PyObjCObject*)pyself)->flags & PyObjCObject_kUNINITIALIZED)) {
-        if (pyself != pyres && !PyErr_Occurred()) {
-            PyObjCObject_ClearObject(pyself);
-        }
-    }
-
-    if (pyres && PyObjCObject_Check(pyres)) {
-        if (self->base.sel_flags & PyObjCSelector_kRETURNS_UNINITIALIZED) {
-            ((PyObjCObject*)pyres)->flags |= PyObjCObject_kUNINITIALIZED;
-        } else if (((PyObjCObject*)pyself)->flags & PyObjCObject_kUNINITIALIZED) {
-            ((PyObjCObject*)pyself)->flags &= ~PyObjCObject_kUNINITIALIZED;
-            if (self->base.sel_self && self->base.sel_self != pyres
-                && !PyErr_Occurred()) {
-                PyObjCObject_ClearObject(self->base.sel_self);
-            }
-        }
-    }
-    Py_CLEAR(methinfo);
-    return res;
+    return PyObjCFFI_Caller_SimpleSEL((PyObject*)self, pyself, args, nargsf);
 }
 #endif
 
@@ -670,8 +637,6 @@ static PyObject* _Nullable objcsel_vectorcall(PyObject* _self,
     PyObject*              pyself  = self->base.sel_self;
     PyObjC_CallFunc        execute = NULL;
     PyObject*              res;
-    PyObject*              pyres;
-    PyObjCMethodSignature* methinfo;
 
     if (PyObjC_CheckNoKwnames(_self, kwnames) == -1) {
         return NULL;
@@ -686,85 +651,75 @@ static PyObject* _Nullable objcsel_vectorcall(PyObject* _self,
         }
 
         pyself = args[0];
-        if (pyself == NULL) {
-            return NULL;
+        if (pyself == NULL) { // LCOV_BR_EXCL_LINE
+            // Guard against invalid input data
+            return NULL; // LCOV_EXCL_LINE
         }
     }
 
-    methinfo = PyObjCSelector_GetMetadata(_self);
-    if (methinfo == NULL) {
-        return NULL;
-    }
-
-    if (version_is_deprecated(methinfo->deprecated)) {
-        char  buf[128];
-        Class cls = PyObjCSelector_GetClass(_self);
-        SEL   sel = PyObjCSelector_GetSelector(_self);
-
-        assert(cls);
-        assert(sel);
-
-        snprintf(buf, 128, "%c[%s %s] is a deprecated API (macOS %d.%d)",
-                 PyObjCSelector_IsClassMethod(_self) ? '+' : '-', class_getName(cls),
-                 sel_getName(sel), methinfo->deprecated / 100,
-                 methinfo->deprecated % 100);
-        if (PyErr_Warn(PyObjCExc_DeprecationWarning, buf) < 0) {
-            Py_CLEAR(methinfo);
-            return NULL;
+    {
+        PyObjCMethodSignature* methinfo = PyObjCSelector_GetMetadata(_self);
+        if (methinfo == NULL) { // LCOV_BR_EXCL_LINE
+            return NULL; // LCOV_EXCL_LINE
         }
+
+        if (version_is_deprecated(methinfo->deprecated)) {
+            char  buf[128];
+            Class cls = PyObjCSelector_GetClass(_self);
+            SEL   sel = PyObjCSelector_GetSelector(_self);
+
+            assert(cls);
+            assert(sel);
+
+            snprintf(buf, 128, "%c[%s %s] is a deprecated API (macOS %d.%d)",
+                     PyObjCSelector_IsClassMethod(_self) ? '+' : '-', class_getName(cls),
+                     sel_getName(sel), methinfo->deprecated / 100,
+                     methinfo->deprecated % 100);
+            if (PyErr_Warn(PyObjCExc_DeprecationWarning, buf) < 0) {
+                Py_CLEAR(methinfo);
+                return NULL;
+            }
+        }
+        Py_CLEAR(methinfo);
     }
 
-    if (self->sel_call_func) {
+    if (self->sel_call_func != NULL) {
         execute = self->sel_call_func;
     } else {
-        PyObjC_Assert(self->base.sel_class != NULL, NULL);
+        assert(self->base.sel_class != NULL);
 
         execute = PyObjC_FindCallFunc(self->base.sel_class, self->base.sel_selector,
                                       self->base.sel_methinfo->signature);
         if (execute == NULL) {
-            Py_CLEAR(methinfo);
             return NULL;
         }
-        self->sel_call_func = execute;
+
+        if (self->sel_call_func == NULL) {
+            self->sel_call_func = execute;
 #if PY_VERSION_HEX >= 0x03090000
-        /* Update the vectorcall slot when a faster call is possible */
-        if (methinfo->shortcut_signature && execute == PyObjCFFI_Caller) {
-            PyObjC_Assert(methinfo->shortcut_signature, NULL);
-            self->base.sel_vectorcall = objcsel_vectorcall_simple;
-        }
+            /* Update the vectorcall slot when a faster call is possible */
+            if (self->base.sel_methinfo->shortcut_signature && execute == PyObjCFFI_Caller) {
+                self->base.sel_vectorcall = objcsel_vectorcall_simple;
+            }
 #endif
+        } else {
+            Py_CLEAR(execute);
+            execute = self->sel_call_func;
+        }
     }
 
     /* XXX: The if statement below can be simplified, both cases are mostly the same */
     if (self->base.sel_self != NULL) {
-        pyres = res = execute((PyObject*)self, self->base.sel_self, args,
+        res = execute((PyObject*)self, self->base.sel_self, args,
                               PyVectorcall_NARGS(nargsf));
-        if (pyres != NULL && PyTuple_Check(pyres) && PyTuple_GET_SIZE(pyres) >= 1
-            && PyTuple_GET_ITEM(pyres, 0) == pyself) {
-
-            pyres = pyself;
-        }
-
-        /* XXX: The test here seems dodgy it will reset the sel_self
-         *      for a partially initialized object on the first method
-         *      call, even if that's not an init method.
-         */
-        if (PyObjCObject_Check(self->base.sel_self)
-            && (((PyObjCObject*)self->base.sel_self)->flags
-                & PyObjCObject_kUNINITIALIZED)) {
-            if (self->base.sel_self != pyres && !PyErr_Occurred()) {
-                PyObjCObject_ClearObject(self->base.sel_self);
-            }
-        }
 
     } else {
         PyObject* myClass;
 
-        PyObjC_Assert(self->base.sel_class != Nil, NULL);
+        assert(self->base.sel_class != Nil);
 
         myClass = PyObjCClass_New(self->base.sel_class);
         if (myClass == NULL) { // LCOV_BR_EXCL_LINE
-            Py_CLEAR(methinfo);
             return NULL;       // LCOV_EXCL_LINE
         }
         if (!(PyObject_IsInstance(pyself, myClass)
@@ -773,7 +728,6 @@ static PyObject* _Nullable objcsel_vectorcall(PyObject* _self,
                                                [NSString class])))) {
 
             Py_DECREF(myClass);
-            Py_CLEAR(methinfo);
             PyErr_Format(PyExc_TypeError,
                          "Expecting instance of %s as self, got one "
                          "of %s",
@@ -782,39 +736,9 @@ static PyObject* _Nullable objcsel_vectorcall(PyObject* _self,
         }
         Py_DECREF(myClass);
 
-        pyres = res =
+        res =
             execute((PyObject*)self, pyself, args + 1, PyVectorcall_NARGS(nargsf) - 1);
-        if (pyres != NULL && PyTuple_Check(pyres) && PyTuple_GET_SIZE(pyres) > 1
-            && PyTuple_GET_ITEM(pyres, 0) == pyself) {
-            pyres = pyself;
-        }
-
-        if (PyObjCObject_Check(pyself)
-            && (((PyObjCObject*)pyself)->flags & PyObjCObject_kUNINITIALIZED)) {
-            if (pyself != pyres && !PyErr_Occurred()) {
-                PyObjCObject_ClearObject(pyself);
-            }
-        }
     }
-
-    if (pyres && PyObjCObject_Check(pyres)) {
-        if (self->base.sel_flags & PyObjCSelector_kRETURNS_UNINITIALIZED) {
-            Py_BEGIN_CRITICAL_SECTION(self);
-            ((PyObjCObject*)pyres)->flags |= PyObjCObject_kUNINITIALIZED;
-            Py_END_CRITICAL_SECTION();
-        } else if (((PyObjCObject*)pyself)->flags & PyObjCObject_kUNINITIALIZED) {
-            Py_BEGIN_CRITICAL_SECTION(self);
-            ((PyObjCObject*)pyself)->flags &= ~PyObjCObject_kUNINITIALIZED;
-            Py_END_CRITICAL_SECTION();
-            if (self->base.sel_self && self->base.sel_self != pyres
-                && !PyErr_Occurred()) {
-                /* XXX: This needs documentation, the logic is not 100% rigorous */
-                PyObjCObject_ClearObject(self->base.sel_self);
-            }
-        }
-    }
-
-    Py_CLEAR(methinfo);
     return res;
 }
 
@@ -871,6 +795,7 @@ static PyObject* _Nullable objcsel_descr_get(PyObject* _self, PyObject* _Nullabl
 #if PY_VERSION_HEX >= 0x03090000
     result->base.sel_vectorcall = objcsel_vectorcall;
 #endif
+
     result->sel_call_func = meth->sel_call_func;
 
     const char* tmp = PyObjCUtil_Strdup(meth->base.sel_python_signature);
@@ -910,15 +835,31 @@ static PyObject* _Nullable objcsel_descr_get(PyObject* _self, PyObject* _Nullabl
 
             /* PyObjCClass_ClassForMetaClass will only return a class proxy for a non-Nil
              * class */
-            meth->sel_call_func = PyObjC_FindCallFunc(
-                (Class _Nonnull)PyObjCClass_GetClass(class_obj), meth->base.sel_selector,
-                meth->base.sel_methinfo->signature);
+            PyObjC_CallFunc func = PyObjC_FindCallFunc(
+                        (Class _Nonnull)PyObjCClass_GetClass(class_obj), meth->base.sel_selector,
+                        meth->base.sel_methinfo->signature);
+
+#ifdef Py_GIL_DISABLED
+                if (meth->sel_call_func == NULL) {
+#endif
+                    meth->sel_call_func = func;
+#ifdef Py_GIL_DISABLED
+                }
+#endif
+
             Py_CLEAR(class_obj);
 
         } else {
-            meth->sel_call_func =
-                PyObjC_FindCallFunc(meth->base.sel_class, meth->base.sel_selector,
+            PyObjC_CallFunc func =  PyObjC_FindCallFunc(meth->base.sel_class, meth->base.sel_selector,
                                     meth->base.sel_methinfo->signature);
+
+#ifdef Py_GIL_DISABLED
+            if (meth->sel_call_func == NULL) {
+#endif
+                meth->sel_call_func = func;
+#ifdef Py_GIL_DISABLED
+            }
+#endif
         }
         if (meth->sel_call_func == NULL) {
             Py_DECREF(result);
@@ -1167,7 +1108,7 @@ PyObjCSelector_NewNative(Class class, SEL selector, const char* signature,
     PyObjCNativeSelector* result;
     const char*           native_signature = signature;
 
-    PyObjC_Assert(signature != NULL, NULL);
+    assert(signature != NULL);
 
     result = PyObject_New(PyObjCNativeSelector, (PyTypeObject*)PyObjCNativeSelector_Type);
     if (result == NULL) // LCOV_BR_EXCL_LINE
@@ -1219,15 +1160,13 @@ PyObjCSelector_NewNative(Class class, SEL selector, const char* signature,
     if (class_method) {
         result->base.sel_flags |= PyObjCSelector_kCLASS_METHOD;
     }
-    if (sel_isEqual(selector, @selector(alloc))
-        || sel_isEqual(selector, @selector(allocWithZone:))) {
-        result->base.sel_flags |= PyObjCSelector_kRETURNS_UNINITIALIZED;
-    }
     result->base.sel_methinfo = NULL;
     PyObjCMethodSignature* methinfo = PyObjCSelector_GetMetadata((PyObject*)result);
-    if (methinfo == NULL) {
+    if (methinfo == NULL) { // LCOV_BR_EXCL_LINE
+        // LCOV_EXCL_START
         Py_DECREF(result);
         return NULL;
+        // LCOV_EXCL_STOP
     } else {
         Py_CLEAR(methinfo);
     }
@@ -1301,9 +1240,11 @@ PyObjCSelector_New(PyObject* callable, SEL selector, const char* _Nullable signa
     result->base.sel_vectorcall = pysel_vectorcall;
 #endif
 
-    if (PyObjC_RemoveInternalTypeCodes((char*)result->base.sel_native_signature) == -1) {
+    if (PyObjC_RemoveInternalTypeCodes((char*)result->base.sel_native_signature) == -1) { // LCOV_BR_EXCL_LINE
+        // LCOV_EXCL_START
         Py_DECREF(result);
         return NULL;
+        // LCOV_EXCL_STOP
     }
 
     if (PyObjCPythonSelector_Check(callable)) {
@@ -1358,10 +1299,6 @@ PyObjCSelector_New(PyObject* callable, SEL selector, const char* _Nullable signa
     if (class_method) {
         result->base.sel_flags |= PyObjCSelector_kCLASS_METHOD;
     }
-    if (sel_isEqual(selector, @selector(alloc))
-        || sel_isEqual(selector, @selector(allocWithZone:))) {
-        result->base.sel_flags |= PyObjCSelector_kRETURNS_UNINITIALIZED;
-    }
 
     result->callable = callable;
     Py_INCREF(result->callable);
@@ -1580,9 +1517,11 @@ static PyObject* _Nullable
              */
             PyObject** temp_args =
                 malloc((PyVectorcall_NARGS(nargsf) + 2) * sizeof(PyObject*));
-            if (temp_args == NULL) {
+            if (temp_args == NULL) { // LCOV_BR_EXCL_LINE
+                // LCOV_EXCL_START
                 PyErr_NoMemory();
                 return NULL;
+                // LCOV_EXCL_STOP
             }
             temp_args[0] = Py_None;
             temp_args[1] = self->base.sel_self;
@@ -1593,18 +1532,10 @@ static PyObject* _Nullable
                                              | PY_VECTORCALL_ARGUMENTS_OFFSET,
                                          kwnames);
             free(temp_args);
-        }
+        } // LCOV_BR_EXCL_LINE
 #endif
     }
 
-    if (result && (self->base.sel_self) && (PyObjCObject_Check(self->base.sel_self))
-        && ((PyObjCObject*)self->base.sel_self)->flags & PyObjCObject_kUNINITIALIZED) {
-
-        Py_BEGIN_CRITICAL_SECTION(self);
-        ((PyObjCObject*)self->base.sel_self)->flags &= ~PyObjCObject_kUNINITIALIZED;
-        Py_END_CRITICAL_SECTION();
-    }
-
     return result;
 }
 
@@ -2043,15 +1974,15 @@ static PyType_Spec pysel_spec = {
     .slots = pysel_slots,
 };
 
-const char* _Nullable PyObjCSelector_Signature(PyObject* obj)
+const char* PyObjCSelector_Signature(PyObject* obj)
 {
-    PyObjC_Assert(PyObjCSelector_Check(obj), NULL);
+    assert(PyObjCSelector_Check(obj));
     return ((PyObjCSelector*)obj)->sel_python_signature;
 }
 
 Class PyObjCSelector_GetClass(PyObject* sel)
 {
-    PyObjC_Assert(PyObjCSelector_Check(sel), (Class _Nonnull)Nil);
+    assert(PyObjCSelector_Check(sel));
 
     return ((PyObjCNativeSelector*)sel)->base.sel_class;
 }
@@ -2059,39 +1990,41 @@ Class PyObjCSelector_GetClass(PyObject* sel)
 SEL
 PyObjCSelector_GetSelector(PyObject* sel)
 {
+    assert(PyObjCSelector_Check(sel));
+
     return ((PyObjCSelector*)sel)->sel_selector;
 }
 
 int
-PyObjCSelector_Required(PyObject* obj)
+PyObjCSelector_Required(PyObject* sel)
 {
-    PyObjC_Assert(PyObjCSelector_Check(obj), -1);
+    assert(PyObjCSelector_Check(sel));
 
-    return (((PyObjCSelector*)obj)->sel_flags & PyObjCSelector_kREQUIRED) != 0;
+    return (((PyObjCSelector*)sel)->sel_flags & PyObjCSelector_kREQUIRED) != 0;
 }
 
 int
-PyObjCSelector_IsClassMethod(PyObject* obj)
+PyObjCSelector_IsClassMethod(PyObject* sel)
 {
-    PyObjC_Assert(PyObjCSelector_Check(obj), -1);
+    assert(PyObjCSelector_Check(sel));
 
-    return (PyObjCSelector_GetFlags(obj) & PyObjCSelector_kCLASS_METHOD) != 0;
+    return (PyObjCSelector_GetFlags(sel) & PyObjCSelector_kCLASS_METHOD) != 0;
 }
 
 int
-PyObjCSelector_IsHidden(PyObject* obj)
+PyObjCSelector_IsHidden(PyObject* sel)
 {
-    PyObjC_Assert(PyObjCSelector_Check(obj), -1);
+    assert(PyObjCSelector_Check(sel));
 
-    return (PyObjCSelector_GetFlags(obj) & PyObjCSelector_kHIDDEN) != 0;
+    return (PyObjCSelector_GetFlags(sel) & PyObjCSelector_kHIDDEN) != 0;
 }
 
 int
-PyObjCSelector_GetFlags(PyObject* obj)
+PyObjCSelector_GetFlags(PyObject* sel)
 {
-    PyObjC_Assert(PyObjCSelector_Check(obj), -1);
+    assert(PyObjCSelector_Check(sel));
 
-    return ((PyObjCSelector*)obj)->sel_flags;
+    return ((PyObjCSelector*)sel)->sel_flags;
 }
 
 int
@@ -2103,8 +2036,8 @@ PyObjCSelector_Setup(PyObject* module)
     }
     PyObjCSelector_Type = tmp;
 
-    if ( // LCOV_BR_EXCL_LINE
-        PyModule_AddObject(module, "selector", PyObjCSelector_Type) == -1) {
+    if (PyModule_AddObject( // LCOV_BR_EXCL_LINE
+            module, "selector", PyObjCSelector_Type) == -1) {
         return -1; // LCOV_EXCL_LINE
     }
     Py_INCREF(PyObjCSelector_Type);
@@ -2132,8 +2065,8 @@ PyObjCSelector_Setup(PyObject* module)
     }
     PyObjCPythonSelector_Type = tmp;
 
-    if ( // LCOV_BR_EXCL_LINE
-        PyModule_AddObject(module, "python_selector", PyObjCPythonSelector_Type) == -1) {
+    if (PyModule_AddObject( // LCOV_BR_EXCL_LINE
+            module, "python_selector", PyObjCPythonSelector_Type) == -1) {
 #if PY_VERSION_HEX <= 0x03090000
         Py_CLEAR(bases);
 #endif
@@ -2153,8 +2086,8 @@ PyObjCSelector_Setup(PyObject* module)
     }
     PyObjCNativeSelector_Type = tmp;
 
-    if ( // LCOV_BR_EXCL_LINE
-        PyModule_AddObject(module, "native_selector", PyObjCNativeSelector_Type) == -1) {
+    if (PyModule_AddObject( // LCOV_BR_EXCL_LINE
+            module, "native_selector", PyObjCNativeSelector_Type) == -1) {
         return -1; // LCOV_EXCL_LINE
     }
     Py_INCREF(PyObjCNativeSelector_Type);
