@@ -17,6 +17,11 @@
 
 NS_ASSUME_NONNULL_BEGIN
 
+/* This variable is initialized int the _Init function, called
+ * before the API using the variable is called.
+ */
+static ffi_cif* _Nonnull init_cif = (ffi_cif* _Nonnull)NULL;
+
 /*
  * First some helpers: easy access to the actual fields
  */
@@ -82,7 +87,7 @@ static PyObject* _Nullable struct_sq_item(PyObject* self, Py_ssize_t offset)
     } else {
         member = Py_TYPE(self)->tp_members + offset;
         res    = GET_STRUCT_FIELD(self, member);
-        PyObjC_Assert(res != NULL, NULL);
+        assert(res != NULL);
 
         Py_INCREF(res);
     }
@@ -97,14 +102,14 @@ static PyObject* _Nullable struct_sq_slice(PyObject* self, Py_ssize_t ilow,
     PyObject*  result;
     Py_ssize_t i;
 
-#ifdef PyObjC_DEBUG
+#ifndef NDEBUG
     Py_ssize_t len = STRUCT_LENGTH(self);
 #endif
-    PyObjC_Assert(ilow >= 0, NULL);
-    PyObjC_Assert(ihigh <= len, NULL);
+    assert(ilow >= 0);
+    assert(ihigh <= len);
 
     result = PyTuple_New(ihigh - ilow);
-    if (result == NULL) {
+    if (result == NULL) { // LCOV_BR_EXCL_LINE
         return NULL; // LCOV_EXCL_LINE
     }
 
@@ -113,10 +118,10 @@ static PyObject* _Nullable struct_sq_slice(PyObject* self, Py_ssize_t ilow,
     for (i = ilow; i < ihigh; i++) {
         PyMemberDef* member = Py_TYPE(self)->tp_members + i;
         PyObject*    v      = GET_STRUCT_FIELD(self, member);
-        PyObjC_Assert(v != NULL, NULL);
+        assert(v != NULL);
         Py_INCREF(v);
         PyTuple_SET_ITEM(result, i - ilow, v);
-    }
+    } // LCOV_BR_EXCL_LINE
 
     Py_END_CRITICAL_SECTION();
     return result;
@@ -128,16 +133,11 @@ struct_sq_ass_item(PyObject* self, Py_ssize_t offset, PyObject* _Nullable newVal
     Py_ssize_t   len;
     PyMemberDef* member;
 
-    if (!PyObjC_StructsIndexable) {
-        PyErr_Format(PyExc_TypeError, "Instances of '%.100s' are not sequences",
-                     Py_TYPE(self)->tp_name);
-        return -1;
-    }
-    if (!PyObjC_StructsWritable) {
-        PyErr_Format(PyExc_TypeError, "Instances of '%.100s' are read-only",
-                     Py_TYPE(self)->tp_name);
-        return -1;
-    }
+    /* It is not necessary to test if sequences are writable and indexable,
+     * both have been checked by our callers.
+     */
+    assert(PyObjC_StructsIndexable);
+    assert(PyObjC_StructsWritable);
 
     if (newVal == NULL) {
         PyErr_Format(PyExc_TypeError,
@@ -173,13 +173,13 @@ struct_sq_ass_slice(PyObject* self, Py_ssize_t ilow, Py_ssize_t ihigh, PyObject*
         return -1;
     }
 
-#ifdef PyObjC_DEBUG
+#ifndef NDEBUG
     Py_ssize_t len = STRUCT_LENGTH(self);
 #endif
-    PyObjC_Assert(ilow >= 0, -1);
-    PyObjC_Assert(ilow <= len, -1);
-    PyObjC_Assert(ihigh >= 0, -1);
-    PyObjC_Assert(ihigh <= len, -1);
+    assert(ilow >= 0);
+    assert(ilow <= len);
+    assert(ihigh >= 0);
+    assert(ihigh <= len);
 
     seq = PyObjCSequence_Tuple(v, "Must assign sequence to slice");
     if (seq == NULL)
@@ -200,9 +200,9 @@ struct_sq_ass_slice(PyObject* self, Py_ssize_t ilow, Py_ssize_t ihigh, PyObject*
         PyMemberDef* member = Py_TYPE(self)->tp_members + i;
 
         x = PyTuple_GET_ITEM(seq, i - ilow);
-        PyObjC_Assert(x != NULL, -1);
+        assert(x != NULL);
         SET_STRUCT_FIELD(self, member, x);
-    }
+    } // LCOV_BR_EXCL_LINE
     Py_DECREF(seq);
     Py_END_CRITICAL_SECTION();
     return 0;
@@ -225,7 +225,7 @@ struct_sq_contains(PyObject* self, PyObject* value)
         int       r;
 
         PyObject* cur = GET_STRUCT_FIELD(self, member);
-        PyObjC_Assert(cur != NULL, -1);
+        assert(cur != NULL);
 
         r = PyObject_RichCompareBool(cur, value, Py_EQ);
         if (r == -1) {
@@ -254,10 +254,10 @@ static PyObject* _Nullable struct_reduce(PyObject* self)
     Py_BEGIN_CRITICAL_SECTION(self);
     for (i = 0; i < len; i++) {
         PyObject* v = GET_STRUCT_FIELD(self, Py_TYPE(self)->tp_members + i);
-        PyObjC_Assert(v != NULL, NULL);
+        assert(v != NULL);
         Py_INCREF(v);
         PyTuple_SET_ITEM(values, i, v);
-    }
+    } // LCOV_BR_EXCL_LINE
     Py_END_CRITICAL_SECTION();
 
     result = PyTuple_Pack(2, Py_TYPE(self), values);
@@ -282,17 +282,17 @@ static PyObject* _Nullable struct__copy__(PyObject* self)
 
     Py_BEGIN_CRITICAL_SECTION(self);
     while (member && member->name) {
-        PyObjC_Assert(member->type == T_OBJECT, NULL);
+        assert(member->type == T_OBJECT);
         *((PyObject**)(((char*)result) + member->offset)) = NULL;
         PyObject* t = GET_STRUCT_FIELD(self, member);
-        PyObjC_Assert(t != NULL, NULL);
+        assert(t != NULL);
 
         if (t != NULL) {
             SET_STRUCT_FIELD(result, member, t);
         }
 
         member++;
-    }
+    } // LCOV_BR_EXCL_LINE
     Py_END_CRITICAL_SECTION();
 
     PyObject_GC_Track(result);
@@ -305,65 +305,46 @@ static PyObject* _Nullable struct__deepcopy__(PyObject* self, PyObject* args, Py
     PyObject*    memo;
     PyObject*    result;
     PyMemberDef* member = Py_TYPE(self)->tp_members;
-    PyObject*    deepcopy;
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "O", keywords, &memo)) {
         return NULL;
     }
 
-    deepcopy= PyObjC_ImportName("copy.deepcopy");
-    if (deepcopy == NULL) { // LCOV_BR_EXCL_LINE
-        return NULL; // LCOV_EXCL_LINE
-    }
-
     result = PyObject_GC_New(PyObject, Py_TYPE(self));
     if (result == NULL) { // LCOV_BR_EXCL_LINE
-        Py_DECREF(deepcopy); // LCOV_EXCL_LINE
         return NULL;      // LCOV_EXCL_LINE
     }
 
-    Py_BEGIN_CRITICAL_SECTION(self);
     while (member && member->name) {
-        PyObjC_Assert(member->type == T_OBJECT, NULL);
+        PyObject* t;
+        assert(member->type == T_OBJECT);
         *((PyObject**)(((char*)result) + member->offset)) = NULL;
-        PyObject* t = GET_STRUCT_FIELD(self, member);
-        PyObjC_Assert(t != NULL, NULL);
 
-        if (t != NULL) {
+        Py_BEGIN_CRITICAL_SECTION(self);
+            t = GET_STRUCT_FIELD(self, member);
+            assert(t != NULL);
             Py_INCREF(t);
-            PyObject* args[3] = {NULL, t, memo};
-            PyObject* v       = PyObject_Vectorcall(
-                    deepcopy, args + 1, 2 | PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
-            if (v == NULL) {
-                Py_DECREF(result);
-                Py_DECREF(deepcopy);
-                Py_EXIT_CRITICAL_SECTION();
-                return NULL;
-            }
-            SET_STRUCT_FIELD(result, member, v);
-            Py_DECREF(v);
+        Py_END_CRITICAL_SECTION();
+
+        PyObject* v = PyObjC_deepcopy(t, memo);
+        Py_DECREF(t);
+        if (v == NULL) {
+            Py_DECREF(result);
+            return NULL;
         }
+        SET_STRUCT_FIELD(result, member, v);
+        Py_DECREF(v);
 
         member++;
     }
-    Py_END_CRITICAL_SECTION();
 
     PyObject_GC_Track(result);
-    Py_DECREF(deepcopy);
     return result;
 }
 
 static PyObject* _Nullable struct_deepcopy(PyObject* self)
 {
-    PyObject* deepcopy= PyObjC_ImportName("copy.deepcopy");
-    if (deepcopy == NULL) { // LCOV_BR_EXCL_LINE
-        return NULL; // LCOV_EXCL_LINE
-    }
-    PyObject* args[2] = {NULL, self};
-    PyObject* result       = PyObject_Vectorcall(
-            deepcopy, args + 1, 1 | PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
-    Py_DECREF(deepcopy);
-    return result;
+    return PyObjC_deepcopy(self, NULL);
 }
 
 
@@ -404,7 +385,7 @@ static PyObject* _Nullable struct_replace_impl(const char* name, int deepcopy, P
                 result = NULL;
                 goto exit;
             }
-        }
+        } // LCOV_BR_EXCL_LINE
 exit:
         (void)0;
         Py_END_CRITICAL_SECTION2();
@@ -439,10 +420,10 @@ static PyObject* _Nullable struct_asdict(PyObject* self)
     Py_BEGIN_CRITICAL_SECTION(self);
     while (member && member->name) {
         PyObject* t;
-        PyObjC_Assert(member->type == T_OBJECT, NULL);
+        assert(member->type == T_OBJECT);
 
         t = GET_STRUCT_FIELD(self, member);
-        PyObjC_Assert(t != NULL, NULL);
+        assert(t != NULL);
 
         PyObject* py_name = PyUnicode_FromString(member->name);
         if (py_name == NULL) { // LCOV_BR_EXCL_LINE
@@ -463,7 +444,7 @@ static PyObject* _Nullable struct_asdict(PyObject* self)
         }
         Py_DECREF(py_name);
         member++;
-    }
+    } // LCOV_BR_EXCL_LINE
 exit:
     (void)0;
     Py_END_CRITICAL_SECTION();
@@ -592,7 +573,7 @@ struct_mp_ass_subscript(PyObject* self, PyObject* item, PyObject* _Nullable valu
                 return -1;
                 // LCOV_EXCL_STOP
             }
-        }
+        } // LCOV_BR_EXCL_LINE
 
         Py_DECREF(seq);
         return 0;
@@ -704,7 +685,7 @@ static PyObject* _Nullable struct_new(PyTypeObject* type, PyObject* args, PyObje
         return NULL;    // LCOV_EXCL_LINE
 
     while (member && member->name) {
-        PyObjC_Assert(member->type == T_OBJECT, NULL);
+        assert(member->type == T_OBJECT);
         *((PyObject**)(((char*)result) + member->offset)) = Py_None;
         Py_INCREF(Py_None);
         member++;
@@ -748,7 +729,7 @@ set_defaults(PyObject* self, const char* typestr)
         /* The encoding cannot have embedded field names,
          * those were removed during type creation
          */
-        PyObjC_Assert(*typestr != '"', -1);
+        assert(*typestr != '"');
         next = PyObjCRT_SkipTypeSpec(typestr);
         if (next == NULL) { // LCOV_BR_EXCL_LINE
             /* Should never happen, the signature was
@@ -834,7 +815,7 @@ set_defaults(PyObject* self, const char* typestr)
             return -1; // LCOV_EXCL_LINE
         }
         typestr = next;
-    }
+    } // LCOV_BR_EXCL_LINE
 
     return 0;
 }
@@ -952,7 +933,6 @@ struct_init(ffi_cif* cif __attribute__((__unused__)), void* retval,
 
 static initproc _Nullable make_init(const char* _typestr)
 {
-    static ffi_cif* init_cif = NULL;
     ffi_closure*    cl       = NULL;
     void*           codeloc;
     const char*     typestr_copy;
@@ -962,21 +942,7 @@ static initproc _Nullable make_init(const char* _typestr)
         return NULL; // LCOV_EXCL_LINE
     }
 
-    if (init_cif == NULL) {
-        PyObjCMethodSignature* signature =
-            PyObjCMethodSignature_WithMetaData("i^v^v^v", NULL, YES);
-        if (signature == NULL) {
-            return NULL; // LCOV_EXCL_LINE
-        }
-        init_cif = PyObjCFFI_CIFForSignature(signature);
-        Py_DECREF(signature);
-        if (init_cif == NULL) {
-            // LCOV_EXCL_START
-            PyMem_Free((void*)typestr_copy);
-            return NULL;
-            // LCOV_EXCL_STOP
-        }
-    }
+    assert(init_cif != NULL);
 
     if (alloc_prepped_closure( // LCOV_BR_EXCL_LINE
             &cl, init_cif, &codeloc, (void*)struct_init, (char*)typestr_copy)
@@ -1019,8 +985,8 @@ static PyObject* _Nullable struct_richcompare(PyObject* self, PyObject* other, i
 
             self_cur  = GET_STRUCT_FIELD(self, Py_TYPE(self)->tp_members + i);
             other_cur = GET_STRUCT_FIELD(other, Py_TYPE(other)->tp_members + i);
-            PyObjC_Assert(self_cur != NULL, NULL);
-            PyObjC_Assert(other_cur != NULL, NULL);
+            assert(self_cur != NULL);
+            assert(other_cur != NULL);
 
             k = PyObject_RichCompareBool(self_cur, other_cur, Py_EQ);
             if (k < 0) {
@@ -1116,7 +1082,7 @@ static PyObject* _Nullable struct_richcompare(PyObject* self, PyObject* other, i
         Py_BEGIN_CRITICAL_SECTION(self);
         self_cur = GET_STRUCT_FIELD(self, Py_TYPE(self)->tp_members + i);
         Py_END_CRITICAL_SECTION();
-        PyObjC_Assert(self_cur != NULL, NULL);
+        assert(self_cur != NULL);
         other_cur = PySequence_GetItem(other, i);
         if (other_cur == NULL) // LCOV_BR_EXCL_LINE
             return NULL;       // LCOV_EXCL_LINE
@@ -1194,8 +1160,8 @@ struct_traverse(PyObject* self, visitproc visit, void* _Nullable arg)
         if (v == NULL) // LCOV_BR_EXCL_LINE
             continue;  // LCOV_EXCL_LINE
         err = visit(v, arg);
-        if (err) {
-            break;
+        if (err) { // LCOV_BR_EXCL_LINE
+            break; // LCOV_EXCL_LINE
         }
     }
     Py_END_CRITICAL_SECTION();
@@ -1255,7 +1221,7 @@ static PyObject* _Nullable struct_repr(PyObject* self)
         Py_BEGIN_CRITICAL_SECTION(self);
         v = GET_STRUCT_FIELD(self, member);
         Py_END_CRITICAL_SECTION();
-        PyObjC_Assert(v != NULL, NULL);
+        assert(v != NULL);
 
         PyObject* repr = PyObject_Repr(v);
         if (repr == NULL) { // LCOV_BR_EXCL_LINE
@@ -1269,7 +1235,7 @@ static PyObject* _Nullable struct_repr(PyObject* self)
         if (cur == NULL) // LCOV_BR_EXCL_LINE
             goto done;   // LCOV_EXCL_LINE
         member++;
-    }
+    } // LCOV_BR_EXCL_LINE
 
     PyUnicode_Append(&cur, PyUnicode_FromString(">"));
 
@@ -1316,7 +1282,7 @@ static struct StructTypeObject StructTemplate_Type = {
 
 PyObject*
 PyObjC_MakeStructType(const char* name, const char* _Nullable doc,
-                      initproc _Nullable tpinit, Py_ssize_t                  numFields,
+                      Py_ssize_t                  numFields,
                       const char* _Nonnull* _Nonnull fieldnames, const char* typestr,
                       Py_ssize_t pack)
 {
@@ -1361,7 +1327,7 @@ PyObjC_MakeStructType(const char* name, const char* _Nullable doc,
         members[i].offset = sizeof(PyObject) + i * sizeof(PyObject*);
         members[i].flags  = 0; /* A read-write field */
         members[i].doc    = NULL;
-    }
+    } // LCOV_BR_EXCL_LINE
     members[numFields].name = NULL;
 
     result = PyMem_Malloc(sizeof(struct StructTypeObject));
@@ -1417,16 +1383,13 @@ PyObjC_MakeStructType(const char* name, const char* _Nullable doc,
 
     Py_CLEAR(fields);
 
-    if (tpinit) {
-        result->base.tp_init = tpinit;
-
-    } else {
-        result->base.tp_init = make_init(typestr);
-        if (result->base.tp_init == NULL) {
-            PyMem_Free(members);
-            PyMem_Free(result);
-            return NULL;
-        }
+    result->base.tp_init = make_init(typestr);
+    if (result->base.tp_init == NULL) { // LCOV_BR_EXCL_LINE
+        // LCOV_EXCL_START
+        PyMem_Free(members);
+        PyMem_Free(result);
+        return NULL;
+        // LCOV_EXCL_STOP
     }
 
     result->pack = pack;
@@ -1449,38 +1412,36 @@ PyObjC_MakeStructType(const char* name, const char* _Nullable doc,
  * This is the start of PyObjC specific code
  */
 
+/* The registry is initialized in PyObjCStruct_Init, which is
+ * called before any function in this file is called.
+ *
+ * The lock is needed to get consistent updates to the registry
+ * dict.
+ */
+
 #ifdef Py_GIL_DISABLED
 PyMutex registry_mutex = { 0 };
 #endif
-static PyObject* structRegistry = NULL;
+static PyObject* _Nonnull structRegistry = (PyObject* _Nonnull)NULL;
 
 PyObject* _Nullable PyObjC_FindRegisteredStruct(const char* signature, Py_ssize_t len)
 {
-    PyObject* type = NULL;
+    PyObject* type;
     PyObject* v;
 
-    if (structRegistry == NULL) { // LCOV_BR_EXCL_LINE
-        return NULL;              // LCOV_EXCL_LINE
-    }
+    assert(structRegistry != NULL);
 
-#ifdef Py_GIL_DISABLED
-    PyMutex_Lock(&registry_mutex);
-#endif
     v = PyUnicode_FromStringAndSize(signature, len);
     if (v == NULL) { // LCOV_BR_EXCL_LINE
-        goto exit; // LCOV_EXCL_LINE
+        return NULL; // LCOV_EXCL_LINE
     }
 
     if (PyDict_GetItemRef(structRegistry, v, &type) != 1) {
         Py_DECREF(v);
-        goto exit;
+        return NULL;
     }
     Py_DECREF(v);
 
-exit:
-#ifdef Py_GIL_DISABLED
-    PyMutex_Unlock(&registry_mutex);
-#endif
     return type;
 }
 
@@ -1494,8 +1455,7 @@ PyObject* _Nullable PyObjC_CreateRegisteredStruct(
     PyMemberDef*  member;
     int r;
 
-    if (structRegistry == NULL)
-        return NULL;
+    assert(structRegistry != NULL);
 
     if (ppack != NULL) {
         *ppack = -1;
@@ -1503,13 +1463,7 @@ PyObject* _Nullable PyObjC_CreateRegisteredStruct(
 
     v = PyUnicode_FromStringAndSize(signature, len);
 
-#ifdef Py_GIL_DISABLED
-    PyMutex_Lock(&registry_mutex);
-#endif
     r = PyDict_GetItemRef(structRegistry, v, (PyObject**)&type);
-#ifdef Py_GIL_DISABLED
-    PyMutex_Unlock(&registry_mutex);
-#endif
 
     if (r != 1) {
         Py_DECREF(v);
@@ -1529,7 +1483,7 @@ PyObject* _Nullable PyObjC_CreateRegisteredStruct(
     }
 
     while (member && member->name) {
-        PyObjC_Assert(member->type == T_OBJECT, NULL);
+        assert(member->type == T_OBJECT);
         *((PyObject**)(((char*)result) + member->offset)) = Py_None;
         Py_INCREF(Py_None);
         member++;
@@ -1538,9 +1492,19 @@ PyObject* _Nullable PyObjC_CreateRegisteredStruct(
     PyObject_GC_Track(result);
 
     if (objc_encoding) {
+        /*
+         * The structRegistry only contains struct wrapper types
+         * which by construction always have a bytes __typestr__
+         * attribute.
+         *
+         * The code below guards against getting invalid values
+         * in case someone uses Python's introspection facilities
+         * to find the registry dict and add an invalid value.
+         */
         PyObject* typestr;
 
-        switch(PyDict_GetItemRef(type->tp_dict, PyObjCNM___typestr__, &typestr)) {
+        switch( // LCOV_BR_EXCL_LINE
+                PyDict_GetItemRef(type->tp_dict, PyObjCNM___typestr__, &typestr)) {
         case -1:
             // LCOV_EXCL_START
             Py_DECREF(type);
@@ -1548,25 +1512,33 @@ PyObject* _Nullable PyObjC_CreateRegisteredStruct(
             return NULL;
             // LCOV_EXCL_STOP
         case 0:
-            /* XXX: Can this ever happen? */
+            // LCOV_EXCL_START
             if (objc_encoding != NULL) {
                 *objc_encoding = signature;
             }
             break;
+            // LCOV_EXCL_STOP
 
         case 1:
-            if (!PyBytes_Check(typestr)) {
+            if (!PyBytes_Check(typestr)) { // LCOV_BR_EXCL_LINE
+                // LCOV_EXCL_START
                 PyErr_SetString(PyExc_TypeError, "__typestr__ not a bytes object");
                 Py_DECREF(type);
                 Py_DECREF(result);
                 return NULL;
+                // LCOV_EXCL_STOP
             }
-            // XXX: this effectively returns a borrowed reference.
-            if (objc_encoding != NULL) {
-                *objc_encoding = PyBytes_AsString(typestr);
-            }
+
+            /* Setting ``*objc_encoding`` effectively returns
+             * a borrowed reference.
+             *
+             * XXX: This might be unsafe when the struct registry
+             * is updated concurrently with a new type.
+             */
+            *objc_encoding = PyBytes_AsString(typestr);
+            Py_DECREF(typestr);
         }
-    }
+    } // LCOV_BR_EXCL_LINE
 
     if (ppack != NULL) {
         *ppack = ((struct StructTypeObject*)type)->pack;
@@ -1578,7 +1550,6 @@ PyObject* _Nullable PyObjC_CreateRegisteredStruct(
 
 PyObject* _Nullable PyObjC_RegisterStructType(const char* signature, const char* name,
                                               const char* _Nullable doc,
-                                              initproc _Nullable tpinit,
                                               Py_ssize_t numFields,
                                               const char* _Nonnull* _Nullable fieldnames,
                                               Py_ssize_t pack)
@@ -1638,8 +1609,6 @@ PyObject* _Nullable PyObjC_RegisterStructType(const char* signature, const char*
                 return NULL;
             }
 
-            if (*sigcur == _C_STRUCT_E)
-                break;
             sigcur = PyObjCRT_NextField(sigcur);
             if (sigcur == NULL) {
                 return NULL;
@@ -1657,16 +1626,16 @@ PyObject* _Nullable PyObjC_RegisterStructType(const char* signature, const char*
 
                 sigcur++;
                 end = strchr(sigcur, '"');
-                PyObjC_Assert(end != NULL, NULL);
+                assert(end != NULL);
 
                 fieldnames[numFields] = PyMem_Malloc(end - sigcur + 1);
                 memcpy((char*)fieldnames[numFields], sigcur, end - sigcur);
                 ((char*)fieldnames[numFields])[end - sigcur] = '\0';
                 sigcur                                       = end + 1;
-            }
+            } // LCOV_BR_EXCL_LINE
             numFields++;
             sigcur = PyObjCRT_NextField(sigcur);
-            PyObjC_Assert(sigcur != NULL, NULL);
+            assert(sigcur != NULL);
         }
         fieldnames[numFields] = NULL;
         freeNames             = 1;
@@ -1693,11 +1662,11 @@ PyObject* _Nullable PyObjC_RegisterStructType(const char* signature, const char*
         }
         signature = sigtmp;
     } else {
-        PyObjC_Assert(fieldnames, NULL);
+        assert(fieldnames);
     }
 
     structType =
-        PyObjC_MakeStructType(name, doc, tpinit, numFields, fieldnames, signature, pack);
+        PyObjC_MakeStructType(name, doc, numFields, fieldnames, signature, pack);
     if (structType == NULL) {
         if (freeNames) { // LCOV_BR_EXCL_LINE
             /* This should never happen unless the system
@@ -1714,7 +1683,7 @@ PyObject* _Nullable PyObjC_RegisterStructType(const char* signature, const char*
             }
             PyMem_Free(fieldnames);
             // LCOV_EXCL_STOP
-        }
+        } // LCOV_EXCL_LINE
         return NULL;
     }
 
@@ -1760,25 +1729,15 @@ PyObject* _Nullable PyObjC_RegisterStructType(const char* signature, const char*
 #ifdef Py_GIL_DISABLED
     PyMutex_Lock(&registry_mutex);
 #endif
-    if (structRegistry == NULL) {
-        structRegistry = PyDict_New();
-        if (structRegistry == NULL) { // LCOV_BR_EXCL_LINE
-            /* This leaks some memory, but we cannot safely
-             * deallocate the type
-             *
-             * XXX: Why is that?
-             */
-            return NULL; // LCOV_EXCL_LINE
-        }
-    }
-
     PyObject* py_signature = PyUnicode_FromString(signature);
     if (py_signature == NULL) { // LCOV_BR_EXCL_LINE
         /* This leaks some memory, but we cannot safely
          * deallocate the type
          */
-        structType = NULL; // LCOV_EXCL_LINE
+        // LCOV_EXCL_START
+        structType = NULL;
         goto exit;
+        // LCOV_EXCL_STOP
     }
 
     r = PyDict_SetItem(structRegistry, py_signature, structType);
@@ -1786,9 +1745,11 @@ PyObject* _Nullable PyObjC_RegisterStructType(const char* signature, const char*
         /* This leaks some memory, but we cannot safely
          * deallocate the type
          */
+        // LCOV_EXCL_START
         Py_DECREF(py_signature);
-        structType = NULL; // LCOV_EXCL_LINE
+        structType = NULL;
         goto exit;
+        // LCOV_EXCL_STOP
     }
     Py_DECREF(py_signature);
 
@@ -1797,8 +1758,10 @@ PyObject* _Nullable PyObjC_RegisterStructType(const char* signature, const char*
         /* We've validated the type signature earlier, the
          * call should never fail.
          */
-        structType = NULL; // LCOV_EXCL_LINE
+        // LCOV_EXCL_START
+        structType = NULL;
         goto exit;
+        // LCOV_EXCL_STOP
     }
 
     py_signature = PyUnicode_FromString(signature);
@@ -1812,8 +1775,10 @@ PyObject* _Nullable PyObjC_RegisterStructType(const char* signature, const char*
 
     r = PyDict_SetItem(structRegistry, py_signature, structType);
     if (r == -1) {   // LCOV_BR_EXCL_LINE
+        // LCOV_EXCL_START
         Py_DECREF(py_signature);
-        structType = NULL; // LCOV_EXCL_LINE
+        structType = NULL;
+        // LCOV_EXCL_STOP
         goto exit; // LCOV_EXCL_LINE
     }
     Py_DECREF(py_signature);
@@ -1841,7 +1806,7 @@ PyObjC_RegisterStructAlias(const char* signature, PyObject* structType)
         return -1;
     }
 
-    if (!PyObject_HasAttrString(structType, "__typestr__")) {
+    if (!PyObject_HasAttr(structType, PyObjCNM___typestr__)) {
         PyErr_SetString(PyExc_TypeError, "struct type is not valid");
         return -1;
     }
@@ -1849,19 +1814,11 @@ PyObjC_RegisterStructAlias(const char* signature, PyObject* structType)
      * compatible encoding (some number of fields, compatible types)
      */
 
+    assert(structRegistry != NULL);
+
 #ifdef Py_GIL_DISABLED
     PyMutex_Lock(&registry_mutex);
 #endif
-    if (structRegistry == NULL) { // LCOV_BR_EXCL_LINE
-        // LCOV_EXCL_START
-        structRegistry = PyDict_New();
-        if (structRegistry == NULL) { // LCOV_BR_EXCL_LINE
-            retval = -1;                // LCOV_EXCL_LINE
-            goto exit;                // LCOV_EXCL_LINE
-        }
-        // LCOV_EXCL_STOP
-    }
-
     PyObject* py_buf = PyUnicode_FromString(buf);
     if (py_buf == NULL) { // LCOV_BR_EXCL_LINE
         retval = -1;                // LCOV_EXCL_LINE
@@ -1909,7 +1866,7 @@ PyObjC_SetStructField(PyObject* self, Py_ssize_t offset, PyObject* newVal)
     Py_ssize_t   len;
     PyMemberDef* member;
 
-    PyObjC_Assert(newVal != NULL, -1);
+    assert(newVal != NULL);
 
     len = STRUCT_LENGTH(self);
 
@@ -1932,7 +1889,7 @@ PyObject* _Nullable StructAsTuple(PyObject* strval)
 {
     Py_ssize_t i, len = STRUCT_LENGTH(strval);
     PyObject*  retval = PyTuple_New(len);
-    if (retval == NULL) {
+    if (retval == NULL) { // LCOV_BR_EXCL_LINE
         return 0; // LCOV_EXCL_LINE
     }
 
@@ -1940,12 +1897,44 @@ PyObject* _Nullable StructAsTuple(PyObject* strval)
     for (i = 0; i < len; i++) {
         PyObject* v;
         v = GET_STRUCT_FIELD(strval, Py_TYPE(strval)->tp_members + i);
-        PyObjC_Assert(v != NULL, NULL);
+        assert(v != NULL);
         PyTuple_SET_ITEM(retval, i, v);
         Py_INCREF(v);
-    }
+    } // LCOV_BR_EXCL_LINE
     Py_END_CRITICAL_SECTION();
     return retval;
 }
 
+int PyObjCStruct_Init(PyObject* module __attribute__((__unused__)))
+{
+    structRegistry = PyDict_New();
+    if (structRegistry == NULL) { // LCOV_BR_EXCL_LINE
+        return -1; // LCOV_EXCL_LINE
+    }
+
+    PyObjCMethodSignature* signature =
+        PyObjCMethodSignature_WithMetaData("i^v^v^v", NULL, YES);
+    if (signature == NULL) { // LCOV_BR_EXCL_LINE
+        // LCOV_EXCL_START
+        Py_DECREF(structRegistry);
+        structRegistry = (PyObject* _Nonnull)NULL;
+        return -1;
+        // LCOV_EXCL_STOP
+    }
+
+    ffi_cif* temp_cif = PyObjCFFI_CIFForSignature(signature);
+    Py_CLEAR(signature);
+    if (temp_cif == NULL) { // LCOV_BR_EXCL_LINE
+        // LCOV_EXCL_START
+        Py_DECREF(structRegistry);
+        structRegistry = (PyObject* _Nonnull)NULL;
+        return -1;
+        // LCOV_EXCL_STOP
+    }
+    init_cif = temp_cif;
+
+    return 0;
+}
+
+
 NS_ASSUME_NONNULL_END
