@@ -37,8 +37,8 @@ static PyObject* special_registry = (PyObject* _Nonnull)NULL;
 int
 PyObjC_InitSuperCallRegistry(void)
 {
-    PyObjC_Assert(signature_registry == NULL, -1);
-    PyObjC_Assert(special_registry == NULL, -1);
+    assert(signature_registry == NULL);
+    assert(special_registry == NULL);
 
     signature_registry = PyDict_New();
     if (signature_registry == NULL) // LCOV_BR_EXCL_LINE
@@ -51,6 +51,12 @@ PyObjC_InitSuperCallRegistry(void)
     return 0;
 }
 
+
+// LCOV_EXCL_START
+/* The code in this file only adds new values to
+ * the registry, but never clears them. Because of
+ * this the capsules are never released.
+ */
 static void
 memblock_capsule_cleanup(PyObject* ptr)
 {
@@ -63,6 +69,7 @@ memblock_capsule_cleanup(PyObject* ptr)
 
     PyMem_Free(mem);
 }
+// LCOV_EXCL_STOP
 
 /*
  * Add a custom mapping for a method in a class
@@ -79,7 +86,9 @@ PyObjC_RegisterMethodMapping(_Nullable Class class, SEL sel, PyObjC_CallFunc cal
     int r;
     int retval =  0;
 
-    PyObjC_Assert(special_registry != NULL, -1);
+    assert(special_registry != NULL);
+    assert(call_to_objc !=  NULL);
+    assert(make_call_to_python_block !=  NULL);
 
 #ifdef Py_GIL_DISABLED
     PyMutex_Lock(&registry_mutex);
@@ -96,10 +105,6 @@ PyObjC_RegisterMethodMapping(_Nullable Class class, SEL sel, PyObjC_CallFunc cal
     }
 #endif
 
-    if (!call_to_objc) {
-        call_to_objc = PyObjCFFI_Caller;
-    }
-
     py_selname = PyUnicode_FromString(sel_getName(sel));
     if (py_selname == NULL) { // LCOV_BR_EXCL_LINE
         // LCOV_EXCL_START
@@ -225,7 +230,7 @@ PyObjC_RegisterSignatureMapping(char* signature, PyObjC_CallFunc call_to_objc,
     int              r;
     int              retval = 0;
 
-    PyObjC_Assert(signature_registry != NULL, -1);
+    assert(signature_registry != NULL);
 
 #ifdef Py_GIL_DISABLED
     PyMutex_Lock(&registry_mutex);
@@ -241,11 +246,13 @@ PyObjC_RegisterSignatureMapping(char* signature, PyObjC_CallFunc call_to_objc,
 
     r = PyObjCRT_SimplifySignature(signature, PyBytes_AS_STRING(key),
                                    PyBytes_GET_SIZE(key));
-    if (r == -1) {
+    if (r == -1) { // LCOV_BR_EXCL_LINE
+        // LCOV_EXCL_START
         Py_DECREF(key);
         PyErr_Format(PyObjCExc_Error, "cannot simplify signature '%s'", signature);
         retval = -1;
         goto exit;
+        // LCOV_EXCL_STOP
     }
 
 #ifdef PyObjC_DEBUG
@@ -322,8 +329,8 @@ static struct registry* _Nullable search_special(Class class, SEL sel)
     Py_ssize_t i;
     int r;
 
-    PyObjC_Assert(special_registry != NULL, NULL);
-    PyObjC_Assert(class != Nil, NULL);
+    assert(special_registry != NULL);
+    assert(class != Nil);
 #ifdef Py_GIL_DISABLED
     PyMutex_Lock(&registry_mutex);
 #endif
@@ -434,7 +441,7 @@ static struct registry* _Nullable find_signature(const char* signature)
     int       res;
     struct registry* result = NULL;
 
-    PyObjC_Assert(signature_registry != NULL, NULL);
+    assert(signature_registry != NULL);
 
 #ifdef Py_GIL_DISABLED
     PyMutex_Lock(&registry_mutex);
@@ -446,10 +453,12 @@ static struct registry* _Nullable find_signature(const char* signature)
 
     res = PyObjCRT_SimplifySignature(signature, PyBytes_AS_STRING(key),
                                      PyBytes_GET_SIZE(key));
-    if (res == -1) {
+    if (res == -1) { // LCOV_BR_EXCL_LINE
+        // LCOV_EXCL_START
         Py_DECREF(key);
         PyErr_Format(PyObjCExc_Error, "cannot simplify signature '%s'", signature);
         goto exit;
+        // LCOV_EXCL_STOP
     }
 
     int r = _PyBytes_Resize(&key, strlen(PyBytes_AS_STRING(key)) + 1);
@@ -475,7 +484,7 @@ PyObjC_CallFunc _Nullable PyObjC_FindCallFunc(Class class, SEL sel, const char*
     struct registry* special;
     PyObjC_CallFunc result;
 
-    PyObjC_Assert(special_registry != NULL, NULL);
+    assert(special_registry != NULL);
 
     special = search_special(class, sel);
     if (special) {
@@ -488,7 +497,7 @@ PyObjC_CallFunc _Nullable PyObjC_FindCallFunc(Class class, SEL sel, const char*
             result = special->call_to_objc;
         } else if (PyErr_Occurred()) { // LCOV_BR_EXCL_LINE
             result = NULL;               // LCOV_EXCL_LINE
-        } else {
+        } else { // LCOV_EXCL_LINE
             result = PyObjCFFI_Caller;
         }
     }
@@ -514,8 +523,8 @@ PyObjC_MakeIMP(Class class __attribute__((__unused__)), Class _Nullable super_cl
     PyObjCMethodSignature*  methinfo;
 
     methinfo = PyObjCSelector_GetMetadata(sel);
-    if (methinfo == NULL) {
-        return NULL;
+    if (methinfo == NULL) { // LCOV_BR_EXCL_LINE
+        return NULL; // LCOV_EXCL_LINE
     }
 
     if (super_class != nil) {
