@@ -82,6 +82,7 @@ static void use_id(id x __attribute__((__unused__))){}
 
 /* Helpers for calling back into python: */
 
++ (NSArray* _Nullable)makeIntArray:(int*)array sameSizeAs:(NSArray**)value on:(OC_MetaDataTest*)obj;;
 + (int*)makeIntArrayOf5On:(OC_MetaDataTest*)obj;
 + (char**)makeStringArrayOn:(OC_MetaDataTest*)obj;
 + (int*)makeIntArrayOf:(int)count on:(OC_MetaDataTest*)obj;
@@ -302,6 +303,20 @@ static void use_id(id x __attribute__((__unused__))){}
     return array;
 }
 
+- (NSArray* _Nullable)makeIntArray:(int*)data sameSizeAs:(NSArray**)cnt
+{
+    NSUInteger count = cnt?(*cnt?[*cnt count]:0):0;
+    NSMutableArray* array;
+    unsigned        i;
+
+    array = [NSMutableArray arrayWithCapacity:count];
+
+    for (i = 0; i < count; i++) {
+        [array addObject:[NSNumber numberWithInt:data[i]]];
+    }
+    return array;
+}
+
 - (NSArray*)make4Tuple:(double*)data
 {
     NSMutableArray* array;
@@ -657,6 +672,17 @@ static void use_id(id x __attribute__((__unused__))){}
     return [obj makeIntArray:array sameSize:cnt];
 }
 
++ (NSArray* _Nullable)makeIntArray:(int*)array sameSizeAs:(NSArray**)cnt on:(OC_MetaDataTest*)obj
+{
+    return [obj makeIntArray:array sameSizeAs:cnt];
+}
+
++ (NSArray* _Nullable)makeIntArray:(int*)array sameSizeAsNilOn:(OC_MetaDataTest*)obj
+{
+    NSArray* cnt = nil;
+    return [obj makeIntArray:array sameSizeAs:&cnt];
+}
+
 + (NSArray*)makeIntArray:(int*)data countPtr:(unsigned*)countPtr on:(OC_MetaDataTest*)obj
 {
     return [obj makeIntArray:data countPtr:countPtr];
@@ -1443,15 +1469,16 @@ static PyMethodDef mod_methods[] = {{0, 0, 0, 0}};
 
 static int mod_exec_module(PyObject* m)
 {
-    if (PyObjC_ImportAPI(m) < 0) {
-        return -1;
+    if (PyObjC_ImportAPI(m) < 0) { // LCOV_BR_EXCL_LINE
+        return -1; // LCOV_EXCL_LINE
     }
 
-    if (PyModule_AddObject(m, "OC_MetaDataTest",
+    if (PyModule_AddObject(m, // LCOV_BR_EXCL_LINE
+                "OC_MetaDataTest",
                            PyObjC_IdToPython([OC_MetaDataTest class]))
         < 0) {
 
-        return -1;
+        return -1; // LCOV_EXCL_LINE
     }
     return 0;
 }
