@@ -577,8 +577,8 @@ static PyMethodDef mod_methods[] = {{0, 0, 0, 0}};
 
 static int mod_exec_module(PyObject* m)
 {
-    if (PyObjC_ImportAPI(m) < 0) {
-        return -1;
+    if (PyObjC_ImportAPI(m) < 0) { // LCOV_BR_EXCL_LINE
+        return -1; // LCOV_EXCL_LINE
     }
 
     PyObject* v = PyCapsule_New(gFunctionMap, "objc.__inline__", NULL);
@@ -586,12 +586,13 @@ static int mod_exec_module(PyObject* m)
         return -1;
     }
 
-    if (PyModule_AddObject(m, "function_list", v) == -1) {
-        return -1;
+    if (PyModule_AddObject(m, // LCOV_BR_EXCL_LINE
+                "function_list", v) == -1) {
+        return -1; // LCOV_EXCL_LINE
     }
     if (PyModule_AddStringConstant(m, "union_SomeUnion", @encode(union SomeUnion))
         == -1) {
-        return -1;
+        return -1; // LCOV_EXCL_LINE
     }
     return 0;
 }
