@@ -28,6 +28,29 @@ NSObject (TestMethods)
 - (id)selectorWithArg:(id)arg1 andArg:(id)arg2;
 @end
 
+@interface OC_NoPythonRepresentation : NSObject <NSCopying, NSCoding>
+/*
+ * Helper class that can be used to test error paths
+ * in creating a proxy for an ObjC object.
+ */
+{
+    bool _allowPython;
+}
+- (instancetype _Nullable)initAllowPython:(bool)allowPython;
+
+
+@end
+
+@interface CallHelper : NSObject
+{}
+@end
+
+@implementation CallHelper
+-(void)voidSelector
+{
+}
+@end
+
 @interface OC_ObjectInt : NSObject {
 }
 @end
@@ -120,6 +143,25 @@ NSObject (TestMethods)
     return [first compare:second];
 }
 
++ (bool)objectEqualToNoProxy:(NSObject*)object
+{
+    NSObject* value = [[OC_NoPythonRepresentation alloc] initAllowPython:NO];
+
+    bool result = [object isEqual:value];
+    [value release];
+    return result;
+}
+
++ (NSComparisonResult)objectCompareToNoProxy:(NSObject*)object
+{
+    NSObject* value = [[OC_NoPythonRepresentation alloc] initAllowPython:NO];
+
+    NSComparisonResult result = [object compare:value];
+    [value release];
+    return result;
+}
+
+
 #pragma clang diagnostic push
 #pragma clang diagnostic ignored "-Wdeprecated-declarations"
 
@@ -313,6 +355,17 @@ NSObject (TestMethods)
     return [object selectorWithArg:arg1 andArg:arg2];
 }
 
++ (id)selectorWithArgAndArgOf:(NSObject*)object
+{
+    NSObject* value = [[OC_NoPythonRepresentation alloc] initAllowPython:NO];
+
+    NSObject* result =  [object selectorWithArg:value andArg:value];
+
+    [value release];
+
+    return result;
+}
+
 + (Class)invokeClassForCoderOf:(NSObject*)object
 {
     Class              result;
@@ -572,6 +625,21 @@ NSObject (TestMethods)
     [object forwardInvocation:inv];
 }
 
++(void)invokeVoidSelectorOf:(NSObject*) object
+{
+    CallHelper* helper = [[CallHelper alloc] init];
+    NSMethodSignature* signature =
+        [helper methodSignatureForSelector:@selector(voidSelector)];
+    [helper dealloc];
+
+
+    NSInvocation* inv = [NSInvocation invocationWithMethodSignature:signature];
+    inv.target   = object;
+    inv.selector = @selector(voidSelector);
+    [object forwardInvocation:inv];
+}
+
+
 + (NSObject*)invokeMethodSignatureForSelector:(SEL)selector of:(NSObject*)object
 {
     NSObject*          result;
@@ -643,19 +711,37 @@ NSObject (TestMethods)
     return [[object class] accessInstanceVariablesDirectly];
 }
 
++(void)addObserver:(NSObject*)observer
+        forKeyPath:(NSString*)keyPath
+            options:(NSKeyValueObservingOptions)options
+        context:(void*)context
+             on:(NSObject*)value
+{
+    [value addObserver:observer forKeyPath:keyPath options:options context:context];
+}
+
++ (void)removeObserver:(NSObject*)observer forKeyPath:(NSString*)keyPath on:(NSObject*)value
+{
+    [value removeObserver:observer forKeyPath:keyPath];
+}
+
 @end
 
-@interface OC_NoPythonRepresentation : NSObject <NSCopying>
-/*
- * Helper class that can be used to test error paths
- * in creating a proxy for an ObjC object.
- */
-{
-    bool _allowPython;
-}
-@end
 
 @implementation OC_NoPythonRepresentation
+-(void)encodeWithCoder:(NSCoder*)coder
+{
+}
+
+-(instancetype)initWithCoder:(NSCoder*)coder
+{
+    self = [super init];
+    if (!self)
+        return nil;
+
+    self->_allowPython = false;
+    return self;
+}
 - (instancetype _Nullable)init
 {
     self = [super init];
@@ -709,19 +795,21 @@ static PyMethodDef mod_methods[] = {{0, 0, 0, 0}};
 
 static int mod_exec_module(PyObject* m)
 {
-    if (PyObjC_ImportAPI(m) < 0) {
-        return -1;
+    if (PyObjC_ImportAPI(m) < 0) { // LCOV_BR_EXCL_LINE
+        return -1; // LCOV_EXCL_LINE
     }
 
-    if (PyModule_AddObject(m, "OC_ObjectInt", PyObjC_IdToPython([OC_ObjectInt class]))
+    if (PyModule_AddObject(m, // LCOV_BR_EXCL_LINE
+                "OC_ObjectInt", PyObjC_IdToPython([OC_ObjectInt class]))
         < 0) {
-        return -1;
+        return -1; // LCOV_EXCL_LINE
     }
 
-    if (PyModule_AddObject(m, "OC_NoPythonRepresentation",
+    if (PyModule_AddObject(m, // LCOV_BR_EXCL_LINE
+                "OC_NoPythonRepresentation",
                            PyObjC_IdToPython([OC_NoPythonRepresentation class]))
         < 0) {
-        return -1;
+        return -1; // LCOV_EXCL_LINE
     }
     return 0;
 }
