@@ -83,20 +83,22 @@ static PyMethodDef mod_methods[] = {{0, 0, 0, 0}};
 
 static int mod_exec_module(PyObject* m)
 {
-    if (PyObjC_ImportAPI(m) < 0) {
-        return -1;
+    if (PyObjC_ImportAPI(m) < 0) { // LCOV_BR_EXCL_LINE
+        return -1; // LCOV_EXCL_LINE
     }
 
-    if (PyModule_AddObject(m, "OC_TestSequence",
+    if (PyModule_AddObject(m, // LCOV_BR_EXCL_LINE
+                "OC_TestSequence",
                            PyObjC_IdToPython([OC_TestSequence class]))
         < 0) {
-        return -1;
+        return -1; // LCOV_EXCL_LINE
     }
 
-    if (PyModule_AddObject(m, "OC_TestMutableSequence",
+    if (PyModule_AddObject(m, // LCOV_BR_EXCL_LINE
+                "OC_TestMutableSequence",
                            PyObjC_IdToPython([OC_TestMutableSequence class]))
         < 0) {
-        return -1;
+        return -1; // LCOV_EXCL_LINE
     }
     return 0;
 }
