@@ -4,6 +4,7 @@
 
 import objc
 from PyObjCTest.block import OCTestBlock
+from PyObjCTest.block2 import OCTestBlock2
 from PyObjCTools.TestSupport import TestCase, min_os_level
 from .fnd import NSMutableArray, NSException
 from .test_metadata import NoObjCClass
@@ -973,6 +974,9 @@ class TestBlocks(TestCase):
             ),
         )
 
+        with self.assertRaisesRegex(ValueError, "Not a block"):
+            objc._block_signature(42)
+
     @min_os_level("10.6")
     def testBlockArgumentToPython(self):
         obj = OCTestBlock.alloc().init()
@@ -1071,6 +1075,11 @@ class TestInvalidCalling(TestCase):
 
         self.assertIs(obj.__block_signature__, None)
 
+        with self.assertRaisesRegex(
+            TypeError, "'__block_signature__' can only be set on Block objects"
+        ):
+            obj.__block_signature__ = 42
+
         with self.assertRaisesRegex(TypeError, "not a block"):
             mod._block_call(42, block.__block_signature__, (), {})
 
@@ -1190,3 +1199,25 @@ class TestInvalidCalling(TestCase):
 
             with self.assertRaisesRegex(ValueError, "bla bla"):
                 obj.callOptionalBlock_withValue_(callback, None)
+
+    def test_without_signature(self):
+        block = OCTestBlock2().getObjectBlock()
+        self.assertIs(block.__block_signature__, None)
+
+        with self.assertRaisesRegex(TypeError, "cannot call block without a signature"):
+            block(1, 2)
+
+        obj = OCTestBlock.alloc().init()
+        block2 = obj.getIntBlock()
+        self.assertIs(block.__block_signature__, None)
+
+        with self.assertRaisesRegex(TypeError, "New value must be a method signature"):
+            block.__block_signature__ = 42
+
+        block.__block_signature__ = block2.__block_signature__
+        self.assertIs(block.__block_signature__, block2.__block_signature__)
+        with self.assertRaisesRegex(TypeError, "Cannot delete '__block_signature__'"):
+            del block.__block_signature__
+
+        with self.assertRaisesRegex(TypeError, "New value must be a method signature"):
+            block.__block_signature__ = None
