@@ -41,6 +41,12 @@ class ReturnAStruct(NSObject):
 
 
 class TestRegressions(TestCase):
+    def test_sizeof_objc_object(self):
+        array = NSArray.array()
+        size = sys.getsizeof(array)
+        self.assertIsInstance(size, int)
+        self.assertGreater(size, sys.getsizeof(object()))
+
     def testSetCompare(self):
         oc = objc.lookUpClass("NSSet").setWithArray_([None])
         oc2 = objc.lookUpClass("NSMutableSet").setWithArray_([None])
@@ -88,8 +94,7 @@ class TestRegressions(TestCase):
             # coverage.py
             gc.collect()
 
-        self.assertTrue(len(w) == 1)
-        self.assertEqual(w[0].category, objc.UninitializedDeallocWarning)
+        self.assertTrue(len(w) == 0)
 
     def testOneWayMethods(self):
         # This one should be in test_methods*.py
@@ -716,10 +721,7 @@ class TestMagic(TestCase):
         o = NSArray.alloc()
         o.init()
 
-        with self.assertRaisesRegex(
-            AttributeError, "cannot access attribute '__pyobjc_magic_coookie__' of NIL"
-        ):
-            o.__pyobjc_magic_coookie__()
+        self.assertFalse(o.__pyobjc_magic_coookie__())
 
     def test_magic_normal(self):
         o = NSArray.alloc().init()
@@ -735,10 +737,7 @@ class TestISA(TestCase):
 
         o = NSArray.alloc()
         o.init()
-        with self.assertRaisesRegex(
-            AttributeError, "cannot access attribute 'pyobjc_ISA' of NIL"
-        ):
-            o.pyobjc_ISA
+        o.pyobjc_ISA
 
 
 class DeallocRevives(NSObject):
@@ -767,4 +766,24 @@ class TestDelRevives(TestCase):
             captured_stderr.getvalue(),
         )
 
-        self.assertEqual(repr(VALUE), "<DeallocRevives objective-c instance 0x0>")
+        self.assertEqual(repr(VALUE), "<null>")
+
+
+class TestConvertNegativeToUnsigedWarns(TestCase):
+    def test_repythonify_negative_int(self):
+        class Number:
+            def __int__(self):
+                return -28
+
+        with warnings.catch_warnings():
+            warnings.filterwarnings("error", category=DeprecationWarning)
+
+            with self.assertRaisesRegex(
+                DeprecationWarning, "converting negative value to unsigned integer"
+            ):
+                objc.repythonify(-40, b"I")
+
+            with self.assertRaisesRegex(
+                DeprecationWarning, "converting negative value to unsigned integer"
+            ):
+                objc.repythonify(Number(), b"I")
